#include <unistd.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>
#include <ctype.h>
//----------------------------------------------------------
static void usage (void);
//----------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  struct tm *timeptr;
  char string[5];
  time_t timer;
  int length;
  char *input;
  int i;
  int status;
  // 
  // There can be at most an argument.
  // 
  if (argc > 2)
    {
      usage ();
      return (1);
    }
  // 
  // Check if there is no argument: must show the
  // date.
  // 
  if (argc == 1)
    {
      timer = time (NULL);
      printf ("%s\n", ctime (&timer));
      return (0);
    }
  // 
  // There is one argument and must be the date do
  // set.
  // 
  input = argv[1];
  // 
  // First get current date, for default values.
  // 
  timer = time (NULL);
  timeptr = gmtime (&timer);
  // 
  // Verify to have a correct input.
  // 
  length = (int) strlen (input);
  if (length == 8 || length == 10 || length == 12)
    {
      for (i = 0; i < length; i++)
        {
          if (!isdigit (input[i]))
            {
              usage ();
              return (2);
            }
        }
    }
  else
    {
      printf ("input: \"%s\"; length: %i\n", input, length);
      usage ();
      return (3);
    }
  // 
  // Select the month.
  // 
  string[0] = input[0];
  string[1] = input[1];
  string[2] = '\0';
  timeptr->tm_mon = atoi (string);
  // 
  // Select the day.
  // 
  string[0] = input[2];
  string[1] = input[3];
  string[2] = '\0';
  timeptr->tm_mday = atoi (string);
  // 
  // Select the hour.
  // 
  string[0] = input[4];
  string[1] = input[5];
  string[2] = '\0';
  timeptr->tm_hour = atoi (string);
  // 
  // Select the minute.
  // 
  string[0] = input[6];
  string[1] = input[7];
  string[2] = '\0';
  timeptr->tm_min = atoi (string);
  // 
  // Select the year: must verify if there is a
  // century.
  // 
  if (length == 12)
    {
      string[0] = input[8];
      string[1] = input[9];
      string[2] = input[10];
      string[3] = input[11];
      string[4] = '\0';
      timeptr->tm_year = atoi (string);
    }
  else if (length == 10)
    {
      sprintf (string, "%04i", timeptr->tm_year);
      string[2] = input[8];
      string[3] = input[9];
      string[4] = '\0';
      timeptr->tm_year = atoi (string);
    }
  // 
  // Now convert to `time_t'.
  // 
  timer = mktime (timeptr);
  // 
  // Save to the system.
  // 
  status = stime (&timer);
  if (status != 0)
    {
      perror (NULL);
    }
  // 
  return (0);
}

//----------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr, "Usage: date [MMDDHHMM[[CC]YY]]\n");
}
