#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <string.h>
#include <limits.h>
#include <libgen.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <stdint.h>
#include <stdbool.h>
#include <fcntl.h>
//----------------------------------------------------------
static void usage (void);
char buffer[BUFSIZ];
//----------------------------------------------------------
int
main (int argc, char *argv[], char *envp[])
{
  bool option_l = 0;
  bool option_u = 0;
  int opt;
//extern char *optarg;              // not used.
  extern int optind;
  extern int optopt;
  // 
  int status;
  int sfdn;
  int sfdn2;
  struct sockaddr_in sa_local;
  struct sockaddr_in sa_remote;
  socklen_t sa_remote_size = sizeof (struct sockaddr_in);
  ssize_t read_size;
  ssize_t sent_size;
  ssize_t recv_size;
  char *addr = NULL;
  char *port = NULL;
  bool can_rx = 1;
  bool can_tx = 1;
  // 
  // Check for options.
  // 
  while ((opt = getopt (argc, argv, ":ul")) != -1)
    {
      switch (opt)
        {
        case 'l':
          option_l = 1;
          break;
        case 'u':
          option_u = 1;
          break;
        case '?':
          fprintf (stderr, "Unknown option -%c.\n", optopt);
          usage ();
          return (1);
          break;
        case ':':
          fprintf (stderr,
                   "Missing argument for option -%c\n",
                   optopt);
          usage ();
          return (1);
          break;
        default:
          fprintf (stderr,
                   "Getopt problem: unknown option %c\n",
                   opt);
          usage ();
          return (1);
        }
    }
  // 
  // Arguments.
  // 
  if (optind == (argc - 2))
    {
      // 
      // There are exactly two arguments: destination
      // address and port.
      // 
      addr = argv[argc - 2];
      port = argv[argc - 1];
    }
  else
    {
      // 
      // Arguments wrong!
      // 
      usage ();
      return (2);
    }
  // 
  // Set the local or the remote address.
  // 
  if (option_l)
    {
      // 
      // Address and port are local.
      // 
      sa_local.sin_family = AF_INET;
      sa_local.sin_port = htons (atoi (port));
      inet_pton (AF_INET, addr, &sa_local.sin_addr.s_addr);
    }
  else
    {
      // 
      // Address and port are remote.
      // 
      sa_remote.sin_family = AF_INET;
      sa_remote.sin_port = htons (atoi (port));
      inet_pton (AF_INET, addr, &sa_remote.sin_addr.s_addr);
    }
  // 
  // Open the socket.
  // 
  if (option_u)
    {
      sfdn = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
    }
  else
    {
      sfdn = socket (AF_INET, SOCK_STREAM, IPPROTO_TCP);
    }
  if (sfdn < 0)
    {
      perror (NULL);
      return (3);
    }
  // 
  // Set it listening or connect.
  // 
  if (option_l)
    {
      // 
      // Bind the local `sa' location.
      // 
      status =
        bind (sfdn, (struct sockaddr *) &sa_local,
              sizeof (sa_local));
      if (status < 0)
        {
          perror (NULL);
          close (sfdn);
          return (4);
        }
      // 
      // Listen (TCP) or wait the first packet (UDP).
      // 
      if (option_u)
        {
          // 
          // Instead of listening, we use the function 
          // `recvfrom()',
          // to get the remote address and port.
          // 
          recv_size =
            recvfrom (sfdn, &buffer,
                      (size_t) BUFSIZ - 1, 0,
                      (struct sockaddr *) &sa_remote,
                      &sa_remote_size);
          if (recv_size < 0)
            {
              perror (NULL);
              close (sfdn);
              return (4);
            }
          // 
          // Now connect the remote destination
          // 
          status =
            connect (sfdn,
                     (struct sockaddr *) &sa_remote,
                     sizeof (sa_remote));
          if (status < 0)
            {
              perror (NULL);
              close (sfdn);
              return (7);
            }
          // 
          // And show what was received as a first
          // packet.
          // 
          buffer[recv_size] = 0;
          printf ("%s", buffer);
        }
      else
        {
          // 
          // TCP: listen.
          // 
          status = listen (sfdn, 1);
          if (status < 0)
            {
              perror (NULL);
              close (sfdn);
              return (5);
            }
          // 
          // Accept.
          // 
          sfdn2 =
            accept (sfdn,
                    (struct sockaddr *) &sa_remote,
                    &sa_remote_size);
          if (sfdn2 < 0)
            {
              perror (NULL);
              close (sfdn);
              return (6);
            }
          // 
          // Close listening socket.
          // 
          close (sfdn);
          // 
          // Variable `sfdn' will be the new socket.
          // 
          sfdn = sfdn2;
        }
    }
  else
    {
      // 
      // Connect the remote destination.
      // 
      status =
        connect (sfdn, (struct sockaddr *) &sa_remote,
                 sizeof (sa_remote));
      if (status < 0)
        {
          perror (NULL);
          close (sfdn);
          return (7);
        }
    }
  // 
  // Define the standard input non blocking.
  // 
  status = fcntl (STDIN_FILENO, F_SETFL, O_NONBLOCK);
  if (status < 0)
    {
      perror (NULL);
      return (8);
    }
  // 
  // Will read from the remote and show to the screen.
  // 
  while (can_rx || can_tx)
    {
      if (can_rx)
        {
          recv_size =
            recv (sfdn, &buffer, (size_t) BUFSIZ - 1, 0);
//        recv_size = read (sfdn, &buffer, (size_t) BUFSIZ-1);
          if (recv_size < 0)
            {
              if (errno == EAGAIN || errno == EWOULDBLOCK)
                {
                  ;
                }
              else
                {
                  perror (NULL);
                  close (sfdn);
                  return (10);
                }
            }
          else if (recv_size == 0)
            {
              // 
              // End of stream.
              // 
              can_rx = 0;
              printf ("--end of receive stream--\n");
            }
          else
            {
              buffer[recv_size] = 0;
              printf ("%s", buffer);
            }
        }
      if (can_tx)
        {
          read_size = read (STDIN_FILENO, buffer, BUFSIZ);
          if (read_size < 0)
            {
              if (errno == EAGAIN || errno == EWOULDBLOCK)
                {
                  ;
                }
              else
                {
                  perror (NULL);
                  close (sfdn);
                  return (11);
                }
            }
          else if (read_size == 0)
            {
              // 
              // End of input.
              // 
              printf ("--closing send stream--\n");
              can_tx = 0;
            }
          else
            {
              // 
              // Send it.
              // 
              sent_size =
                send (sfdn, &buffer, (size_t) read_size, 0);
              if (sent_size < 0)
                {
                  if (errno == EAGAIN
                      || errno == EWOULDBLOCK)
                    {
                      ;
                    }
                  else
                    {
                      perror (NULL);
                      close (sfdn);
                      return (12);
                    }
                }
            }
        }
    }
  // 
  // All done.
  // 
  close (sfdn);
  return (0);
}

//----------------------------------------------------------
static void
usage (void)
{
  fprintf (stderr,
           "os32 netcat usage:\n"
           "\n"
           "nc [-u][-l] ADDRESS PORT\n"
           "\n"
           "-u      Use UDP protocol instead of TCP.\n"
           "-l      Listen for incoming connection requests.\n"
           "ADDRESS IPv4 numeric address; if option -l is used, this\n"
           "        is the local address, otherwise it is the remote\n"
           "        address.\n"
           "PORT    TCP or UDP port; if option -l is used, this is\n"
           "        local address, otherwise it is the remote\n"
           "        address.\n");
}
