#include <sys/os32.h>
#include <kernel/blk.h>
#include <kernel/dev.h>
#include <kernel/driver/ata.h>
#include <kernel/lib_k.h>
#include <kernel/dm.h>
#include <sys/types.h>
#include <ctype.h>
#include <string.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
void *
blk_ata (dev_t device, int rw, unsigned int n, void *buffer)
{
  ata_sector_t *destination = buffer;
  ata_sector_t *source = buffer;
  int dev_minor = minor (device);
  int d = ((dev_minor & 0x00F0) >> 4);
  ptrdiff_t ptrdiff;
  int drive;
  unsigned int sector = n * (BLK_SIZE / ATA_SECTOR_SIZE);
  size_t count = (BLK_SIZE / ATA_SECTOR_SIZE);
  int status;
  int c;
  void *cache;
  // 
  // Convert the table pointer to a drive number.
  // 
  ptrdiff = (((intptr_t) dm_table[d].table)
             - ((intptr_t) ata_table));
  drive = ptrdiff / (sizeof (ata_t));
  // 
  if (DEBUG)
    {
      k_printf ("%s: R/W=%i dev=%04x n=%ui ", __FILE__,
                rw, (int) device, n);
      blk_cache_check ();
    }
  // 
  // If reading, check if we already have inside
  // cache.
  // 
  if (rw == DEV_READ)
    {
      cache = blk_cache_read (device, n);
      if (cache != NULL)
        {
          return (cache);
        }
    }
  // 
  // Read or write.
  // 
  for (c = 0, status = 0;
       c < count && status == 0; c++, sector++)
    {
      // 
      if (DEBUG)
        {
          k_printf ("sec=%i ", sector);
        }
      // 
      if (rw == DEV_READ)
        {
          status = ata_read_sector (drive, sector,
                                    &destination[c]);
        }
      else
        {
          status = ata_write_sector (drive, sector,
                                     &source[c]);
        }
    }
  // 
  // If a block was read or written inside ATA
  // hardware, then
  // save it inside the cache.
  // 
  if (status == 0)
    {
      cache = blk_cache_save (device, n, buffer);
    }
  else
    {
      cache = NULL;
    }
  // 
  // 
  // 
  if (DEBUG)
    {
      k_printf ("\n");
    }
  // 
  return (cache);
}
