#include <kernel/blk.h>
#include <string.h>
//----------------------------------------------------------
void *
blk_cache_save (dev_t device, unsigned int n, void *block)
{
  int i;
  int j;
  int age;
  // 
  device &= 0xFFF0;
  // 
  // Look inside the cache, if we already have
  // that old block.
  // 
  for (i = 0; i < BLK_CACHE_SIZE; i++)
    {
      if (blk_table[i].device == device
          && blk_table[i].n == n)
        {
          age = blk_table[i].age;
          for (j = 0; j < BLK_CACHE_SIZE; j++)
            {
              if (blk_table[j].age < age)
                {
                  blk_table[j].age++;
                }
            }
          blk_table[i].age = 0;
          // 
          // Check if the block is the same memory.
          // 
          if (blk_table[i].block == block)
            {
              // 
              // No need to transfer data.
              // 
              ;
            }
          else
            {
              memcpy (blk_table[i].block, block,
                      (size_t) BLK_SIZE);
            }
          return (&blk_table[i].block);
        }
    }
  // 
  // The block is new for the cache: must find
  // the older and replace it.
  // 
  for (i = 0; i < BLK_CACHE_SIZE; i++)
    {
      if (blk_table[i].age == BLK_CACHE_MAX_AGE)
        {
          for (j = 0; j < BLK_CACHE_SIZE; j++)
            {
              if (blk_table[j].age < BLK_CACHE_MAX_AGE)
                {
                  blk_table[j].age++;
                }
            }
          blk_table[i].age = 0;
          blk_table[i].device = device;
          blk_table[i].n = n;
          memcpy (blk_table[i].block, block,
                  (size_t) BLK_SIZE);
          // 
          return (&blk_table[i].block);
        }
    }
  // 
  // It should never happen to fail.
  // 
  return (NULL);
}
