#include <sys/os32.h>
#include <kernel/dev.h>
#include <kernel/blk.h>
#include <kernel/driver/ata.h>
#include <kernel/lib_k.h>
#include <kernel/dm.h>
#include <sys/types.h>
#include <ctype.h>
#include <errno.h>
//----------------------------------------------------------
ssize_t
dev_ata (pid_t pid, dev_t device, int rw, off_t offset,
         void *buffer, size_t size, int *eof)
{
  ssize_t m;
  ssize_t n = 0;
  unsigned char *data_buffer = buffer;
  unsigned int n_blk;
  int i;
  int j = 0;
  char *block;
  // 
  // Read the first block.
  // 
  n_blk = offset / BLK_SIZE;
  i = offset % BLK_SIZE;
  // 
  block = blk_ata (device, DEV_READ, n_blk, NULL);
  if (block == NULL)
    {
      errset (errno);
      return ((ssize_t) - 1);
    }
  // 
  // Read or write.
  // 
  if (rw == DEV_READ)
    {
      while (size)
        {
          for (;
               i < BLK_SIZE && size > 0;
               i++, j++, n++, size--, offset++)
            {
              data_buffer[j] = block[i];
            }
          if (size)
            {
              n_blk = offset / BLK_SIZE;
              i = offset % BLK_SIZE;
              block = blk_ata (device, DEV_READ, n_blk,
                               NULL);
              if (block == NULL)
                {
                  errset (errno);
                  return (n);   // Up to the last
                  // block read.
                }
            }
        }
    }
  else
    {
      // 
      // Write.
      // 
      while (size)
        {
          // 
          // The last block was written, so update
          // the counter `m'.
          // 
          m = n;
          // 
          for (;
               i < BLK_SIZE && size > 0;
               i++, j++, n++, size--, offset++)
            {
              block[i] = data_buffer[j];
            }
          block = blk_ata (device, DEV_WRITE, n_blk, block);
          if (block == NULL)
            {
              errset (errno);
              return (m);       // Up to the last
              // block written.
            }
          if (size)
            {
              n_blk = offset / BLK_SIZE;
              i = offset % BLK_SIZE;
              block = blk_ata (device, DEV_READ, n_blk,
                               NULL);
              if (block == NULL)
                {
                  errset (errno);
                  return (m);   // Up to the last
                  // block written.
                }
            }
        }
    }
  // 
  // Everything was right, so `n' is valid for both
  // read or write.
  // 
  return (n);
}
