#include <sys/os32.h>
#include <kernel/dev.h>
#include <kernel/driver/ata.h>
#include <kernel/lib_k.h>
#include <kernel/dm.h>
#include <sys/types.h>
#include <ctype.h>
#include <errno.h>
//----------------------------------------------------------
ssize_t
dev_dm (pid_t pid, dev_t device, int rw, off_t offset,
        void *buffer, size_t size, int *eof)
{
  int dev_minor = minor (device);
  int p = dev_minor & 0x000F;
  int d = ((dev_minor & 0x00F0) >> 4);
  // 
  // If it is a partition, must verify if such
  // partition exists.
  // 
  if (p)
    {
      // 
      // It is a partition.
      // 
      if (dm_table[d].part[p].type == PART_TYPE_NONE)
        {
          errset (ENODEV);
          return ((ssize_t) - 1);
        }
    }
  // 
  // Shift the offset to the start of partition.
  // 
  offset += (dm_table[d].part[p].start);
  // 
  // Call the right hardware driver.
  // 
  switch (dm_table[d].type)
    {
    case DM_TYPE_ATA:
      return (dev_ata
              (pid, device, rw, offset, buffer, size, eof));
      break;
    default:
      errset (ENODEV);
      return ((ssize_t) - 1);
    }
}
