#include <sys/os32.h>
#include <kernel/dev.h>
#include <sys/types.h>
#include <errno.h>
#include <kernel/memory.h>
#include <kernel/ibm_i386.h>
#include <kernel/proc.h>
#include <string.h>
#include <signal.h>
#include <kernel/lib_k.h>
#include <ctype.h>
//----------------------------------------------------------
ssize_t
dev_mem (pid_t pid, dev_t device, int rw, off_t offset,
         void *buffer, size_t size, int *eof)
{
  uint8_t *buffer08 = (uint8_t *) buffer;
  uint16_t *buffer16 = (uint16_t *) buffer;
  ssize_t n;

  if (device == DEV_MEM)        // DEV_MEM
    {
      if (rw == DEV_READ)
        {
          memcpy (buffer, (void *) (int) offset, size);
          n = size;
        }
      else
        {
          if (pid == 0)
            {
              memcpy ((void *) (int) offset, buffer, size);
              n = size;
            }
          else
            {
              k_printf
                ("kernel alert: only the kernel "
                 "can write the memory where it "
                 "likes!\n");
              errset (EIO);     // I/O error.
              return ((ssize_t) - 1);
            }
        }
    }
  else if (device == DEV_NULL)  // DEV_NULL
    {
      n = 0;
    }
  else if (device == DEV_ZERO)  // DEV_ZERO
    {
      if (rw == DEV_READ)
        {
          for (n = 0; n < size; n++)
            {
              buffer08[n] = 0;
            }
        }
      else
        {
          n = 0;
        }
    }
  else if (device == DEV_PORT)  // DEV_PORT
    {
      if (rw == DEV_READ)
        {
          if (size == 1)
            {
              buffer08[0] = in_8 (offset);
              n = 1;
            }
          else if (size == 2)
            {
              buffer16[0] = in_16 (offset);
              n = 2;
            }
          else
            {
              n = 0;
            }
        }
      else
        {
          if (size == 1)
            {
              out_8 (offset, buffer08[0]);
            }
          else if (size == 2)
            {
              out_16 (offset, buffer16[0]);
              n = 2;
            }
          else
            {
              n = 0;
            }
        }
    }
  else
    {
      errset (ENODEV);
      return ((ssize_t) - 1);
    }
  return (n);
}
