#include <sys/os32.h>
#include <kernel/dev.h>
#include <sys/types.h>
#include <errno.h>
#include <kernel/memory.h>
#include <kernel/ibm_i386.h>
#include <kernel/proc.h>
#include <string.h>
#include <signal.h>
#include <kernel/lib_k.h>
#include <ctype.h>
#include <kernel/driver/tty.h>
//----------------------------------------------------------
ssize_t
dev_tty (pid_t pid, dev_t device, int rw, off_t offset,
         void *buffer, size_t size, int *eof)
{
  uint8_t *buffer08 = (uint8_t *) buffer;
  ssize_t n;
  proc_t *ps;
  int key;
  // 
  // Get process. Variable `ps' will be `NULL' if the
  // process ID is
  // not valid.
  // 
  ps = proc_reference (pid);
  // 
  // Convert `DEV_TTY' with the controlling terminal
  // for the process.
  // 
  if (device == DEV_TTY)
    {
      device = ps->device_tty;
      // 
      // As a last resort, use the generic
      // `DEV_CONSOLE'.
      // 
      if (device == DEV_UNDEFINED || device == DEV_TTY)
        {
          device = DEV_CONSOLE;
        }
    }
  // 
  // Convert `DEV_CONSOLE' to the currently active
  // console.
  // 
  if (device == DEV_CONSOLE)
    {
      device = tty_console ((dev_t) 0);
      // 
      // As a last resort, use the first console:
      // `DEV_CONSOLE0'.
      // 
      if (device == DEV_UNDEFINED || device == DEV_TTY)
        {
          device = DEV_CONSOLE0;
        }
    }
  // 
  // Read or write.
  // 
  if (rw == DEV_READ)
    {
      for (n = 0; n < size; n++)
        {
          key = tty_read (device);
          if (key == 0 && n == 0)
            {
              // 
              // A single line contains zero: this is
              // made by a VEOF
              // character (^d), that is, the input is 
              // closed,
              // so return zero read and EOF.
              // 
              *eof = 1;
              return (0);
            }
          else if (key == -1 && n == 0)
            {
              // 
              // At the moment, there is just nothing
              // to read.
              // 
              errset (EAGAIN);
              return (-1);
            }
          else if (key == -1 && n > 0)
            {
              // 
              // Finished to read.
              // 
              break;
            }
          else
            {
              buffer08[n] = key;
            }
        }
    }
  else
    {
      for (n = 0; n < size; n++)
        {
          tty_write (device, (int) buffer08[n]);
        }
    }
  return (n);
}
