#include <kernel/dm.h>
#include <kernel/part.h>
#include <kernel/driver/ata.h>
#include <kernel/lib_k.h>
#include <stdint.h>
#include <errno.h>
//----------------------------------------------------------
void
dm_init (void)
{
  int d;
  int a;
  int p;
  ata_sector_t sector_buffer;
  part_t *part;
  int status;
  // 
  // Reset the data-memory table.
  // 
  for (d = 0; d < DM_MAX_DEVICES; d++)
    {
      dm_table[d].type = DM_TYPE_NONE;
      dm_table[d].table = NULL;
      dm_table[d].part[0].start = 0;
      dm_table[d].part[0].size = 0;
      dm_table[d].part[0].type = PART_TYPE_NO_PART;
      for (p = 0; p < PART_MAX; p++)
        {
          dm_table[d].part[p + 1].start = 0;
          dm_table[d].part[p + 1].size = 0;
          dm_table[d].part[p + 1].type = PART_TYPE_NONE;
        }
    }
  // 
  // Reset data-memory index.
  // 
  d = 0;
  // 
  // Init ATA devices.
  // 
  ata_init ();
  // 
  // Assign ATA devices to the first data-memory
  // items.
  // 
  for (a = 0; a < ATA_MAX_DEVICES; a++)
    {
      if (ata_table[a].present == 0)
        {
          // 
          // Current data-memory device will be
          // used for the next ATA device, if any.
          // 
          continue;
        }
      // 
      // Show something.
      // 
      k_printf ("[%s] ATA drive=%i total sectors=%i\n",
                __func__, a, (int) ata_table[d].sectors);
      // 
      dm_table[d].type = DM_TYPE_ATA;
      dm_table[d].table = &ata_table[a];
      dm_table[d].part[0].start = 0;
      dm_table[d].part[0].size = ata_table[a].sectors;
      dm_table[d].part[0].type = PART_TYPE_NO_PART;
      // 
      // Read partitions.
      // 
      status = ata_read_sector (a, 0, &sector_buffer);
      // 
      if (status)
        {
          errset (errno);
          k_perror (NULL);
        }
      else
        {
          part =
            (((void *) &sector_buffer) + PART_TABLE_OFF);
          // 
          for (p = 0; p < PART_MAX; p++)
            {
              // 
              dm_table[d].part[p + 1].start =
                part->l_start * ATA_SECTOR_SIZE;
              dm_table[d].part[p + 1].size =
                part->size * ATA_SECTOR_SIZE;
              dm_table[d].part[p + 1].type = part->type;
              // 
              // Show info.
              // 
              if (part->type != 0)
                {
                  k_printf ("[%s] partition type=%02x "
                            "start sector=%i "
                            "total sectors=%i\n",
                            __func__, (int) part->type,
                            (int) part->l_start,
                            (int) part->size);
                }
              // 
              part++;
            }
        }
      // 
      // Next data-memory device.
      // 
      d++;
    }
}
