#include <kernel/driver/ata.h>
#include <kernel/lib_k.h>
#include <kernel/ibm_i386.h>
#include <stdint.h>
#include <errno.h>
//----------------------------------------------------------
int
ata_cmd_identify_device (int drive, void *buffer)
{
  unsigned char status;
  int s;
  int i;
  uint16_t *id = buffer;
  // 
  // Register `device'.
  // 
  s = ata_device (drive, 0);
  if (s < 0)
    {
      errset (errno);
      return (-1);
    }
  // 
  // Send `command'
  // 
  out_8 (ata_table[drive].r_command,
         ATA_COMMAND_IDENTIFY_DEVICE);
  // 
  // Read the regular status port.
  // 
  status = in_8 (ata_table[drive].r_status);
  // 
  // If the status is zero, there is no drive.
  // 
  if (status == 0)
    {
      // 
      // Clear the `id[]' array and return.
      // 
      for (i = 0; i < (ATA_SECTOR_SIZE / 2); i++)
        {
          id[i] = 0;
        }
      return (0);
    }
  // 
  // Wait for the drive ready to send data.
  // 
  s = ata_drq (drive, ATA_TIMEOUT);
  if (s < 0)
    {
      errset (errno);
      return (-1);
    }
  // 
  // Read data.
  // 
  for (i = 0; i < (ATA_SECTOR_SIZE / 2); i++)
    {
      id[i] = in_16 (ata_table[drive].r_data);
    }
  // 
  // Return.
  // 
  return (0);
}
