#include <kernel/driver/ata.h>
#include <kernel/lib_s.h>
#include <kernel/ibm_i386.h>
#include <stdint.h>
#include <errno.h>
//----------------------------------------------------------
int
ata_device (int drive, unsigned int sector)
{
  unsigned char device;
  int s;
  // 
  // Verify `drive' argument.
  // 
  s = ata_valid (drive);
  if (s < 0)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  // Start building the `device' register.
  // 
  device = 0;
  // 
  // The access will be LBA: no CHS at all here!
  // 
  device |= ATA_DEVICE_LBA;
  // 
  // Set the device number, relative to the bus.
  // 
  device |= ata_table[drive].target;
  // 
  // Put the highest four bits of the sector number,
  // that can use at most 28 bits.
  // 
  device |= ((sector & 0x0F000000) >> 24);
  // 
  // Must select the new drive.
  // 
  out_8 (ata_table[drive].r_device, device);
  // 
  // Wait for selected drive ready.
  // 
  s = ata_rdy (drive, ATA_TIMEOUT);
  if (s < 0)
    {
      errset (errno);
      return (-1);
    }
  // 
  // Ok.
  // 
  return (0);
}
