#include <kernel/driver/ata.h>
#include <kernel/lib_k.h>
#include <kernel/ibm_i386.h>
#include <stdint.h>
#include <errno.h>
//----------------------------------------------------------
void
ata_init (void)
{
  unsigned char status;
  int s;
  int d;
  // 
  // Set bus numbers and I/O ports for each possible
  // drive.
  // I/O ports are related to the bus, so every couple 
  // of
  // drive has the same ports.
  // 
  if (ATA_MAX_DEVICES > 0)
    {
      ata_table[0].bus = 0;
      ata_table[0].r_data = ATA0_DATA;
      ata_table[0].r_feature = ATA0_FEATURE;
      ata_table[0].r_error = ATA0_ERROR;
      ata_table[0].r_count = ATA0_COUNT;
      ata_table[0].r_low = ATA0_LOW;
      ata_table[0].r_mid = ATA0_MID;
      ata_table[0].r_high = ATA0_HIGH;
      ata_table[0].r_device = ATA0_DEVICE;
      ata_table[0].r_command = ATA0_COMMAND;
      ata_table[0].r_status = ATA0_STATUS;
      ata_table[0].r_control = ATA0_CONTROL;
      ata_table[0].r_alternate = ATA0_ALTERNATE;
      ata_table[0].target = ATA_DEVICE_MASTER;
      // 
      ata_table[1].bus = 0;
      ata_table[1].r_data = ATA0_DATA;
      ata_table[1].r_feature = ATA0_FEATURE;
      ata_table[1].r_error = ATA0_ERROR;
      ata_table[1].r_count = ATA0_COUNT;
      ata_table[1].r_low = ATA0_LOW;
      ata_table[1].r_mid = ATA0_MID;
      ata_table[1].r_high = ATA0_HIGH;
      ata_table[1].r_device = ATA0_DEVICE;
      ata_table[1].r_command = ATA0_COMMAND;
      ata_table[1].r_status = ATA0_STATUS;
      ata_table[1].r_control = ATA0_CONTROL;
      ata_table[1].r_alternate = ATA0_ALTERNATE;
      ata_table[1].target = ATA_DEVICE_SLAVE;
    }
  // 
  if (ATA_MAX_DEVICES > 2)
    {
      ata_table[2].bus = 1;
      ata_table[2].r_data = ATA1_DATA;
      ata_table[2].r_feature = ATA1_FEATURE;
      ata_table[2].r_error = ATA1_ERROR;
      ata_table[2].r_count = ATA1_COUNT;
      ata_table[2].r_low = ATA1_LOW;
      ata_table[2].r_mid = ATA1_MID;
      ata_table[2].r_high = ATA1_HIGH;
      ata_table[2].r_device = ATA1_DEVICE;
      ata_table[2].r_command = ATA1_COMMAND;
      ata_table[2].r_status = ATA1_STATUS;
      ata_table[2].r_control = ATA1_CONTROL;
      ata_table[2].r_alternate = ATA1_ALTERNATE;
      ata_table[2].target = ATA_DEVICE_MASTER;
      // 
      ata_table[3].bus = 1;
      ata_table[3].r_data = ATA1_DATA;
      ata_table[3].r_feature = ATA1_FEATURE;
      ata_table[3].r_error = ATA1_ERROR;
      ata_table[3].r_count = ATA1_COUNT;
      ata_table[3].r_low = ATA1_LOW;
      ata_table[3].r_mid = ATA1_MID;
      ata_table[3].r_high = ATA1_HIGH;
      ata_table[3].r_device = ATA1_DEVICE;
      ata_table[3].r_command = ATA1_COMMAND;
      ata_table[3].r_status = ATA1_STATUS;
      ata_table[3].r_control = ATA1_CONTROL;
      ata_table[3].r_alternate = ATA1_ALTERNATE;
      ata_table[3].target = ATA_DEVICE_SLAVE;
    }
  // 
  if (ATA_MAX_DEVICES > 4)
    {
      ata_table[4].bus = 2;
      ata_table[4].r_data = ATA2_DATA;
      ata_table[4].r_feature = ATA2_FEATURE;
      ata_table[4].r_error = ATA2_ERROR;
      ata_table[4].r_count = ATA2_COUNT;
      ata_table[4].r_low = ATA2_LOW;
      ata_table[4].r_mid = ATA2_MID;
      ata_table[4].r_high = ATA2_HIGH;
      ata_table[4].r_device = ATA2_DEVICE;
      ata_table[4].r_command = ATA2_COMMAND;
      ata_table[4].r_status = ATA2_STATUS;
      ata_table[4].r_control = ATA2_CONTROL;
      ata_table[4].r_alternate = ATA2_ALTERNATE;
      ata_table[4].target = ATA_DEVICE_MASTER;
      // 
      ata_table[5].bus = 2;
      ata_table[5].r_data = ATA2_DATA;
      ata_table[5].r_feature = ATA2_FEATURE;
      ata_table[5].r_error = ATA2_ERROR;
      ata_table[5].r_count = ATA2_COUNT;
      ata_table[5].r_low = ATA2_LOW;
      ata_table[5].r_mid = ATA2_MID;
      ata_table[5].r_high = ATA2_HIGH;
      ata_table[5].r_device = ATA2_DEVICE;
      ata_table[5].r_command = ATA2_COMMAND;
      ata_table[5].r_status = ATA2_STATUS;
      ata_table[5].r_control = ATA2_CONTROL;
      ata_table[5].r_alternate = ATA2_ALTERNATE;
      ata_table[5].target = ATA_DEVICE_SLAVE;
    }
  // 
  if (ATA_MAX_DEVICES > 6)
    {
      ata_table[6].bus = 3;
      ata_table[6].r_data = ATA3_DATA;
      ata_table[6].r_feature = ATA3_FEATURE;
      ata_table[6].r_error = ATA3_ERROR;
      ata_table[6].r_count = ATA3_COUNT;
      ata_table[6].r_low = ATA3_LOW;
      ata_table[6].r_mid = ATA3_MID;
      ata_table[6].r_high = ATA3_HIGH;
      ata_table[6].r_device = ATA3_DEVICE;
      ata_table[6].r_command = ATA3_COMMAND;
      ata_table[6].r_status = ATA3_STATUS;
      ata_table[6].r_control = ATA3_CONTROL;
      ata_table[6].r_alternate = ATA3_ALTERNATE;
      ata_table[6].target = ATA_DEVICE_MASTER;
      // 
      ata_table[7].bus = 3;
      ata_table[7].r_data = ATA3_DATA;
      ata_table[7].r_feature = ATA3_FEATURE;
      ata_table[7].r_error = ATA3_ERROR;
      ata_table[7].r_count = ATA3_COUNT;
      ata_table[7].r_low = ATA3_LOW;
      ata_table[7].r_mid = ATA3_MID;
      ata_table[7].r_high = ATA3_HIGH;
      ata_table[7].r_device = ATA3_DEVICE;
      ata_table[7].r_command = ATA3_COMMAND;
      ata_table[7].r_status = ATA3_STATUS;
      ata_table[7].r_control = ATA3_CONTROL;
      ata_table[7].r_alternate = ATA3_ALTERNATE;
      ata_table[7].target = ATA_DEVICE_SLAVE;
    }
  // 
  // Scan and eliminate buses with no drive at all.
  // 
  for (d = 0; d < ATA_MAX_DEVICES; d += 2)
    {
      status = in_8 (ata_table[d].r_status);
      if (status == 0xFF)
        {
          ata_table[d].present = 0;
          ata_table[d + 1].present = 0;
        }
      else
        {
          ata_table[d].present = 1;
          ata_table[d + 1].present = 1;
        }
    }
  // 
  // Identify drives.
  // 
  for (d = 0; d < ATA_MAX_DEVICES; d++)
    {
      if (ata_table[d].present == 0)
        {
          continue;
        }
      // 
      // Register `device'.
      // 
      s = ata_device (d, 0);
      if (s < 0)
        {
          errset (errno);
          k_perror (NULL);
          ata_table[d].present = 0;
          continue;
        }
      // 
      // Send command `IDENTIFY DEVICE'
      // 
      s =
        ata_cmd_identify_device (d, &(ata_table[d].id[0]));
      if (s < 0)
        {
          ata_table[d].present = 0;
          continue;
        }
      // 
      // Verify again if the drive is present: the
      // function might have found that it does not
      // exist.
      // 
      if (ata_table[d].present == 0)
        {
          continue;
        }
      // 
      // Find total sectors (for 28 bit LBA access).
      // It is written
      // inside the integer formed by `identity[60]'
      // and
      // `identity[61]', considering it in little
      // endian mode.
      // It is taken the pointer to `identity[60]',
      // transformed
      // into a pointer to a 32 bit integer, and then
      // dereferenced
      // again.
      // 
      ata_table[d].sectors
        = *((uint32_t *) & (ata_table[d].id[60]));
      // 
      // Check if the size value is right.
      // 
      if (ata_table[d].sectors == 0)
        {
          ata_table[d].present = 0;
        }
      else
        {
          // 
          // Show info.
          // 
          k_printf ("[%s] ATA drive %i size %i Kib\n",
                    __func__, d, ata_table[d].sectors / 2);
        }
    }
}
