#include <kernel/driver/kbd.h>
#include <kernel/ibm_i386.h>
//----------------------------------------------------------
void
kbd_isr (void)
{
  // 
  // Get a character from the keyboard channel.
  // 
  unsigned char scancode = (int) in_8 (0x60);
  // 
  // Check for [Shift], [Shift-Lock], [Ctrl] and [Alt] 
  // keys.
  // 
  switch (scancode)
    {
    case 0x2A:
      kbd.shift = 1;
      break;
    case 0x36:
      kbd.shift = 1;
      break;
    case 0xAA:
      kbd.shift = 0;
      break;
    case 0xB6:
      kbd.shift = 0;
      break;
    case 0x1D:
      kbd.ctrl = 1;
      break;
    case 0x9D:
      kbd.ctrl = 0;
      break;
    case 0x38:
      kbd.alt = 1;
      break;
    case 0xB8:
      kbd.alt = 0;
      break;
    case 0x3A:
      kbd.shift_lock = !kbd.shift_lock;
      break;
    }
  // 
  // Check for usual keys, but only if `kbd.key' is
  // currently empty.
  // 
  if (scancode <= 127 && kbd.ctrl && kbd.key == 0)
    {
      // 
      // Something was pressed, combined with [Ctrl].
      // 
      switch (kbd.map1[scancode])
        {
        case 'a':
          kbd.key = 0x01;
          break;        // SOH
        case 'b':
          kbd.key = 0x02;
          break;        // STX
        case 'c':
          kbd.key = 0x03;
          break;        // ETX
        case 'd':
          kbd.key = 0x04;
          break;        // EOT
        case 'e':
          kbd.key = 0x05;
          break;        // ENQ
        case 'f':
          kbd.key = 0x06;
          break;        // ACK
        case 'g':
          kbd.key = 0x07;
          break;        // BEL
        case 'h':
          kbd.key = 0x08;
          break;        // BS
        case 'i':
          kbd.key = 0x09;
          break;        // HT
        case 'j':
          kbd.key = 0x0A;
          break;        // LF
        case 'k':
          kbd.key = 0x0B;
          break;        // VT
        case 'l':
          kbd.key = 0x0C;
          break;        // FF
        case 'm':
          kbd.key = 0x0D;
          break;        // CR
        case 'n':
          kbd.key = 0x0E;
          break;        // SO
        case 'o':
          kbd.key = 0x0F;
          break;        // SI
        case 'p':
          kbd.key = 0x10;
          break;        // DLE
        case 'q':
          kbd.key = 0x11;
          break;        // DC1
        case 'r':
          kbd.key = 0x12;
          break;        // DC2
        case 's':
          kbd.key = 0x13;
          break;        // DC3
        case 't':
          kbd.key = 0x14;
          break;        // DC4
        case 'u':
          kbd.key = 0x15;
          break;        // NAK
        case 'v':
          kbd.key = 0x16;
          break;        // SYN
        case 'w':
          kbd.key = 0x17;
          break;        // ETB
        case 'x':
          kbd.key = 0x18;
          break;        // CAN
        case 'y':
          kbd.key = 0x19;
          break;        // EM
        case 'z':
          kbd.key = 0x1A;
          break;        // SUB
        case '[':
          kbd.key = 0x1B;
          break;        // ESC
        case '\\':
          kbd.key = 0x1C;
          break;        // FS
        case ']':
          kbd.key = 0x1D;
          break;        // GS
        case '^':
          kbd.key = 0x1E;
          break;        // RS
        case '_':
          kbd.key = 0x1F;
          break;        // US
        }
    }
  else if (scancode <= 127 && kbd.key == 0
           && kbd.map1[scancode] != 0)
    {
      // 
      // Something was pressed, but no [Ctrl] is
      // there.
      // 
      if (kbd.shift || kbd.shift_lock)
        {
          kbd.key = kbd.map2[scancode];
        }
      else
        {
          kbd.key = kbd.map1[scancode];
        }
    }
}
