#include <kernel/driver/nic/ne2k.h>
#include <kernel/ibm_i386.h>
#include <errno.h>
//----------------------------------------------------------
int
ne2k_check (uintptr_t io)
{
  int status;
  int reg_00;
  int reg_0d;
  // 
  // Read and save the command register (CR, 0x00): if 
  // it is really the
  // command register, should not be equal to 0xFF.
  // 
  reg_00 = in_8 (io + NE2K_CR);
  if (reg_00 == 0xFF)
    {
      errset (E_DRIVER_FAULT);
      return (-1);
    }
  // 
  // Command register (CR)
  // .-------------------------------.
  // |PS1|PS0|RD2|RD1|RD0|TXP|STA|STP|
  // |-------------------------------|
  // | 0 | 1 | 1 | 0 | 0 | 0 | 0 | 1 | 0x61
  // `-------------------------------'
  // \___/ \_______/ |
  // | | STOP
  // | Abort/complete
  // | remote DMA
  // Register
  // page 1
  // 
  // Stop and select page 1.
  // 
  out_8 ((io + NE2K_CR), 0x61);
  // 
  // Read, save and overwrite the register MAR5 (0x0D
  // at
  // page 1).
  // 
  reg_0d = in_8 (io + NE2K_MAR5);
  out_8 (io + NE2K_MAR5, 0xFF);
  // 
  // Command register (CR)
  // .-------------------------------.
  // |PS1|PS0|RD2|RD1|RD0|TXP|STA|STP|
  // |-------------------------------|
  // | 0 | 0 | 1 | 0 | 0 | 0 | 0 | 1 | 0x21
  // `-------------------------------'
  // \___/ \_______/ |
  // | | STOP
  // | Abort/complete
  // | remote DMA
  // Register
  // page 0
  // 
  // Stop and select page 0.
  // 
  out_8 ((io + NE2K_CR), 0x21);
  // 
  // Read the tally counter 0 (CNTR0) to clear it.
  // 
  in_8 (io + NE2K_CNTR0);
  // 
  // Now the tally counter 0 (CNTR0) should be zero.
  // 
  status = in_8 (io + NE2K_CNTR0);
  if (status)
    {
      // 
      // The value obtained is not zero, so it is not
      // a NE2000 nic and the page change had probably
      // no effect. So, restore the values found
      // inside
      // 0x00 and 0x0D, without trying to change page.
      // 
      out_8 (io, reg_00);
      out_8 ((io + 0x0D), reg_0d);
      errset (E_DRIVER_FAULT);
      return (-1);
    }
  // 
  // Everything is ok: it might be a NE2000 nic.
  // 
  return (0);
}
