#include <kernel/driver/pci.h>
#include <kernel/driver/nic/ne2k.h>
#include <kernel/ibm_i386.h>
#include <errno.h>
#include <kernel/lib_k.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------
#define DEBUG 1
//----------------------------------------------------------
int
ne2k_rx_reset (uintptr_t io)
{
  int status;
  // 
  // Command register (CR)
  // .-------------------------------.
  // |PS1|PS0|RD2|RD1|RD0|TXP|STA|STP|
  // |-------------------------------|
  // | 0 | 0 | 1 | 0 | 0 | 0 | 0 | 1 | 0x21
  // `-------------------------------'
  // \_____/ \_________/ |
  // | | STOP
  // | |
  // | Abort/complete remote DMA
  // |
  // Register page 0
  // 
  out_8 ((io + NE2K_CR), 0x21);
  // 
  // Interrupt status register (ISR)
  // .-------------------------------.
  // |RST|RDC|CNT|OVW|TXE|RXE|PTX|PRX|
  // |-------------------------------|
  // | 1 | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 0x80
  // `-------------------------------'
  // |
  // Reset
  // 
  status = ne2k_isr_expect (io, 0x80);
  if (status)
    {
      errset (errno);
      return (-1);
    }
  // 
  // 
  // 
  out_8 ((io + NE2K_RBCR0), 0);
  out_8 ((io + NE2K_RBCR1), 0);
  // 
  // Command register (CR)
  // .-------------------------------.
  // |PS1|PS0|RD2|RD1|RD0|TXP|STA|STP|
  // |-------------------------------|
  // | 0 | 0 | 1 | 0 | 0 | 0 | 1 | 0 | 0x22
  // `-------------------------------'
  // \_____/ \_________/ |
  // | | START
  // | |
  // | Abort/complete remote DMA
  // |
  // Register page 0
  // 
  out_8 (io + NE2K_CR, 0x22);
  // 
  return (0);
}
