#ifndef _KERNEL_DRIVER_PCI_H
#define _KERNEL_DRIVER_PCI_H    1
//----------------------------------------------------------
#include <stdint.h>
#include <sys/types.h>
//----------------------------------------------------------
//
#define PCI_MAX_DEVICES                 8
#define PCI_MAX_BUSES                 256       // Fixed.
#define PCI_MAX_SLOTS                  32       // Fixed.
//
#define PCI_CONFIG_ADDRESS         0x0CF8
#define PCI_CONFIG_DATA            0x0CFC
//
// CONFIG_ADDRESS register structure.
//
typedef union
{
  uint32_t selector;
  struct
  {
    uint32_t zero:2,
      reg:6,
      function:3, slot:5, bus:8, reserved:7, enable:1;
  };
} pci_address_t;
//
// CONFIG_DATA register structures.
//
typedef union
{
  uint32_t r[16];
  struct
  {
    struct
    {
      uint32_t vendor_id:16, device_id:16;
      // 
      uint32_t command:16, status:16;
      // 
      uint32_t revision_id:8,
        prog_if:8, subclass:8, class_code:8;
      // 
      uint32_t cache_line_size:8,
        latency_timer:8,
        header_type:7, multi_function:1, bist:8;
      // 
      uint32_t bar0;
      uint32_t bar1;
      uint32_t bar2;
      uint32_t bar3;
      uint32_t bar4;
      uint32_t bar5;
      uint32_t cardbus_cis_pointer;
      uint32_t expansion_rom_base_address;
      // 
      uint32_t subsystem_vendor_id:16, subsystem_id:16;
      // 
      uint32_t capabilities_pointer:8, reserved_1:24;
      // 
      uint32_t reserved_2;
      // 
      uint32_t interrupt_line:8,
        interrupt_pin:8, min_grant:8, max_latency:8;
    };
  };
} pci_header_type_00_t;
//
//----------------------------------------------------------
//
// PCI table row.
//
typedef struct
{
  unsigned char bus;
  unsigned char slot;
  unsigned short int vendor_id;
  unsigned short int device_id;
  unsigned char class_code;
  unsigned char subclass;
  unsigned char prog_if;
  uintptr_t base_io;
  unsigned char irq;
} pci_t;
//
extern pci_t pci_table[PCI_MAX_DEVICES];
//
//----------------------------------------------------------
void pci_init (void);
//----------------------------------------------------------
#endif
