#include <kernel/driver/pci.h>
#include <kernel/ibm_i386.h>
#include <errno.h>
//----------------------------------------------------------
extern pci_t pci_table[PCI_MAX_DEVICES];
//----------------------------------------------------------
void
pci_init (void)
{
  pci_header_type_00_t pci;
  pci_address_t pci_addr;
  // 
  int t;        // PCI table index.
  int b;        // PCI bus index.
  int s;        // PCI slot index.
  int r;        // PCI header register index.
  // 
  // Reset the PCI table.
  // 
  for (t = 0; t < PCI_MAX_DEVICES; t++)
    {
      pci_table[t].bus = 0;
      pci_table[t].slot = 0;
      pci_table[t].vendor_id = 0;
      pci_table[t].device_id = 0;
      pci_table[t].class_code = 0;
      pci_table[t].subclass = 0;
      pci_table[t].prog_if = 0;
      pci_table[t].base_io = 0;
      pci_table[t].irq = 0;
    }
  // 
  // Scan PCI buses and slots.
  // 
  t = 0;
  // 
  for (b = 0; b < PCI_MAX_BUSES && t < PCI_MAX_DEVICES; b++)
    {
      // 
      // Will not check multi functions devices (we
      // are shure that
      // we don't have them).
      // 
      for (s = 0;
           s < PCI_MAX_SLOTS && t < PCI_MAX_DEVICES; s++)
        {
          pci_addr.selector = 0;
          pci_addr.enable = 1;
          pci_addr.bus = b;
          pci_addr.slot = s;
          // 
          pci_addr.reg = 0;
          out_32 (PCI_CONFIG_ADDRESS, pci_addr.selector);
          pci.r[0] = in_32 (PCI_CONFIG_DATA);
          // 
          if (pci.r[0] == 0xFFFFFFFF)
            {
              // 
              // There is no such bus:slot
              // combination!
              // 
              continue;
            }
          else
            {
              for (r = 1; r < 16; r++)
                {
                  pci_addr.reg = r;
                  out_32 (PCI_CONFIG_ADDRESS,
                          pci_addr.selector);
                  pci.r[r] = in_32 (PCI_CONFIG_DATA);
                }
            }
          // 
          // We consider only PCI header type 0x00!
          // 
          if (pci.header_type != 0)
            {
              continue;
            }
          // 
          // We do not consider PCI bridge devices!
          // 
          if (pci.class_code == 0x06)
            {
              continue;
            }
          // 
          // Save the device inside the PCI table.
          // 
          pci_table[t].bus = b;
          pci_table[t].slot = s;
          pci_table[t].vendor_id = pci.vendor_id;
          pci_table[t].device_id = pci.device_id;
          pci_table[t].class_code = pci.class_code;
          pci_table[t].subclass = pci.subclass;
          pci_table[t].prog_if = pci.prog_if;
          pci_table[t].base_io = pci.bar0 & 0xFFFFFFFC;
          pci_table[t].irq = pci.interrupt_line;
          // 
          k_printf ("[%s]: %04x:%04x io=%04x irq=%i\n",
                    __func__, pci_table[t].vendor_id,
                    pci_table[t].device_id,
                    pci_table[t].base_io, pci_table[t].irq);
          // 
          // Next PCI table row.
          // 
          t++;
        }
    }
}
