#include <sys/os32.h>
#include <kernel/driver/tty.h>
#include <kernel/driver/screen.h>
#include <termios.h>
//----------------------------------------------------------
void
tty_init (void)
{
  int page;     // console page.
  // 
  // Console initialization: console pages correspond
  // to the first
  // terminal items.
  // 
  for (page = 0; page < TTYS_CONSOLE; page++)
    {
      tty_table[page].device = DEV_CONSOLE0 + page;
      tty_table[page].pgrp = 0;
      tty_table[page].line[0] = 0;
      tty_table[page].lpr = 0;
      tty_table[page].lpw = 0;
      tty_table[page].status = TTY_INPUT_LINE_EDITING;
      // 
      // Termios default configuration.
      // 
      tty_table[page].attr.c_iflag = BRKINT | ICRNL;
      tty_table[page].attr.c_oflag = 0;
      tty_table[page].attr.c_cflag = 0;
      tty_table[page].attr.c_lflag =
        ECHO | ECHOE | ECHOK | ECHONL | ICANON | ISIG;
      // 
      // VEOF == ASCII EOT
      // 
      tty_table[page].attr.c_cc[VEOF] = 0x04;
      // 
      // VEOL == undefined
      // 
      tty_table[page].attr.c_cc[VEOL] = 0x00;
      // 
      // VERASE == ASCII BS
      // 
      tty_table[page].attr.c_cc[VERASE] = 0x08;
      // 
      // VINTR == ASCII ETX
      // 
      tty_table[page].attr.c_cc[VINTR] = 0x03;
      // 
      // VKILL == undefined
      // 
      tty_table[page].attr.c_cc[VKILL] = 0x00;
      // 
      // VMIN == 0
      // 
      tty_table[page].attr.c_cc[VMIN] = 0x00;
      // 
      // VQUIT == ASCII FS
      // 
      tty_table[page].attr.c_cc[VQUIT] = 0x1C;
      // 
      // VSTART == undefined
      // 
      tty_table[page].attr.c_cc[VSTART] = 0x00;
      // 
      // VSUSP == undefined
      // 
      tty_table[page].attr.c_cc[VSUSP] = 0x00;
      // 
      // VTIME == 0
      // 
      tty_table[page].attr.c_cc[VTIME] = 0x00;
    }
  // 
  // Set video mode.
  // 
  screen_init ();
  // 
  // Select the first console.
  // 
  tty_console (DEV_CONSOLE0);
  // 
  // Nothing else to configure (only consoles are
  // available).
  // 
  return;
}
