#include <kernel/proc.h>
#include <kernel/fs.h>
#include <errno.h>
#include <fcntl.h>
//----------------------------------------------------------
int
fd_dup (pid_t pid, int fdn_old, int fdn_min)
{
  proc_t *ps;
  int fdn_new;
  // 
  // Verify argument.
  // 
  if (fdn_min < 0 || fdn_min >= OPEN_MAX)
    {
      errset (EINVAL);  // Invalid argument.
      return (-1);
    }
  // 
  // Get process.
  // 
  ps = proc_reference (pid);
  // 
  // Verify if `fdn_old' is a valid value.
  // 
  if (fdn_old < 0 ||
      fdn_old >= OPEN_MAX || ps->fd[fdn_old].file == NULL)
    {
      errset (EBADF);   // Bad file descriptor.
      return (-1);
    }
  // 
  // Find the first free slot and duplicate the file
  // descriptor.
  // 
  for (fdn_new = fdn_min; fdn_new < OPEN_MAX; fdn_new++)
    {
      if (ps->fd[fdn_new].file == NULL)
        {
          ps->fd[fdn_new].fl_flags =
            ps->fd[fdn_old].fl_flags;
          ps->fd[fdn_new].fd_flags =
            ps->fd[fdn_old].fd_flags & ~FD_CLOEXEC;
          ps->fd[fdn_new].file = ps->fd[fdn_old].file;
          ps->fd[fdn_new].file->references++;
          return (fdn_new);
        }
    }
  // 
  // No fd slot available.
  // 
  errset (EMFILE);      // Too many open files.
  return (-1);
}
