#include <kernel/proc.h>
#include <errno.h>
#include <fcntl.h>
//----------------------------------------------------------
file_t *
file_stdio_dev_make (dev_t device, mode_t mode, int oflags)
{
  inode_t *inode;
  file_t *file;
  // 
  // Try to allocate a device inode.
  // 
  inode = inode_stdio_dev_make (device, mode);
  if (inode == NULL)
    {
      // 
      // Variable `errno' is already set by
      // `inode_stdio_dev_make()'.
      // 
      errset (errno);
      return (NULL);
    }
  // 
  // Inode allocated: need to allocate the system file 
  // item.
  // 
  file = file_reference (-1);
  if (file == NULL)
    {
      // 
      // Remove the inode and return an error.
      // 
      inode_put (inode);
      errset (ENFILE);  // Too many files open in
      // system.
      return (NULL);
    }
  // 
  // Fill with data the system file item.
  // 
  file->references = 1;
  file->oflags = (oflags & (O_RDONLY | O_WRONLY));
  file->inode = inode;
  // 
  // Return system file pointer.
  // 
  return (file);
}
