#include <kernel/fs.h>
#include <errno.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------
int
inode_dir_empty (inode_t * inode)
{
  off_t start;
  char buffer[SB_MAX_ZONE_SIZE];
  directory_t *dir;
  ssize_t size_read;
  int d;        // Directory buffer index.
  // 
  // Check argument: must be a directory.
  // 
  if (inode == NULL || !S_ISDIR (inode->mode))
    {
      errset (EINVAL);  // Invalid argument.
      return (0);       // false
    }
  // 
  // Read the directory content: if an item is present 
  // (except `.' and
  // `..'), the directory is not empty.
  // 
  for (start = 0;
       start < inode->size; start += inode->sb->blksize)
    {
      size_read =
        inode_file_read (inode, start, buffer,
                         inode->sb->blksize, NULL);
      if (size_read < sizeof (directory_t))
        {
          break;
        }
      // 
      // Scan the directory portion just read.
      // 
      dir = (directory_t *) buffer;
      // 
      for (d = 0; d < size_read;
           d += (sizeof (directory_t)), dir++)
        {
          if (dir->ino != 0 &&
              strncmp (dir->name, ".", NAME_MAX) != 0
              && strncmp (dir->name, "..", NAME_MAX) != 0)
            {
              // 
              // There is an item and the directory is 
              // not empty.
              // 
              return (0);       // false
            }
        }
    }
  // 
  // Nothing was found; good!
  // 
  return (1);   // true
}
