#include <kernel/fs.h>
#include <errno.h>
//----------------------------------------------------------
ssize_t
inode_pipe_read (inode_t * inode, void *buffer,
                 size_t count, int *eof)
{
  unsigned char *buffer_s;
  unsigned char *buffer_d = buffer;
  int i;
  // 
  // The inode pointer must be valid.
  // 
  if (inode == NULL)
    {
      errset (EINVAL);  // Invalid argument.
      return ((ssize_t) - 1);
    }
  // 
  // Check the current pipe direction and see if can
  // be
  // read something.
  // 
  if (inode->pipe_dir)
    {
      // 
      // Write: if indexes are the same, cannot read
      // anything.
      // 
      if (inode->pipe_off_write == inode->pipe_off_read)
        {
          // 
          // Cannot read.
          // 
          if (inode->pipe_ref_write == 0)
            {
              if (eof != NULL)
                {
                  *eof = 1;
                }
            }
          return ((ssize_t) 0);
        }
    }
  else
    {
      // 
      // Read: the pipe is waiting for a read.
      // 
      ;
    }
  // 
  // Might read something. Set the pointer to the
  // source buffer,
  // that is the area used for direct zones, including 
  // first
  // indirect pointers (total: (7+2)*2 = 18 bytes).
  // 
  buffer_s = (void *) &(inode->direct[0]);
  // 
  i = 0;
  // 
  if (inode->pipe_off_read >= inode->pipe_off_write)
    {
      for (; i < count; i++)
        {
          if (inode->pipe_off_read < INODE_PIPE_BUFFER_SIZE)
            {
              buffer_d[i] = buffer_s[inode->pipe_off_read];
              inode->pipe_off_read++;
            }
          else
            {
              inode->pipe_off_read = 0;
              break;
            }
        }
    }
  // 
  if (inode->pipe_off_read < inode->pipe_off_write)
    {
      for (; i < count; i++)
        {
          if (inode->pipe_off_read < inode->pipe_off_write)
            {
              buffer_d[i] = buffer_s[inode->pipe_off_read];
              inode->pipe_off_read++;
            }
          else
            {
              break;
            }
        }
    }
  // 
  // At this point, it is time to set the direction to 
  // write;
  // it doesn't matter if the direction is already set 
  // so.
  // 
  if (inode->pipe_off_read == inode->pipe_off_write)
    {
      inode->pipe_dir = 1;
    }
  // 
  // Ok.
  // 
  return ((ssize_t) i);
}
