#include <kernel/fs.h>
#include <errno.h>
//----------------------------------------------------------
inode_t *
inode_reference (dev_t device, ino_t ino)
{
  int s;        // Slot index.
  sb_t *sb_table = sb_reference (0);
  // 
  // If device is zero, and inode is zero, a reference 
  // to the whole
  // table is returned.
  // 
  if (device == 0 && ino == 0)
    {
      return (inode_table);
    }
  // 
  // If device is ((dev_t) -1) and the inode is
  // ((ino_t) -1), a
  // reference to a free inode slot is returned.
  // 
  if (device == (dev_t) - 1 && ino == ((ino_t) - 1))
    {
      for (s = 0; s < INODE_MAX_SLOTS; s++)
        {
          if (inode_table[s].references == 0)
            {
              return (&inode_table[s]);
            }
        }
      return (NULL);
    }
  // 
  // If device is zero and the inode is 1, a reference 
  // to the root
  // directory inode is returned.
  // 
  if (device == 0 && ino == 1)
    {
      // 
      // The super block table is to be scanned.
      // 
      for (device = 0, s = 0; s < SB_MAX_SLOTS; s++)
        {
          if (sb_table[s].device != 0
              && (sb_table[s].inode_mounted_on->
                  sb_attached->device ==
                  sb_table[s].device))
            {
              device = sb_table[s].device;
              break;
            }
        }
      if (device == 0)
        {
          errset (E_CANNOT_FIND_ROOT_DEVICE);
          return (NULL);
        }
      // 
      // Scan the inode table to find inode 1 and the
      // same device.
      // 
      for (s = 0; s < INODE_MAX_SLOTS; s++)
        {
          if (inode_table[s].sb->device == device &&
              inode_table[s].ino == 1)
            {
              return (&inode_table[s]);
            }
        }
      // 
      // Cannot find a root file system inode.
      // 
      errset (E_CANNOT_FIND_ROOT_INODE);
      return (NULL);
    }
  // 
  // A device and an inode number were selected: find
  // the inode
  // associated to it.
  // 
  for (s = 0; s < INODE_MAX_SLOTS; s++)
    {
      if (inode_table[s].sb->device == device &&
          inode_table[s].ino == ino)
        {
          return (&inode_table[s]);
        }
    }
  // 
  // The inode was not found.
  // 
  return (NULL);
}
