#include <kernel/fs.h>
#include <errno.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------
int
inode_truncate (inode_t * inode)
{
  unsigned int indirect_zones;
  zno_t zone_table1[INODE_MAX_INDIRECT_ZONES];
  zno_t zone_table2[INODE_MAX_INDIRECT_ZONES];
  unsigned int i;       // Direct index.
  unsigned int i0;      // Single indirect index.
  unsigned int i1;      // Double indirect first
  // index.
  unsigned int i2;      // Double indirect second
  // index.
  int status;   // `zone_read()' return value.
  // 
  // Check argument.
  // 
  if (inode == NULL)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  // Calculate how many indirect zone numbers are
  // stored inside
  // a zone: it depends on the zone size.
  // 
  indirect_zones = inode->sb->blksize / 2;
  // 
  // Scan and release direct zones. Errors are
  // ignored.
  // 
  for (i = 0; i < 7; i++)
    {
      zone_free (inode->sb, inode->direct[i]);
      inode->direct[i] = 0;
    }
  // 
  // Scan single indirect zones, if present.
  // 
  if (inode->blkcnt > 7 && inode->indirect1 != 0)
    {
      // 
      // There is a single indirect table to load.
      // Errors are
      // almost ignored.
      // 
      status =
        zone_read (inode->sb, inode->indirect1,
                   zone_table1);
      if (status == 0)
        {
          // 
          // Scan the table and remove zones.
          // 
          for (i0 = 0; i0 < indirect_zones; i0++)
            {
              zone_free (inode->sb, zone_table1[i0]);
            }
        }
      // 
      // Remove indirect table too.
      // 
      zone_free (inode->sb, inode->indirect1);
      // 
      // Clear single indirect reference inside the
      // inode.
      // 
      inode->indirect1 = 0;
    }
  // 
  // Scan double indirect zones, if present.
  // 
  if (inode->blkcnt > (7 + indirect_zones)
      && inode->indirect2 != 0)
    {
      // 
      // There is a double indirect table to load.
      // Errors are
      // almost ignored.
      // 
      status =
        zone_read (inode->sb, inode->indirect2,
                   zone_table1);
      if (status == 0)
        {
          // 
          // Scan the table and get second level
          // indirection.
          // 
          for (i1 = 0; i1 < indirect_zones; i1++)
            {
              if ((inode->blkcnt
                   >
                   (7 + indirect_zones +
                    indirect_zones * i1))
                  && zone_table1[i1] != 0)
                {
                  // 
                  // There is a second level table to
                  // load.
                  // 
                  status =
                    zone_read (inode->sb,
                               zone_table1[i1],
                               zone_table2);
                  if (status == 0)
                    {
                      // 
                      // Release zones.
                      // 
                      for (i2 = 0;
                           i2 < indirect_zones &&
                           (inode->blkcnt >
                            (7 + indirect_zones +
                             indirect_zones * i1 +
                             i2)); i2++)
                        {
                          zone_free (inode->sb,
                                     zone_table2[i2]);
                        }
                      // 
                      // Remove second level indirect
                      // table.
                      // 
                      zone_free (inode->sb,
                                 zone_table1[i1]);
                    }
                }
            }
          // 
          // Remove first level indirect table.
          // 
          zone_free (inode->sb, inode->indirect2);
        }
      // 
      // Clear single indirect reference inside the
      // inode.
      // 
      inode->indirect2 = 0;
    }
  // 
  // Update super block and inode data.
  // 
  sb_save (inode->sb);
  inode->size = 0;
  inode->changed = 1;
  inode_save (inode);
  // 
  // Successful return.
  // 
  return (0);
}
