#include <kernel/fs.h>
#include <kernel/dev.h>
#include <errno.h>
//----------------------------------------------------------
zno_t
zone_alloc (sb_t * sb)
{
  int m;        // Index inside the inode map.
  int map_element;
  int map_bit;
  int map_mask;
  zno_t zone;
  char buffer[SB_MAX_ZONE_SIZE];
  int status;
  // 
  // Verify if write is allowed.
  // 
  if (sb->options & MOUNT_RO)
    {
      errset (EROFS);   // Read-only file system.
      return ((zno_t) 0);
    }
  // 
  // Write allowed: scan the zone map, to find a free
  // zone. If a free zone can be found, allocate it
  // inside the map.
  // Index `m' starts from one, because the first bit
  // of the map is reserved for a `zero' data-zone
  // that does not exist: the second bit is for the
  // real first data-zone.
  // 
  for (zone = 0, m = 1; m < (SB_MAP_ZONE_SIZE * 16); m++)
    {
      map_element = m / 16;
      map_bit = m % 16;
      map_mask = 1 << map_bit;
      if (!(sb->map_zone[map_element] & map_mask))
        {
          // 
          // Found a free place: set the map.
          // 
          sb->map_zone[map_element] |= map_mask;
          sb->changed = 1;
          // 
          // The *second* bit inside the map is for
          // the first data zone (the zone after the
          // inode table inside the file system),
          // because the first is for a special
          // `zero' data zone, not really used.
          // 
          zone = sb->first_data_zone + m - 1;   // Found 
          // a free
          // zone.
          // 
          // If the zone is outside the disk size, let 
          // set the map bit, but reset variable
          // `zone'.
          // 
          if (zone >= sb->zones)
            {
              zone = 0;
            }
          else
            {
              break;
            }
        }
    }
  if (zone == 0)
    {
      errset (ENOSPC);  // No space left on device.
      return ((zno_t) 0);
    }
  // 
  // A free zone was found and the map was modified
  // inside
  // the super block in memory. The zone must be
  // cleared.
  // 
  status = zone_write (sb, zone, buffer);
  if (status != 0)
    {
      zone_free (sb, zone);
      return ((zno_t) 0);
    }
  // 
  // A zone was allocated: return the number.
  // 
  return (zone);
}
