#include <kernel/fs.h>
#include <kernel/dev.h>
#include <errno.h>
//----------------------------------------------------------
int
zone_free (sb_t * sb, zno_t zone)
{
  int map_element;
  int map_bit;
  int map_mask;
  // 
  // Check arguments.
  // 
  if (sb == NULL || zone < sb->first_data_zone)
    {
      errset (EINVAL);  // Invalid argument.
      return (-1);
    }
  // 
  // Calculate the map element, the map bit and the
  // map mask.
  // 
  // The *second* bit inside the map is for the first
  // data-zone
  // (the zone after the inode table inside the file
  // system),
  // because the first is for a special `zero'
  // data-zone, not
  // really used.
  // 
  map_element = (zone - sb->first_data_zone + 1) / 16;
  map_bit = (zone - sb->first_data_zone + 1) % 16;
  map_mask = 1 << map_bit;
  // 
  // Verify if the requested zone is inside the file
  // system area.
  // 
  if (zone >= sb->zones)
    {
      errset (EINVAL);  // Invalid argument.
      return (-1);
    }
  // 
  // Free the zone and return.
  // 
  if (sb->map_zone[map_element] & map_mask)
    {
      sb->map_zone[map_element] &= ~map_mask;
      sb->changed = 1;
      return (0);
    }
  else
    {
      errset (EUNKNOWN);        // The zone was
      // already free.
      return (-1);
    }
}
