#include <kernel/ibm_i386.h>
#include <kernel/multiboot.h>
//----------------------------------------------------------
void
gdt (void)
{
  uint32_t blocks;      // Total available memory
  // blocks.
  // 
  // Calculate memory blocks.
  // 
  blocks = (multiboot.mem_upper * 1024) / 4096;
  // 
  // Set data for GDTR register.
  // 
  gdt_register.limit = (sizeof (gdt_table) - 1);
  gdt_register.base = (uint32_t) & gdt_table[0];
  // 
  // Reset items inside `gdt_table[]'.
  // gdt_table[0] must be null and is not to be
  // used.
  // 
  int i;
  for (i = 0; i < GDT_ITEMS; i++)
    {
      gdt_segment (i, 0, 0, 0, 0, 0);
    }
  // 
  // gdt_table[1] is for kernel code.
  // It covers all the available memory, with DPL 0.
  // The selector is 8+0 = 0x08+0.
  // 
  gdt_segment (1, 0, blocks, 1, 1, 0);
  // 
  // gdt_table[2] is for kernel data, including stack
  // (the stack
  // MUST be in the same address space of data).
  // It covers all the available memory, with DPL 0.
  // The selector is 16+0 = 0x10+0.
  // 
  gdt_segment (2, 0, blocks, 1, 0, 0);
  // 
  // Load the GDT table.
  // 
  gdt_load (&gdt_register);
}
