#include <kernel/ibm_i386.h>
#include <errno.h>
//----------------------------------------------------------
int
gdt_segment (int segment,
             uint32_t base,
             uint32_t limit,
             bool present, bool code, unsigned char dpl)
{
  // 
  // Verify if the segment is valid.
  // 
  if ((segment >= ((sizeof (gdt_table)) / 8))
      || (segment < 0))
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  // Limit.
  // 
  gdt_table[segment].limit_a = (limit & 0x0000FFFF);
  gdt_table[segment].limit_b = limit / 0x10000;
  // 
  // Base address.
  // 
  gdt_table[segment].base_a = (base & 0x0000FFFF);
  gdt_table[segment].base_b =
    ((base / 0x10000) & 0x000000FF);
  gdt_table[segment].base_c = (base / 0x1000000);
  // 
  // Attributes.
  // 

  // 
  // Internal update.
  // 
  gdt_table[segment].accessed = 0;
  // 
  // r, w, x.
  // 
  gdt_table[segment].write_execute = 1;
  // 
  // Normal and conforming.
  // 
  gdt_table[segment].expansion_conforming = 0;
  // 
  gdt_table[segment].code_or_data = code;
  gdt_table[segment].code_data_or_system = 1;
  gdt_table[segment].dpl = dpl;
  gdt_table[segment].present = present;
  gdt_table[segment].available = 0;     // 0
  gdt_table[segment].reserved = 0;      // 0
  gdt_table[segment].big = 1;   // 32 bit
  gdt_table[segment].granularity = 1;   // 4 Kibyte
  // 
  return (0);
}
