#include <kernel/lib_k.h>
#include <kernel/lib_s.h>
#include <kernel/proc.h>
#include <time.h>
#include <unistd.h>
//----------------------------------------------------------
int
k_usleep (useconds_t usec)
{
  clock_t time_start;
  clock_t time_now;
  clock_t time_elapsed;
  clock_t time;
  unsigned long long int loops;
  // 
  // Calculate time, in terms of internal clocks
  // 
  if (usec < 10000000)
    {
      time = (usec * CLOCKS_PER_SEC) / 1000000;
    }
  else
    {
      time = (usec / 1000000) * CLOCKS_PER_SEC;
    }
  // 
  // Fix time: if it is zero, it means that it was
  // requested a sleep
  // shorter than the internal clock timer impulse.
  // So, if it is zero,
  // correct to at least a one.
  // 
  if (time == 0 && usec != 0)
    time = 1;
  // 
  // Calculate how many times the following loop
  // should
  // be run to get the requested time.
  // 
  loops = proc_loops_per_clock * time;
  // 
  // Do a wasting time loop.
  // 
  time_elapsed = 0;
  time_start = s_clock ((pid_t) 0);
  // 
  // If the function `s_clock()' can help, it will
  // exit even if the loop is become slow, but the
  // time was correctly accounted and is elapsed.
  // 
  for (; time_elapsed < time && loops > 0; loops--)
    {
      time_now = s_clock ((pid_t) 0);
      time_elapsed = time_now - time_start;
    }
  // 
  // The sleep is always complete.
  // 
  return (0);
}
