#ifndef _KERNEL_LIB_S_H
#define _KERNEL_LIB_S_H  1
//----------------------------------------------------------
#include <sys/types.h>
#include <sys/stat.h>
#include <kernel/fs.h>
#include <sys/os32.h>
#include <stddef.h>
#include <stdint.h>
#include <time.h>
#include <termios.h>
#include <setjmp.h>
//----------------------------------------------------------
void s__exit (pid_t pid, int status);
int s_brk (pid_t pid, void *address);
void *s_sbrk (pid_t pid, intptr_t increment);
pid_t s_fork (pid_t ppid);
int s_kill (pid_t pid_killer, pid_t pid_target, int sig);
void s_longjmp (pid_t pid, jmp_buf env, int val);
int s_seteuid (pid_t pid, uid_t euid);
int s_setjmp (pid_t pid, jmp_buf env);
int s_setuid (pid_t pid, uid_t uid);
int s_setegid (pid_t pid, gid_t egid);
int s_setgid (pid_t pid, gid_t gid);
pid_t s_wait (pid_t pid, int *status);
sighandler_t s_signal (pid_t pid, int sig,
                       sighandler_t handler,
                       uintptr_t wrapper);
//----------------------------------------------------------
int s_chdir (pid_t pid, const char *path);
int s_chmod (pid_t pid, const char *path, mode_t mode);
int s_chown (pid_t pid, const char *path, uid_t uid,
             gid_t gid);
int s_link (pid_t pid, const char *path_old,
            const char *path_new);
int s_mkdir (pid_t pid, const char *path, mode_t mode);
int s_mknod (pid_t pid, const char *path, mode_t mode,
             dev_t device);
int s_open (pid_t pid, const char *path, int oflags,
            mode_t mode);
int s_stat (pid_t pid, const char *path,
            struct stat *buffer);
int s_unlink (pid_t pid, const char *path);
//
int s_pipe (pid_t pid, int pipefd[2]);
//
int s_close (pid_t pid, int fdn);
int s_dup (pid_t pid, int fdn_old);
int s_dup2 (pid_t pid, int fdn_old, int fdn_new);
int s_fchmod (pid_t pid, int fdn, mode_t mode);
int s_fchown (pid_t pid, int fdn, uid_t uid, gid_t gid);
int s_fcntl (pid_t pid, int fdn, int cmd, int arg);
int s_fstat (pid_t pid, int fdn, struct stat *buffer);
off_t s_lseek (pid_t pid, int fdn, off_t offset,
               int whence);
ssize_t s_read (pid_t pid, int fdn, void *buffer,
                size_t count);
ssize_t s_write (pid_t pid, int fdn,
                 const void *buffer, size_t count);
//
int s_mount (pid_t pid, const char *path_dev,
             const char *path_mnt, int options);
int s_umount (pid_t pid, const char *path_mnt);
//----------------------------------------------------------
int s_accept (pid_t pid, int sfdn,
              struct sockaddr *addr, socklen_t * addrlen);
int s_bind (pid_t pid, int sfdn,
            const struct sockaddr *addr, socklen_t addrlen);
int s_connect (pid_t pid, int sfdn,
               const struct sockaddr *addr,
               socklen_t addrlen);
int s_listen (pid_t pid, int sfdn, int backlog);
int s_socket (pid_t pid, int family, int type,
              int protocol);
ssize_t s_send (pid_t pid, int sfdn,
                const void *buffer, size_t size, int flags);
ssize_t s_recv (pid_t pid, int sfdn, void *buffer,
                size_t size, int flags);
ssize_t s_recvfrom (pid_t pid, int sfdn, void *buffer,
                    size_t length, int flags,
                    struct sockaddr *addrfrom,
                    socklen_t * addrlen);
//
int s_ipconfig (pid_t pid, int n, h_addr_t address, int m);
int s_routeadd (pid_t pid, h_addr_t destination, int m,
                h_addr_t router, int device);
int s_routedel (pid_t pid, h_addr_t destination, int m);
//----------------------------------------------------------
int s_tcgetattr (pid_t pid, int fdn,
                 struct termios *termios_p);
int s_tcsetattr (pid_t pid, int fdn, int action,
                 struct termios *termios_p);
//----------------------------------------------------------
clock_t s_clock (pid_t pid);    // [p]
time_t s_time (pid_t pid, time_t * timer);      // [p]
int s_stime (pid_t pid, time_t * timer);
//
// [p] The PID information, here, is not used. The
//     argument is required only for syntax coherence
//     with other functions, that do
//     system calls, inside the kernel.
//
//----------------------------------------------------------
#endif
