#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <arpa/inet.h>
//----------------------------------------------------------
int
s_accept (pid_t pid, int sfdn, struct sockaddr *addr,
          socklen_t * addrlen)
{
  fd_t *sfd;
  fd_t *sfd2;
  int sfdn2;
  struct sockaddr_in sa;
  int q;
  // 
  // Get file descriptor and verify that it is a
  // socket.
  // 
  sfd = fd_reference (pid, &sfdn);
  if (sfd == NULL || sfd->file == NULL)
    {
      errset (EBADF);   // Bad file descriptor.
      return (-1);
    }
  if (sfd->file->sock == NULL)
    {
      errset (ENOTSOCK);        // Not a socket.
      return (-1);
    }
  // 
  // The socket must be a stream.
  // 
  if (sfd->file->sock->type != SOCK_STREAM)
    {
      errset (EOPNOTSUPP);
      return (-1);
    }
  // 
  // The socket must be a TCP stream: no other stream
  // types are
  // available.
  // 
  if (sfd->file->sock->protocol != IPPROTO_TCP)
    {
      errset (EOPNOTSUPP);
      return (-1);
    }
  // 
  // The socket itself must be listening.
  // 
  if (sfd->file->sock->tcp.conn != TCP_LISTEN)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  // The connections must be related to the same PID.
  // 
  if (sfd->file->sock->tcp.listen_pid != pid)
    {
      k_printf
        ("[%s] the connection pid %i is not the same "
         "as the current pid %i!\n", __func__,
         sfd->file->sock->tcp.listen_pid, pid);
      errset (EUNKNOWN);
      return (-1);
    }
  // 
  // Ok: find a connected socket from the queue.
  // 
  for (q = 0; q < sfd->file->sock->tcp.listen_max; q++)
    {
      if (sfd->file->sock->tcp.listen_queue[q] != -1)
        {
          // 
          // Found.
          // 
          break;
        }
    }
  if (q >= sfd->file->sock->tcp.listen_max)
    {
      // 
      // At the moment, there is no new connection.
      // 
      errset (EAGAIN);  // Try again.
      return (-1);
    }
  // 
  // Descriptor found: check it.
  // 
  sfdn2 = sfd->file->sock->tcp.listen_queue[q];
  sfd2 = fd_reference (pid, &sfdn2);
  if (sfd2 == NULL || sfd->file == NULL)
    {
      k_printf
        ("[%s] the connected file descriptor %i "
         "for process %i is not a file descriptor!",
         __func__, sfdn2, pid);
      errset (EBADF);   // Bad file descriptor.
      return (-1);
    }
  if (sfd2->file->sock == NULL)
    {
      k_printf
        ("[%s] the connected file descriptor %i "
         "for process %i is not a socket!", __func__,
         sfdn2, pid);
      errset (ENOTSOCK);        // Not a socket.
      return (-1);
    }
  // 
  // Ok.
  // 
  if (addrlen != NULL && addr != NULL && *addrlen > 0)
    {
      sa.sin_family = AF_INET;
      sa.sin_port = htons (sfd2->file->sock->rport);
      sa.sin_addr.s_addr = htonl (sfd2->file->sock->raddr);
      // 
      memcpy (addr, &sa,
              min (sizeof (sa), (size_t) * addrlen));
      *addrlen = sizeof (sa);
    }
  // 
  // Reset the queue element and return.
  // 
  sfd->file->sock->tcp.listen_queue[q] = -1;
  return (sfdn2);
}
