#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------
int
s_chdir (pid_t pid, const char *path)
{
  proc_t *ps;
  inode_t *inode_directory;
  int status;
  char path_directory[PATH_MAX];
  // 
  // Get process.
  // 
  ps = proc_reference (pid);
  // 
  // The full directory path is needed.
  // 
  status = path_full (path, ps->path_cwd, path_directory);
  if (status < 0)
    {
      return (-1);
    }
  // 
  // Try to load the new directory inode.
  // 
  inode_directory = path_inode (pid, path_directory);
  if (inode_directory == NULL)
    {
      // 
      // Cannot access the directory: it does not
      // exists or
      // permissions are not sufficient. Variable
      // `errno' is set by
      // function `inode_directory()'.
      // 
      errset (errno);
      return (-1);
    }
  // 
  // Inode loaded: release the old directory and set
  // the new one.
  // 
  inode_put (ps->inode_cwd);
  // 
  ps->inode_cwd = inode_directory;
  strncpy (ps->path_cwd, path_directory, PATH_MAX);
  // 
  // Return.
  // 
  return (0);
}
