#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------
int
s_chmod (pid_t pid, const char *path, mode_t mode)
{
  proc_t *ps;
  inode_t *inode;
  // 
  // Get process.
  // 
  ps = proc_reference (pid);
  // 
  // Try to load the file inode.
  // 
  inode = path_inode (pid, path);
  if (inode == NULL)
    {
      // 
      // Cannot access the file: it does not exists or 
      // permissions are
      // not sufficient. Variable `errno' is set by
      // function
      // `inode_directory()'.
      // 
      return (-1);
    }
  // 
  // Verify to be root or to be the owner.
  // 
  if (ps->euid != 0 && ps->euid != inode->uid)
    {
      errset (EACCES);  // Permission denied.
      return (-1);
    }
  // 
  // Update the mode: the file type is kept and the
  // rest is taken form the parameter `mode'.
  // 
  inode->mode = (S_IFMT & inode->mode) | (~S_IFMT & mode);
  // 
  // Save and release the inode.
  // 
  inode->changed = 1;
  inode_save (inode);
  inode_put (inode);
  // 
  // Return.
  // 
  return (0);
}
