#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <sys/stat.h>
#include <errno.h>
//----------------------------------------------------------
int
s_fchmod (pid_t pid, int fdn, mode_t mode)
{
  proc_t *ps;
  inode_t *inode;
  // 
  // Get process.
  // 
  ps = proc_reference (pid);
  // 
  // Verify if the file descriptor is valid.
  // 
  if (ps->fd[fdn].file == NULL)
    {
      errset (EBADF);   // Bad file descriptor.
      return (-1);
    }
  // 
  // Reach the inode.
  // 
  inode = ps->fd[fdn].file->inode;
  // 
  // If the Inode does not exist, exit with error.
  // 
  if (inode == NULL)
    {
      errset (ENOENT);
      return (-1);
    }
  // 
  // Verify to be the owner, or at least to be UID ==
  // 0.
  // 
  if (ps->euid != inode->uid && ps->euid != 0)
    {
      errset (EACCES);  // Permission denied.
      return (-1);
    }
  // 
  // Update the mode: the file type is kept and the
  // rest is taken form the parameter `mode'.
  // 
  inode->mode = (S_IFMT & inode->mode) | (~S_IFMT & mode);
  // 
  // Save the inode.
  // 
  inode->changed = 1;
  inode_save (inode);
  // 
  // Return.
  // 
  return (0);
}
