#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <errno.h>
//----------------------------------------------------------
int
s_kill (pid_t pid_killer, pid_t pid_target, int sig)
{
  uid_t euid = proc_table[pid_killer].euid;
  uid_t uid = proc_table[pid_killer].uid;
  pid_t pgrp = proc_table[pid_killer].pgrp;
  int p;        // Index inside the process table.
  // 
  if (pid_target < -1)
    {
      errset (ESRCH);
      return (-1);
    }
  else if (pid_target == -1)
    {
      if (sig == 0)
        {
          return (0);
        }
      if (euid == 0)
        {
          // 
          // Because `pid_target' is qual to `-1' and
          // the effective
          // user identity is `0', then, all
          // processes,
          // except the kernel and init, will receive
          // the signal.
          // 
          // The following scan starts from 2, to
          // preserve the
          // kernel and init processes.
          // 
          for (p = 2; p < PROCESS_MAX; p++)
            {
              if (proc_table[p].status != PROC_EMPTY
                  && proc_table[p].status != PROC_ZOMBIE)
                {
                  proc_sig_on (p, sig);
                }
            }
        }
      else
        {
          // 
          // Because `pid_target' is qual to `-1', but 
          // the effective
          // user identity is not `0', then, all
          // processes owned
          // by the same effective user identity, will 
          // receive the
          // signal.
          // 
          // The following scan starts from 1, to
          // preserve the
          // kernel process.
          // 
          for (p = 1; p < PROCESS_MAX; p++)
            {
              if (proc_table[p].status != PROC_EMPTY
                  && proc_table[p].status !=
                  PROC_ZOMBIE && proc_table[p].uid == euid)
                {
                  proc_sig_on (p, sig);
                }
            }
        }
      return (0);
    }
  else if (pid_target == 0)
    {
      if (sig == 0)
        {
          return (0);
        }
      // 
      // The following scan starts from 1, to preserve 
      // the
      // kernel process.
      // 
      for (p = 1; p < PROCESS_MAX; p++)
        {
          if (proc_table[p].status != PROC_EMPTY
              && proc_table[p].status != PROC_ZOMBIE
              && proc_table[p].pgrp == pgrp)
            {
              proc_sig_on (p, sig);
            }
        }
      return (0);
    }
  else if (pid_target >= PROCESS_MAX)
    {
      errset (ESRCH);
      return (-1);
    }
  else  // (pid_target > 0)
    {
      if (proc_table[pid_target].status == PROC_EMPTY
          || proc_table[pid_target].status == PROC_ZOMBIE)
        {
          errset (ESRCH);
          return (-1);
        }
      else if (uid == proc_table[pid_target].uid ||
               uid == proc_table[pid_target].suid ||
               euid == proc_table[pid_target].uid ||
               euid == proc_table[pid_target].suid
               || euid == 0)
        {
          if (sig == 0)
            {
              return (0);
            }
          else
            {
              proc_sig_on (pid_target, sig);
              return (0);
            }
        }
      else
        {
          errset (EPERM);
          return (-1);
        }
    }
}
