#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
//----------------------------------------------------------
int
s_link (pid_t pid, const char *path_old,
        const char *path_new)
{
  proc_t *ps;
  inode_t *inode_old;
  inode_t *inode_new;
  char path_new_full[PATH_MAX];
  // 
  // Get process.
  // 
  ps = proc_reference (pid);
  // 
  // Try to get the old path inode.
  // 
  inode_old = path_inode (pid, path_old);
  if (inode_old == NULL)
    {
      // 
      // Cannot get the inode: `errno' is already set
      // by
      // `path_inode()'.
      // 
      errset (errno);
      return (-1);
    }
  // 
  // The inode is available and checks are done:
  // arrange to get a
  // packed full path name and then the destination
  // directory path.
  // 
  path_full (path_new, ps->path_cwd, path_new_full);
  // 
  // 
  // 
  inode_new =
    path_inode_link (pid, path_new_full, inode_old,
                     (mode_t) 0);
  if (inode_new == NULL)
    {
      inode_put (inode_old);
      return (-1);
    }
  if (inode_new != inode_old)
    {
      inode_put (inode_new);
      inode_put (inode_old);
      errset (EUNKNOWN);        // Unknown error.
      return (-1);
    }
  // 
  // Inode data is already updated by
  // `path_inode_link()': just put
  // it and return. Please note that only one is put,
  // because it is
  // just the same of the other.
  // 
  inode_put (inode_new);
  return (0);
}
