#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <fcntl.h>
//----------------------------------------------------------
int
s_pipe (pid_t pid, int pipefd[2])
{
  file_t *file;
  fd_t *fd_read;
  fd_t *fd_write;
  int fdn_read;
  int fdn_write;
  // 
  // Allocate the file inside the file table and the
  // inode inside
  // the inode table.
  // 
  file = file_pipe_make ();
  if (file == NULL)
    {
      errset (errno);
      return (-1);
    }
  // 
  // Prepare file descriptor for read.
  // 
  fdn_read = -1;
  fd_read = fd_reference (pid, &fdn_read);
  if (fd_read == NULL)
    {
      // 
      // Cannot allocate the file descriptor: remove
      // the item from
      // file table and put the relative inode.
      // 
      file->references = 0;
      file->oflags = 0;
      inode_put (file->inode);
      file->inode = NULL;
      // 
      // Return an error.
      // 
      errset (EMFILE);  // Too many open files.
      return (-1);
    }
  // 
  // File descriptor allocated: put some data inside
  // the
  // file descriptor item and increment the pipe
  // references
  // for read.
  // 
  fd_read->fl_flags = O_RDONLY;
  fd_read->fd_flags = 0;
  fd_read->file = file;
  fd_read->file->offset = 0;
  fd_read->file->inode->pipe_ref_read++;
  // 
  // Prepare file descriptor for write.
  // 
  fdn_write = -1;
  fd_write = fd_reference (pid, &fdn_write);
  if (fd_write == NULL)
    {
      // 
      // Cannot allocate the file descriptor: remove
      // the item from
      // file table and put the relative inode.
      // 
      file->references = 0;
      file->oflags = 0;
      inode_put (file->inode);
      file->inode = NULL;
      // 
      // Remove file descriptor for read.
      // 
      fd_read->file->inode->pipe_ref_read--;
      fd_read->fl_flags = 0;
      fd_read->fd_flags = 0;
      fd_read->file = NULL;
      // 
      // Return an error.
      // 
      errset (EMFILE);  // Too many open files.
      return (-1);
    }
  // 
  // File descriptor allocated: put some data inside
  // the
  // file descriptor item.
  // 
  fd_write->fl_flags = O_WRONLY;
  fd_write->fd_flags = 0;
  fd_write->file = file;
  fd_write->file->offset = 0;
  fd_write->file->inode->pipe_ref_write++;
  // 
  // Save file descriptor numbers inside the
  // `pipefd[]' array.
  // 
  pipefd[0] = fdn_read;
  pipefd[1] = fdn_write;
  // 
  // Ok.
  // 
  return (0);
}
