#include <arpa/inet.h>
#include <sys/os32.h>
#include <kernel/net/route.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <netinet/in.h>
#include <kernel/proc.h>
//----------------------------------------------------------
// This syscall is present only inside os32.
//----------------------------------------------------------
int
s_routeadd (pid_t pid, in_addr_t dest, int m,
            in_addr_t router, int device)
{
  int r;
  h_addr_t netmask;
  h_addr_t network;
  // 
  // Must be a privileged process.
  // 
  if (proc_table[pid].euid != 0)
    {
      errset (EPERM);
      return (-1);
    }
  // 
  // 
  // 
  if (m > 32 || m < 0)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  // Calculate the netmask.
  // 
  netmask = ip_mask (m);
  // 
  // Fix the destination address, with the mask.
  // 
  network = ntohl (dest) & netmask;
  // 
  // Check if there is already. If there is: update
  // it.
  // 
  for (r = 0; r < ROUTE_MAX_ROUTES; r++)
    {
      if (network == route_table[r].network
          && m == route_table[r].m)
        {
          // 
          // Update.
          // 
          route_table[r].router = ntohl (router);
          route_table[r].netmask = netmask;
          route_table[r].interface = device;
          return (0);
        }
    }
  // 
  // The item is new. Find an empty place.
  // 
  for (r = 0; r < ROUTE_MAX_ROUTES; r++)
    {
      if (route_table[r].network == 0xFFFFFFFF)
        {
          // 
          // Empty.
          // 
          route_table[r].network = network;
          route_table[r].netmask = netmask;
          route_table[r].m = m;
          route_table[r].router = ntohl (router);
          route_table[r].interface = device;
          // 
          route_sort ();
          // 
          return (0);
        }
    }
  // 
  // No free space found.
  // 
  errset (ENOMEM);
  return (-1);
}
