#include <kernel/lib_s.h>
#include <kernel/proc.h>
#include <errno.h>
//----------------------------------------------------------
sighandler_t
s_signal (pid_t pid, int sig, sighandler_t handler,
          uintptr_t wrapper)
{
  unsigned long int flag = 1L << (sig - 1);
  sighandler_t previous;
  // 
  if (sig <= 0)
    {
      errset (EINVAL);
      return (SIG_ERR);
    }
  // 
  if (proc_table[pid].sig_ignore & flag)
    {
      previous = SIG_IGN;
    }
  else if (proc_table[pid].sig_handler[sig] !=
           (uintptr_t) NULL)
    {
      previous =
        (sighandler_t) proc_table[pid].sig_handler[sig];
    }
  else
    {
      previous = SIG_DFL;
    }
  // 
  if (handler == SIG_DFL)
    {
      // 
      // Enable signal.
      // 
      proc_table[pid].sig_ignore &= ~flag;
      // 
      return (previous);
    }
  else if (handler == SIG_IGN)
    {
      // 
      // Disable signal.
      // 
      proc_table[pid].sig_ignore |= flag;
      // 
      return (previous);
    }
  else
    {
      // 
      // Enable signal, store the handler address and
      // the
      // handler-wrapper.
      // 
      proc_table[pid].sig_ignore &= ~flag;
      proc_table[pid].sig_handler[sig] =
        (uintptr_t) handler;
      proc_table[pid].sig_handler_wrapper = wrapper;
      // 
      return (previous);
    }
}
