#include <kernel/fs.h>
#include <errno.h>
#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <termios.h>
#include <sys/types.h>
//----------------------------------------------------------
// The following are masks of the implemented
// attributes.
//
#define MASK_C_IFLAG (BRKINT|ICRNL|IGNBRK|IGNCR)
#define MASK_C_OFLAG 0
#define MASK_C_CFLAG 0
#define MASK_C_LFLAG \
    (ECHO|ECHOE|ECHOK|ECHONL|ICANON|ISIG)
//----------------------------------------------------------
int
s_tcsetattr (pid_t pid, int fdn, int action,
             struct termios *termios_p)
{
  file_t *file;
  inode_t *inode;
  dev_t device;
  int t;        // `tty_table[]' subscript.
  int c;        // `c_cc[]' subscript.
  // 
  file = proc_table[pid].fd[fdn].file;
  // 
  if (file == NULL)
    {
      errset (EBADF);
      return (-1);
    }
  // 
  inode = proc_table[pid].fd[fdn].file->inode;
  // 
  if (inode == NULL)
    {
      errset (EBADF);
      return (-1);
    }
  // 
  if (!S_ISCHR (inode->mode))
    {
      errset (ENOTTY);
      return (-1);
    }
  // 
  device = inode->direct[0];
  // 
  if (major (device) != DEV_CONSOLE_MAJOR)
    {
      errset (ENOTTY);
      return (-1);
    }
  // 
  t = minor (device);
  if (t >= TTYS_TOTAL)
    {
      errset (ENOTTY);
      return (-1);
    }
  // 
  // The parameter `actions' is silently ignored: only 
  // immediate update will take place.
  // 
  // The function will not notice if at least a
  // successful attribute change is done, so,
  // after this point, the return value is
  // always zero.
  // 
  tty_table[t].attr.c_iflag =
    (termios_p->c_iflag & MASK_C_IFLAG);
  tty_table[t].attr.c_oflag =
    (termios_p->c_oflag & MASK_C_OFLAG);
  tty_table[t].attr.c_cflag =
    (termios_p->c_cflag & MASK_C_CFLAG);
  tty_table[t].attr.c_lflag =
    (termios_p->c_lflag & MASK_C_LFLAG);
  for (c = 0; c < NCCS; c++)
    {
      // 
      // Should be done some check here?
      // 
      tty_table[t].attr.c_cc[c] = termios_p->c_cc[c];
    }
  // 
  // Ok.
  // 
  return (0);
}
