#include <kernel/proc.h>
#include <kernel/lib_s.h>
#include <errno.h>
//----------------------------------------------------------
pid_t
s_wait (pid_t pid, int *status)
{
  pid_t parent = pid;
  pid_t child;
  int child_available = 0;
  // 
  // Find a dead child process.
  // 
  for (child = 1; child < PROCESS_MAX; child++)
    {
      if (proc_table[child].ppid == parent)
        {
          child_available = 1;  // Child found!
          if (proc_table[child].status == PROC_ZOMBIE)
            {
              break;    // It is dead!
            }
        }
    }
  // 
  // If the index `child' is a valid process number,
  // a dead child was found.
  // 
  if (child < PROCESS_MAX)
    {
      *status = proc_table[child].ret;
      proc_available (child);
      return (child);
    }
  else
    {
      if (child_available)
        {
          // 
          // There are child, but all alive.
          // 
          // Go to sleep.
          // 
          proc_table[parent].status = PROC_SLEEPING;
          proc_table[parent].wakeup_events =
            WAKEUP_EVENT_SIGNAL;
          proc_table[parent].wakeup_signal = SIGCHLD;
          return ((pid_t) 0);
        }
      else
        {
          // 
          // There are no child at all.
          // 
          errset (ECHILD);
          return ((pid_t) - 1);
        }
    }
}
