#include <kernel/memory.h>
#include <kernel/ibm_i386.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
static int mb_block_set1 (int block);
//----------------------------------------------------------
ssize_t
mb_alloc (addr_t address, size_t size)
{
  unsigned int bstart;
  unsigned int bsize;
  unsigned int bend;
  unsigned int i;
  ssize_t allocated = 0;
  addr_t block_address;
  // 
  if (size == 0)
    {
      // 
      // Zero means nothing.
      // 
      allocated = 0;
      return (allocated);
    }
  // 
  // Show what was requested.
  // 
  if (DEBUG)
    {
      k_printf ("%s(%i, %zi)", __func__,
                (unsigned int) address, size);
    }
  // 
  // Calculate starting block of memory.
  // 
  bstart = address / MEM_BLOCK_SIZE;
  // 
  // Calculating size in term of blocks.
  // 
  if (size % MEM_BLOCK_SIZE)
    {
      bsize = size / MEM_BLOCK_SIZE + 1;
    }
  else
    {
      bsize = size / MEM_BLOCK_SIZE;
    }
  // 
  // Calculate the block number after the
  // end of the requested memory area.
  // 
  bend = bstart + bsize;
  // 
  // Scan the memory map and allocate.
  // 
  for (i = bstart; i < bend; i++)
    {
      if (mb_block_set1 (i))
        {
          allocated += MEM_BLOCK_SIZE;
        }
      else
        {
          block_address = i;
          block_address *= MEM_BLOCK_SIZE;
          k_printf
            ("[%s] Kernel alert: mem block "
             "%x, at address "
             "%x, already allocated!\n", __FILE__, i,
             (unsigned int) block_address);
          break;
        }
    }
  // 
  // 
  // 
  return (allocated);
}

//----------------------------------------------------------
static int
mb_block_set1 (int block)
{
  int i = block / 32;
  int j = block % 32;
  uint32_t mask = 0x80000000 >> j;
  if (mb_table[i] & mask)
    {
      return (0);       // The block is already set to 
      // 1 inside the map!
    }
  else
    {
      mb_table[i] = mb_table[i] | mask;
      return (1);
    }
}
