#include <kernel/memory.h>
#include <kernel/ibm_i386.h>
#include <kernel/lib_k.h>
#include <sys/os32.h>
#include <errno.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
static int mb_block_status (int block);
//----------------------------------------------------------
addr_t
mb_alloc_size (size_t size)
{
  unsigned int bsize;
  unsigned int i;
  unsigned int j;
  unsigned int found = 0;
  addr_t alloc_addr;
  ssize_t alloc_size;
  // 
  // 
  // 
  if (size == 0)
    {
      errset (EINVAL);
      return ((addr_t) 0);
    }
  // 
  // Show what was requested.
  // 
  if (DEBUG)
    {
      k_printf ("%s(%zi)", __func__, size);
    }
  // 
  // Calculate block size.
  // 
  if (size % MEM_BLOCK_SIZE)
    {
      bsize = size / MEM_BLOCK_SIZE + 1;
    }
  else
    {
      bsize = size / MEM_BLOCK_SIZE;
    }
  // 
  // Scan for a contiguous space in memory.
  // 
  for (i = 0; i < (mb_max - bsize) && !found; i++)
    {
      for (j = 0; j < bsize; j++)
        {
          found = !mb_block_status (i + j);
          if (!found)
            {
              i += j;
              break;
            }
        }
    }
  // 
  // If the space was found, allocate it.
  // 
  if (found && (j == bsize))
    {
      alloc_addr = i - 1;
      alloc_addr *= MEM_BLOCK_SIZE;
      alloc_size = bsize * MEM_BLOCK_SIZE;
      alloc_size =
        mb_alloc (alloc_addr, (size_t) alloc_size);
      // 
      if (alloc_size <= 0)
        {
          errset (ENOMEM);
          return ((addr_t) 0);
        }
      else if (alloc_size < size)
        {
          mb_free (alloc_addr, (size_t) alloc_size);
          errset (ENOMEM);
          return ((addr_t) 0);
        }
      else
        {
          // 
          // Clean memory before return.
          // 
          mb_clean (alloc_addr, (size_t) alloc_size);
          // 
          // 
          // 
          return (alloc_addr);
        }
    }
  else
    {
      errset (ENOMEM);
      return ((addr_t) 0);
    }
}

//----------------------------------------------------------
static int
mb_block_status (int block)
{
  int i = block / 32;
  int j = block % 32;
  uint32_t mask = 0x80000000 >> j;
  return ((int) (mb_table[i] & mask));
}
