#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <arpa/inet.h>
#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/icmp.h>
#include <netinet/icmp.h>
#include <netinet/udp.h>
#include <kernel/lib_k.h>
#include <errno.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
int
icmp_rx (int i)
{
  icmp_packet_t *icmp;
  size_t size;
  struct iphdr *ip;
  struct udphdr *ports;
  h_addr_t dest = 0;
  h_port_t port = 0;
  int s;        // Socket table index.
  // 
  // 
  // 
  if ((i >= IP_MAX_PACKETS) || i < 0)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  // Find the ICMP packet start, and the ICMP packet
  // size (the IP
  // packet size - the IP header size).
  // 
  icmp = (icmp_packet_t *) ip_table[i].pdu4;
  size = ntohs (ip_table[i].packet.header.tot_len)
    - (ip_table[i].packet.header.ihl * 4);
  // 
  // This function is used by the kernel, to do
  // something automatically,
  // when some ICMP packets arrive. The kernel does
  // not remove the
  // serviced packets, but must remember what it
  // already have seen.
  // If it finds that the packet clock time stamp is
  // the same as
  // the value saved for the kernel, there is nothing
  // more to be done.
  // 
  if (ip_table[i].kernel_serviced == ip_table[i].clock)
    {
      return (0);
    }
  // 
  // 
  // 
  if (icmp->header.type == ICMP_ECHO)   // ECHO
    // REQUEST
    {
      size -= sizeof (struct icmphdr);
      // 
      // Reply: please note that source and
      // destination IP addresses
      // are now inverted.
      // 
      icmp_tx_echo (ntohl
                    (ip_table[i].packet.header.daddr),
                    ntohl (ip_table[i].packet.header.saddr),
                    ICMP_ECHOREPLY, 0,
                    ntohs (icmp->header.un.echo.id),
                    ntohs (icmp->header.un.echo.sequence),
                    icmp->data, size);
      // 
      // ICMP echo request are resolved internally,
      // but the packet
      // might be read from the RAW socket too. So it
      // is not removed
      // from the ip_table[].
      // 
      ip_table[i].kernel_serviced = ip_table[i].clock;
    }
  else if (icmp->header.type == ICMP_DEST_UNREACH)
    {
      // 
      // UNREACHABLE
      // 
      // 
      // The code (subtype) is not checked.
      // After the ICMP header there is a copy of the
      // original IP
      // header.
      // 
      ip = (struct iphdr *)
        (((uint8_t *) icmp) + sizeof (struct icmphdr));
      // 
      dest = ntohl (ip->daddr);
      // 
      // If the IP protocol is TCP or UDP, there are
      // also ports.
      // 
      if (ip->protocol == IPPROTO_TCP
          || ip->protocol == IPPROTO_UDP)
        {
          // 
          // After the IP header copy, there is the
          // TCP or UDP header
          // copy. To be able to get the ports, the
          // UDP and TCP headers
          // are the same.
          // 
          ports =
            (struct udphdr *) (((uint8_t *) ip) +
                               (ip->ihl * 4));
          // 
          port = ntohs (ports->dest);
          // 
        }
      // 
      // Set corresponding sockets unreachable.
      // 
      for (s = 0; s < SOCK_MAX_SLOTS; s++)
        {
          if (sock_table[s].active)
            {
              if (sock_table[s].raddr == dest)
                {
                  if (port == 0
                      || port == sock_table[s].rport)
                    {
                      if (icmp->header.code ==
                          ICMP_NET_UNREACH)
                        {
                          sock_table[s].unreach_net = 1;
                        }
                      else if (icmp->header.code ==
                               ICMP_HOST_UNREACH)
                        {
                          sock_table[s].unreach_host = 1;
                        }
                      else if (icmp->header.code ==
                               ICMP_PROT_UNREACH)
                        {
                          sock_table[s].unreach_prot = 1;
                        }
                      else if (icmp->header.code ==
                               ICMP_PORT_UNREACH)
                        {
                          sock_table[s].unreach_port = 1;
                        }
                      else
                        {
                          sock_table[s].unreach_host = 1;
                        }
                    }
                }
            }
        }
    }
  else if (icmp->header.type == ICMP_ECHOREPLY)
    {
      // 
      // ECHO REPLY
      // 
      if (DEBUG)
        {
          k_printf ("received an echo reply\n");
        }
    }
  else
    {
      // 
      // No other ICMP type is handled at the moment,
      // but keep
      // the packet, because might be read from a RAW
      // ICMP socket.
      // 
      ;
    }
  // 
  return (0);
}
