#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <arpa/inet.h>
#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/icmp.h>
#include <errno.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
int
icmp_tx_unreachable (h_addr_t src, h_addr_t dst,
                     int type, int code,
                     uint8_t * data, size_t size)
{
  icmp_packet_t icmp;
  // 
  // Check the data size and reduce if necessary.
  // 
  size = min (size, ICMP_MAX_DATA_SIZE);
  // 
  if (size < 8)
    {
      // 
      // The ICMP destination unreachable data must
      // contain
      // at least 64 bits of the original packet.
      // 
      errset (EINVAL);
      return (-1);
    }
  // 
  // Check the type: must be ICMP_DEST_UNREACH.
  // 
  if (type != ICMP_DEST_UNREACH)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  // Must reset the «destination unreachable»
  // header.
  // 
  memset (&icmp.header.un, 0, (size_t) 4);
  // 
  // Copy the data inside the ICMP packet.
  // 
  memcpy (icmp.data, data, size);
  // 
  // Send to the lower network level.
  // 
  return (icmp_tx
          (src, dst, type, code, (void *) &icmp,
           (sizeof (struct icmphdr) + size)));
}
