#ifndef _KERNEL_NET_IP_H
#define _KERNEL_NET_IP_H    1
//----------------------------------------------------------
#include <stdint.h>
#include <sys/types.h>
#include <kernel/net.h>
#include <netinet/ip.h>
//----------------------------------------------------------
#define IP_VERSION          4
#define IP_TTL              64
//----------------------------------------------------------
#define IP_MAX_PACKETS      64
//----------------------------------------------------------
//
// IP packet, for transmission (no options here).
// It is just the same as `ip_header_t', with the
// `data[]' member addition.
//
typedef struct
{
  struct iphdr header;
  uint8_t data[NET_IP_MAX_DATA_SIZE];
} __attribute__ ((packed)) ip_packet_t;
//
// IP table for IP packets.
//
typedef struct
{
  clock_t clock;        // [1]
  clock_t kernel_serviced;      // [2]
  uint8_t *pdu4;
  union
  {
    uint8_t octet[NET_IP_MAX_PACKET_SIZE];
    struct iphdr header;
  } packet;
} ip_t;
//
// [1] This is the arrival packet clock time stamp.
//     When a new packet is to be saved inside the
//     ip_table[], the older packet is replaced, even
//     if it was not used. No packets are removed,
//     because they might be read from a RAW socket,
//     for some reason.
//
// [2] This is only for kernel usage, when a connection
//     is established by the kernel, without a socket.
//     The member kernel_serviced is used to remember
//     to have see a certain packet and that the kernel
//     does not have to try to service it again. For
//     example, when an ECHO REQUEST packet arrive, the
//     kernel answares with an ECHO REPLY, but does not
//     remove the packet that might be read from a true
//     RAW socket. So, the kernel writes inside
//     kernel_serviced the same value found inside
//     clock, so that it know that it was already read.
//
//
// External IP table data.
//
extern ip_t ip_table[IP_MAX_PACKETS];
//----------------------------------------------------------
uint16_t ip_checksum (uint16_t * data1, size_t size1,
                      uint16_t * data2, size_t size2);
int ip_rx (int n, int f);
ip_t *ip_reference (void);
ssize_t ip_header (h_addr_t src, h_addr_t dst,
                   uint16_t id, uint8_t ttl,
                   uint8_t protocol, void *buffer,
                   size_t length);
int ip_tx (h_addr_t src, h_addr_t dst, int protocol,
           const void *buffer, size_t size);
h_addr_t ip_mask (int m);
//----------------------------------------------------------
#endif
