#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netinet/ip.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
ssize_t
ip_header (h_addr_t src, h_addr_t dst, uint16_t id,
           uint8_t ttl, uint8_t protocol, void *buffer,
           size_t length)
{
  struct iphdr header;
  uint16_t checksum;
  // 
  // Check size.
  // 
  if (length < sizeof (struct iphdr))
    {
      errset (EINVAL);
      return ((ssize_t) - 1);
    }
  // 
  // Prepare the header.
  // 
  header.version = IP_VERSION;
  header.ihl = sizeof (struct iphdr) / 4;
  header.tos = 0x00;    // Routine, normal.
  header.tot_len = htons (length);
  header.id = htons (id);
  header.frag_off = htons (0x4000);     // Do not
  // fragment!
  header.ttl = ttl;
  header.protocol = protocol;
  header.check = 0;
  header.saddr = htonl (src);
  header.daddr = htonl (dst);
  // 
  // Fix the length.
  // 
  length = sizeof (struct iphdr);
  // 
  // Now set the header checksum.
  // 
  checksum = ~(ip_checksum ((void *) &header, length,
                            NULL, (size_t) 0));
  header.check = htons (checksum);
  // 
  // Copy the header to the buffer
  // 
  memcpy (buffer, &header, length);
  // 
  // Return size written.
  // 
  return (length);
}
