#include <sys/os32.h>
#include <kernel/net/route.h>
#include <kernel/net/ip.h>
#include <kernel/net/arp.h>
#include <kernel/driver/nic/ne2k.h>
#include <kernel/driver/pci.h>
#include <kernel/ibm_i386.h>
#include <errno.h>
//----------------------------------------------------------
int
net_eth_ip_tx (h_addr_t src, h_addr_t dst,
               const void *packet, size_t size)
{
  net_ethernet_frame_t frame;
  int n;        // NET table index.
  int a;        // ARP table index.
  int i;
  h_addr_t router;
  // 
  // Check for PDU size.
  // 
  if (size > NET_ETHERNET_MAX_PACKET_SIZE)
    {
      errset (E_PDU_TOO_BIG);
      return (-1);
    }
  // 
  // Find the sender interface address.
  // 
  n = net_index_eth (src, NULL, (uintptr_t) 0);
  if (n < 0)
    {
      errset (errno);   // ENODEV.
      return (-1);
    }
  // 
  // Copy the Ethernet source address into the
  // Ethernet frame
  // header.
  // 
  memcpy (frame.header.src, net_table[n].ethernet.mac,
          NET_ETHERNET_ADDRESS_LENGTH);
  // 
  // Find if we need a router.
  // 
  router = route_remote_to_router (dst);
  // 
  if (router != 0 && router != ((h_addr_t) - 1))
    {
      // 
      // We need to find the router destination MAC
      // address.
      // 
      a = arp_index (NULL, router);
      if (a < 0)
        {
          // 
          // There is not the item inside the ARP
          // table. Send a request
          // and return.
          // 
          arp_request (router);
          errset (E_ARP_MISSING);
          return (-1);
        }
    }
  else
    {
      // 
      // The destination is inside the local network.
      // Find the destination Ethernet address.
      // 
      a = arp_index (NULL, dst);
      if (a < 0)
        {
          // 
          // There is not the item inside the ARP
          // table. Send a request
          // and return.
          // 
          arp_request (dst);
          errset (E_ARP_MISSING);
          return (-1);
        }
    }
  // 
  // Copy the Ethernet destination address into the
  // Ethernet frame
  // header: might be the real destination interface,
  // or the
  // router.
  // 
  memcpy (frame.header.dst, arp_table[a].mac,
          NET_ETHERNET_ADDRESS_LENGTH);
  // 
  // Set the frame type.
  // 
  frame.header.type = htons (NET_PROT_IP);
  // 
  // Copy the IP packet.
  // 
  memcpy (&frame.packet, packet, size);
  // 
  // Fill if the size is too little.
  // 
  for (i = size; i < NET_ETHERNET_MIN_PACKET_SIZE; i++)
    {
      frame.packet.octet[i] = 0;
    }
  // 
  size = max (size, NET_ETHERNET_MIN_PACKET_SIZE);
  // 
  // Now, send the Ethernet frame. Index `n' is the
  // network
  // device number.
  // 
  return (net_eth_tx
          (n, &frame, size + NET_ETHERNET_HEADER_SIZE));
}
