#include <kernel/net.h>
#include <kernel/net/route.h>
#include <kernel/net/arp.h>
#include <kernel/lib_s.h>
#include <kernel/proc.h>
#include <kernel/multiboot.h>
#include <stdlib.h>
#include <kernel/lib_k.h>
#include <string.h>
#include <errno.h>
#include <kernel/driver/pci.h>
#include <kernel/driver/nic/ne2k.h>
//----------------------------------------------------------
static void net_eth_init (int start);
//----------------------------------------------------------
void
net_init (void)
{
  int n;        // NET device table index.
  int b;        // Buffer NET device index.
  int i;
  char *net = "net0";
  char *route = "route0";
  char **argument;
  in_addr_t ip_a;
  in_addr_t ip_b;
  int status;
  // 
  // Reset the NET device table.
  // 
  for (n = 0; n < NET_MAX_DEVICES; n++)
    {
      net_table[n].type = NET_DEV_NULL;
    }
  // 
  // Set up the loopback interface.
  // 
  net_table[0].type = NET_DEV_LOOPBACK;
  net_table[0].ip = INADDR_LOOPBACK;
  net_table[0].m = 8;
  // 
  for (b = 0; b < NET_MAX_BUFFERS; b++)
    {
      net_table[0].loopback.buffer[b].clock = 0;
    }
  // 
  // Prepare ARP table
  // 
  arp_init ();
  // 
  // Add Ethernet devices, but starting from the
  // second interface
  // inside the net_table[].
  // 
  net_eth_init (1);
  // 
  // Prepare routes.
  // 
  route_init ();
  route_sort ();
  // 
  // Command line options: counter `i' is scanned like 
  // a character.
  // 
  for (i = '0'; i <= '9'; i++)
    {
      net[3] = i;
      argument = mboot_cmdline_opt (net, ",");
      if (argument != NULL)
        {
          // 
          status = inet_pton (AF_INET, argument[2], &ip_a);
          if (status != 1)
            {
              continue;
            }
          // 
          s_ipconfig ((pid_t) 0, atoi (argument[1]),
                      ip_a, atoi (argument[3]));
        }
    }
  // 
  for (i = '0'; i <= '9'; i++)
    {
      route[5] = i;
      argument = mboot_cmdline_opt (route, ",");
      if (argument != NULL)
        {
          // 
          status = inet_pton (AF_INET, argument[1], &ip_a);
          if (status != 1)
            {
              continue;
            }
          status = inet_pton (AF_INET, argument[3], &ip_b);
          if (status != 1)
            {
              continue;
            }
          // 
          s_routeadd ((pid_t) 0,
                      ip_a, atoi (argument[2]),
                      ip_b, atoi (argument[4]));
        }
    }
  // 
  // 
  // 
  net_print ();
  route_print ();
}

//----------------------------------------------------------
static void
net_eth_init (int start)
{
  int p;        // PCI table index.
  int n;        // NET devices table index.
  int i;
  int j;
  // 
  static const struct
  {
    unsigned short vendor;
    unsigned short device;
  } type_ne2k[] =
  {
    {
    0x10ec, 0x8029},    // RealTek_RTL_8029
    {
    0x1050, 0x0940},    // Winbond_89C940
    {
    0x11f6, 0x1401},    // Compex_RL2000
    {
    0x8e2e, 0x3000},    // KTI_ET32P2
    {
    0x4a14, 0x5000},    // NetVin_NV5000SC
    {
    0x1106, 0x0926},    // Via_86C926
    {
    0x10bd, 0x0e34},    // SureCom_NE34
    {
    0x1050, 0x5a5a},    // Winbond_W89C940F
    {
    0x12c3, 0x0058},    // Holtek_HT80232
    {
    0x12c3, 0x5598},    // Holtek_HT80229
    {
    0x8c4a, 0x1980},    // Winbond_89C940_8c4a
  };
  // 
  // 
  // 
  n = start;
  // 
  // Scan the PCI table and find NE2K Ethernet
  // devices.
  // 
  for (p = 0;
       p < PCI_MAX_DEVICES && n < NET_MAX_DEVICES; p++)
    {
      for (i = 0; i < sizeof_array (type_ne2k); i++)
        {
          if (pci_table[p].vendor_id ==
              type_ne2k[i].vendor
              && pci_table[p].device_id ==
              type_ne2k[i].device)
            {
              // 
              // Verify if the NIC is really a NE2K.
              // 
              if (ne2k_check (pci_table[p].base_io) == 0)
                {
                  // 
                  // Reset the NIC and get the
                  // physical address.
                  // 
                  if (ne2k_reset (pci_table[p].base_io,
                                  net_table[n].
                                  ethernet.mac) == 0)
                    {
                      // 
                      // New.
                      // 
                      net_table[n].type = NET_DEV_ETH_NE2K;
                      net_table[n].ethernet.base_io =
                        pci_table[p].base_io;
                      net_table[n].ethernet.irq =
                        pci_table[p].irq;
                      // 
                      for (j = 0; j < NET_MAX_BUFFERS; j++)
                        {
                          net_table[n].ethernet.
                            buffer[j].clock = 0;
                        }
                      // 
                      // Go to next NET table element.
                      // 
                      n++;
                      // 
                      break;
                    }
                }
            }
        }
    }
}
