#include <sys/os32.h>
#include <kernel/net/route.h>
#include <errno.h>
//----------------------------------------------------------
static int part (char *array, size_t size, int a,
                 int z, int (*compare) (void *, void *));
static void sort (char *array, size_t size, int a,
                  int z, int (*compare) (void *, void *));
static void qsort (void *base, size_t nmemb,
                   size_t size, int (*compare) (void *,
                                                void *));
//
static uint8_t swap[sizeof (route_t)];
static int comp (void *a, void *b);
//----------------------------------------------------------
void
route_sort (void)
{
  qsort (route_table, ROUTE_MAX_ROUTES,
         sizeof (route_t), comp);
}

//----------------------------------------------------------
static int
comp (void *a, void *b)
{
  route_t *route_a = a;
  route_t *route_b = b;
  uint8_t m_a = route_a->m;
  uint8_t m_b = route_b->m;
  // 
  if (m_a > m_b)
    return (-1);
  if (m_a < m_b)
    return (1);
  return (0);
}

//----------------------------------------------------------
static void
qsort (void *base, size_t nmemb, size_t size,
       int (*compare) (void *, void *))
{
  if (size <= 1)
    {
      // 
      // There is nothing to sort!
      // 
      return;
    }
  else
    {
      sort ((char *) base, size, 0, (int) (nmemb - 1),
            compare);
    }
}

//----------------------------------------------------------
static void
sort (char *array, size_t size, int a, int z,
      int (*compare) (void *, void *))
{
  int loc;
  // 
  if (z > a)
    {
      loc = part (array, size, a, z, compare);
      if (loc >= 0)
        {
          sort (array, size, a, loc - 1, compare);
          sort (array, size, loc + 1, z, compare);
        }
    }
}

//----------------------------------------------------------
static int
part (char *array, size_t size, int a, int z,
      int (*compare) (void *, void *))
{
  int i;
  int loc;
  // 
  if (z <= a)
    {
      errset (EUNKNOWN);        // Should never
      // happen.
      return (-1);
    }
  // 
  // Index `i' after the first element; index `loc' at 
  // the last
  // position.
  // 
  i = a + 1;
  loc = z;
  // 
  // Loop as long as index `loc' is higher than index
  // `i'.
  // When index `loc' is less or equal to index `i',
  // then, index `loc' is the right position for the
  // first element of the current piece of array.
  // 
  for (;;)
    {
      // 
      // Index `i' goes up...
      // 
      for (; i < loc; i++)
        {
          if (compare
              (&array[i * size], &array[a * size]) > 0)
            {
              break;
            }
        }
      // 
      // Index `loc' gose down...
      // 
      for (;; loc--)
        {
          if (compare
              (&array[loc * size], &array[a * size]) <= 0)
            {
              break;
            }
        }
      // 
      // Swap elements related to index `i' and `loc'.
      // 
      if (loc <= i)
        {
          // 
          // The array is completely scanned.
          // 
          break;
        }
      else
        {
          memcpy (swap, &array[loc * size], size);
          memcpy (&array[loc * size], &array[i * size],
                  size);
          memcpy (&array[i * size], swap, size);
        }
    }
  // 
  // Swap the first element with the one related to
  // the
  // index `loc'.
  // 
  memcpy (swap, &array[loc * size], size);
  memcpy (&array[loc * size], &array[a * size], size);
  memcpy (&array[a * size], swap, size);
  // 
  // Return the index `loc'.
  // 
  return (loc);
}
