#include <stdlib.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <errno.h>
#include <kernel/net.h>
#include <kernel/net/tcp.h>
#include <kernel/fs.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
int
tcp_close (void *sock_item)
{
  sock_t *sock = sock_item;
  // 
  // Is there already a connection?
  // 
  if (sock->tcp.conn == 0 || sock->tcp.conn == TCP_CLOSE)
    {
      // 
      // Done or never opened.
      // 
      return (0);
    }
  else if (sock->tcp.conn == TCP_RESET)
    {
      // 
      // Change to closed.
      // 
      if (DEBUG)
        {
          k_printf ("[%s] TCP_CLOSE\n", __func__);
        }
      sock->tcp.conn = TCP_CLOSE;
      return (0);
    }
  else if (sock->tcp.conn == TCP_TIME_WAIT)
    {
      // 
      // Assume that the time is elapsed.
      // 
      sock->tcp.conn = TCP_CLOSE;
      if (DEBUG)
        {
          k_printf ("[%s] TCP_CLOSE\n", __func__);
        }
      sock->tcp.conn = TCP_CLOSE;
      return (0);
    }
  else if (sock->tcp.conn == TCP_FIN_WAIT1)
    {
      errset (EALREADY);
      return (-1);
    }
  else if (sock->tcp.conn == TCP_FIN_WAIT2)
    {
      errset (EALREADY);
      return (-1);
    }
  else if (sock->tcp.conn == TCP_ESTABLISHED)
    {
      sock->tcp.can_send = 1;
      sock->tcp.lsq[++sock->tcp.lsqi] = sock->tcp.lsq_ack;
      sock->tcp.send_flags = TCP_FLAG_FIN | TCP_FLAG_ACK;
      tcp_tx_sock (sock);
      sock->tcp.conn = TCP_FIN_WAIT1;
      if (DEBUG)
        k_printf ("[%s] TCP_FIN_WAIT1\n", __func__);
      errset (EINPROGRESS);
      return (-1);
    }
  else if (sock->tcp.conn == TCP_CLOSE_WAIT)
    {
      sock->tcp.can_send = 1;
      sock->tcp.lsq[++sock->tcp.lsqi] = sock->tcp.lsq_ack;
      sock->tcp.send_flags = TCP_FLAG_FIN | TCP_FLAG_ACK;
      tcp_tx_sock (sock);
      sock->tcp.conn = TCP_LAST_ACK;
      if (DEBUG)
        k_printf ("[%s] TCP_LAST_ACK\n", __func__);
      errset (EINPROGRESS);
      return (-1);
    }
  else
    {
      sock->tcp.can_send = 1;
      sock->tcp.lsq[++sock->tcp.lsqi] = sock->tcp.lsq_ack;
      sock->tcp.send_flags = TCP_FLAG_FIN | TCP_FLAG_ACK;
      tcp_tx_sock (sock);
      sock->tcp.conn = TCP_CLOSE;
      if (DEBUG)
        k_printf ("[%s] TCP_CLOSE\n", __func__);
      return (0);
    }
}
