#include <stdlib.h>
#include <netinet/ip.h>
#include <netinet/tcp.h>
#include <errno.h>
#include <kernel/net.h>
#include <kernel/net/tcp.h>
#include <kernel/fs.h>
#include <kernel/lib_k.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
int
tcp_connect (void *sock_item)
{
  sock_t *sock = sock_item;
  unsigned int lseq;
  // 
  // Is there already a connection?
  // 
  if (sock->tcp.conn == 0 || sock->tcp.conn == TCP_CLOSE)
    {
      // 
      // There isn't.
      // 
      memset (sock->tcp.lsq, 0x00, sizeof (sock->tcp.lsq));
      memset (sock->tcp.rsq, 0x00, sizeof (sock->tcp.rsq));
      srand ((unsigned int) s_clock ((pid_t) 0));
      lseq = rand ();
      sock->tcp.lsq_ack = lseq + 1;
      sock->tcp.lsq[++sock->tcp.lsqi] = lseq;
      // 
      sock->tcp.can_send = 1;
      sock->tcp.send_size = 0;
      sock->tcp.send_flags = TCP_FLAG_SYN;
      tcp_tx_sock (sock);
      // 
      sock->tcp.conn = TCP_SYN_SENT;
      // 
      // The operation has begun and will take some
      // time.
      // 
      errset (EINPROGRESS);
      return (-1);
    }
  else if (sock->tcp.conn == TCP_RESET)
    {
      // 
      // Cannot connect: the socket status is reset to 
      // closed, to let
      // the process retry, if it is really willing to 
      // do it.
      // 
      sock->tcp.conn = TCP_CLOSE;
      errset (ECONNREFUSED);
      return (-1);
    }
  else if (sock->tcp.conn == TCP_SYN_SENT
           || sock->tcp.conn == TCP_SYN_RECV)
    {
      // 
      // Already in progress.
      // 
      errset (EALREADY);
      return (-1);
    }
  else if (sock->tcp.conn == TCP_ESTABLISHED)
    {
      // 
      // Connection established.
      // 
      return (0);
    }
  else
    {
      // 
      // Cannot reconnect.
      // 
      errset (EISCONN);
      return (-1);
    }
}
