#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/route.h>
#include <kernel/net/tcp.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
int
tcp_tx_raw (h_port_t sport, h_port_t dport,
            uint32_t seq, uint32_t ack_seq, int flags,
            h_addr_t saddr, h_addr_t daddr,
            const void *buffer, size_t size)
{
  tcp_packet_t packet;
  tcp_pseudo_header_t pseudo;
  uint16_t checksum;
  size_t tcp_size = size + sizeof (struct tcphdr);
  // 
  // Verify to have the source address: it is
  // necessary here for
  // the checksum calculation.
  // 
  if (saddr == 0)
    {
      // 
      // Default source address: get the source
      // address from the routing
      // table, based on the destination.
      // 
      saddr = route_remote_to_local (daddr);
      if (saddr == ((h_addr_t) - 1))
        {
          errset (errno);
          return (-1);
        }
    }
  // 
  // Prepare the TCP packet.
  // 
  memset (&packet.header, 0, sizeof (struct tcphdr));
  // 
  packet.header.source = htons (sport);
  packet.header.dest = htons (dport);
  packet.header.seq = htonl (seq);
  packet.header.ack_seq = htonl (ack_seq);
  packet.header.doff = (sizeof (struct tcphdr) / 4);
  if (flags & TCP_FLAG_ACK)
    packet.header.ack = 1;
  if (flags & TCP_FLAG_PSH)
    packet.header.psh = 1;
  if (flags & TCP_FLAG_RST)
    packet.header.rst = 1;
  if (flags & TCP_FLAG_SYN)
    packet.header.syn = 1;
  if (flags & TCP_FLAG_FIN)
    packet.header.fin = 1;
  if (flags & TCP_FLAG_RST)
    {
      packet.header.window = htons (0);
    }
  else
    {
      // 
      // Minimal window.
      // 
      packet.header.window = htons (TCP_MSS);
    }
  packet.header.check = 0;
  // 
  memcpy (packet.data, buffer, size);
  // 
  // Prepare the pseudo header.
  // 
  pseudo.saddr = htonl (saddr);
  pseudo.daddr = htonl (daddr);
  pseudo.zero = 0;
  pseudo.protocol = IPPROTO_TCP;
  pseudo.length = htons (tcp_size);
  // 
  // Now set the header checksum.
  // 
  checksum = ~(ip_checksum ((void *) &packet, tcp_size,
                            (void *) &pseudo,
                            sizeof (tcp_pseudo_header_t)));
  if (checksum == 0)
    {
      checksum = 0xFFFF;
    }
  packet.header.check = htons (checksum);
  // 
  // Send to the lower network level.
  // 
  return (ip_tx
          (saddr, daddr, (int) IPPROTO_TCP, &packet,
           tcp_size));
}
