#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/route.h>
#include <kernel/net/tcp.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <kernel/fs.h>
#include <errno.h>
#include <stdlib.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
int
tcp_tx_rst (void *ip_packet)
{
  struct iphdr *iphdr = ip_packet;
  struct tcphdr *tcphdr;
  uint32_t seq;
  uint32_t ack_seq;
  tcp_packet_t packet;
  tcp_pseudo_header_t pseudo;
  uint16_t checksum;
  // 
  if (ip_packet == NULL)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  iphdr = ip_packet;
  tcphdr = (struct tcphdr *)
    &(((uint8_t *) ip_packet)[iphdr->ihl * 4]);
  // 
  if (iphdr->saddr == 0 || iphdr->daddr == 0)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  if (tcphdr->source == 0 || tcphdr->dest == 0)
    {
      errset (EINVAL);
      return (-1);
    }
  // 
  // If the bad TCP packet has a ACK sequence, we
  // replay with
  // the same sequence (the one that the other side
  // expects).
  // 
  if (tcphdr->ack)
    {
      seq = ntohl (tcphdr->ack_seq);
    }
  else
    {
      seq = rand ();
    }
  // 
  // Our reset ACK has the same sequence received.
  // 
  ack_seq = ntohl (tcphdr->seq);
  // 
  // Prepare the TCP packet.
  // 
  memset (&packet.header, 0, sizeof (struct tcphdr));
  // 
  packet.header.source = tcphdr->dest;
  packet.header.dest = tcphdr->source;
  packet.header.seq = htonl (seq);
  packet.header.ack_seq = htonl (ack_seq);
  packet.header.doff = (sizeof (struct tcphdr) / 4);
  packet.header.ack = 1;
  packet.header.rst = 1;
  packet.header.window = 0;
  packet.header.check = 0;
  // 
  // Prepare the pseudo header.
  // 
  pseudo.saddr = iphdr->saddr;
  pseudo.daddr = iphdr->daddr;
  pseudo.zero = 0;
  pseudo.protocol = IPPROTO_TCP;
  pseudo.length = htons (sizeof (struct tcphdr));
  // 
  // Now set the header checksum.
  // 
  checksum =
    ~(ip_checksum
      ((void *) &packet, sizeof (struct tcphdr),
       (void *) &pseudo, sizeof (tcp_pseudo_header_t)));
  if (checksum == 0)
    {
      checksum = 0xFFFF;
    }
  packet.header.check = htons (checksum);
  // 
  // Send to the lower network level.
  // 
  if (DEBUG)
    {
      tcp_show (ntohl (iphdr->saddr),
                ntohl (iphdr->daddr),
                (struct tcphdr *) &packet);
    }
  return (ip_tx
          (ntohl (iphdr->saddr), ntohl (iphdr->daddr),
           IPPROTO_TCP, &packet, sizeof (struct tcphdr)));
}
