#include <kernel/net.h>
#include <kernel/net/ip.h>
#include <kernel/net/route.h>
#include <kernel/net/udp.h>
#include <sys/os32.h>
#include <kernel/lib_k.h>
#include <errno.h>
#include <arpa/inet.h>
#include <netinet/udp.h>
//----------------------------------------------------------
#define DEBUG 0
//----------------------------------------------------------
int
udp_tx (h_port_t sport, h_port_t dport, h_addr_t saddr,
        h_addr_t daddr, const void *buffer, size_t size)
{
  udp_packet_t packet;
  udp_pseudo_header_t pseudo;
  uint16_t checksum;
  size_t udp_size = size + sizeof (struct udphdr);
  // 
  // Verify to have the source address: it is
  // necessary here for
  // the checksum calculation.
  // 
  if (saddr == 0)
    {
      // 
      // Default source address: get the source
      // address from the routing
      // table, based on the destination.
      // 
      saddr = route_remote_to_local (daddr);
      if (saddr == ((h_addr_t) - 1))
        {
          errset (errno);
          return (-1);
        }
    }
  // 
  // Prepare the UDP packet.
  // 
  packet.header.source = htons (sport);
  packet.header.dest = htons (dport);
  packet.header.len = htons (udp_size);
  packet.header.check = 0;
  memcpy (packet.data, buffer, size);
  // 
  // Prepare the pseudo header.
  // 
  pseudo.saddr = htonl (saddr);
  pseudo.daddr = htonl (daddr);
  pseudo.zero = 0;
  pseudo.protocol = IPPROTO_UDP;
  pseudo.length = htons (udp_size);
  // 
  // Now set the header checksum.
  // 
  checksum = ~(ip_checksum ((void *) &packet, udp_size,
                            (void *) &pseudo,
                            sizeof (udp_pseudo_header_t)));
  if (checksum == 0)
    {
      checksum = 0xFFFF;
    }
  packet.header.check = htons (checksum);
  // 
  // Send to the lower network level.
  // 
  return (ip_tx
          (saddr, daddr, (int) IPPROTO_UDP, &packet,
           udp_size));
}
