#include <kernel/ibm_i386.h>
#include <kernel/proc.h>
#include <kernel/lib_k.h>
#include <kernel/lib_s.h>
#include <string.h>
#include <kernel/multiboot.h>
#include <kernel/dm.h>
//----------------------------------------------------------
extern uint32_t _k_start;
extern uint32_t _k_end;
extern uint32_t _k_text_end;
extern uint32_t _k_data_end;
extern uint32_t _k_bss_end;
extern uint32_t _k_stack_top;
extern uint32_t _k_stack_bottom;
//----------------------------------------------------------
void
proc_init (void)
{
  pid_t pid;
  int fdn;      // File descriptor index;
  inode_t *inode;
  sb_t *sb;
  clock_t time_start;
  clock_t time_now;
  clock_t time_elapsed;
  unsigned long long int count;
  uintptr_t stack_top = (uintptr_t) & _k_stack_top;
  uintptr_t stack_bottom = (uintptr_t) & _k_stack_bottom;
  int sig;
  // 
  // Set up the GDT table.
  // 
  gdt ();
  // 
  // Set up timer.
  // 
  proc_timer_init (CLOCKS_PER_SEC);
  // 
  // Disable external interrupt.
  // 
  cli ();
  // 
  // Set up the IDT table.
  // 
  idt ();
  // 
  // Set all memory reference to some invalid data.
  // 
  for (pid = 0; pid < PROCESS_MAX; pid++)
    {
      proc_available (pid);
    }
  // 
  // Set up the process table with the kernel.
  // 
  proc_table[0].ppid = 0;
  proc_table[0].pgrp = 0;
  proc_table[0].uid = 0;
  proc_table[0].euid = 0;
  proc_table[0].suid = 0;
  proc_table[0].gid = 0;
  proc_table[0].egid = 0;
  proc_table[0].sgid = 0;
  proc_table[0].device_tty = DEV_UNDEFINED;
  proc_table[0].sig_status = 0;
  proc_table[0].sig_ignore = 0;
  proc_table[0].usage = 0;
  proc_table[0].status = PROC_RUNNING;
  proc_table[0].wakeup_events = 0;
  proc_table[0].wakeup_signal = 0;
  proc_table[0].wakeup_timer = 0;
  proc_table[0].wakeup_inode = NULL;
  proc_table[0].address_text = 0;                     // [1]
  proc_table[0].domain_text = (size_t) (&_k_end);     // [2]
  proc_table[0].address_data = 0;                     // [2]
  proc_table[0].domain_data = (size_t) 0;             // [2]
  proc_table[0].domain_stack =
    (size_t) (stack_bottom - stack_top);
  proc_table[0].extra_data = (size_t) 0;
  proc_table[0].sp = 0; // [3]
  proc_table[0].ret = 0;
  proc_table[0].umask = 0022;   // Default umask.
  strncpy (proc_table[0].path_cwd, "/", PATH_MAX);
  strncpy (proc_table[0].name, "os32 kernel", PATH_MAX);
  // 
  // [1] The kernel text starts at 0x100000, that is,
  // 1 Mibyte,
  // but the code expect to start at that address,
  // just like
  // the space from address 0 to 0xFFFFF is anyway
  // part
  // of the kernel. If the kernel is forked, as it
  // happens
  // when the `init' process is to be run, it is
  // necessary
  // to consider part of the kernel all the addresses
  // starting
  // from zero.
  // 
  // [2] The kernel is a unique block, where text and
  // data live
  // together.
  // 
  // [3] The saved stack pointer location will be set
  // at the
  // next interrupt, or system call. That is why the
  // kernel
  // must send a null system call at the beginning of
  // its
  // work.
  // ---------------------------------------------------
  // 
  // Ensure to have a terminated string.
  // 
  proc_table[0].name[PATH_MAX - 1] = 0;
  // 
  // Reset file descriptors.
  // 
  for (fdn = 0; fdn < OPEN_MAX; fdn++)
    {
      proc_table[0].fd[fdn].fl_flags = 0;
      proc_table[0].fd[fdn].fd_flags = 0;
      proc_table[0].fd[fdn].file = NULL;
    }
  // 
  // Reset `sig_handler[]'.
  // 
  for (sig = 0; sig < MAX_SIGNALS; sig++)
    {
      proc_table[0].sig_handler[sig] = (uintptr_t) NULL;
    }
  // 
  // Allocate memory for the kernel.
  // 
  // The BIOS data area (BDA) and extra BIOS at the
  // bottom
  // of 640 Kibyte, is already, formally, included
  // inside the
  // kernel.
  // 
  // The allocation for data has no effect here,
  // because it is
  // the same as the text.
  // 
  mb_alloc (proc_table[0].address_text,
            proc_table[0].domain_text);
  mb_alloc (proc_table[0].address_data,
            proc_table[0].domain_data);
  // 
  // Enable and disable hardware interrupts (IRQ).
  // 
  irq_on (0);   // timer.
  irq_on (1);   // keyboard
  irq_on (2);   // PIC2: keep it ON! [4]
  irq_on (3);   // 
  irq_on (4);   // 
  irq_on (5);   // 
  irq_on (6);   // 
  irq_on (7);   // 
  irq_on (8);   // 
  irq_on (9);   // 
  irq_on (10);  // 
  irq_on (11);  // 
  irq_on (12);  // 
  irq_on (13);  // 
  irq_on (14);  // 
  irq_on (15);  // 
  // 
  // [4] IRQ 2 must be let working, because inside the 
  // file
  // `kernel/ibm_i386/isr_s', the IRQ 2 is used to
  // reset
  // properly PIC 1, so that, the IRQ from 8 to 15
  // will
  // reset only PIC 2.
  // The problem is that any PIC must be reset exactly
  // once, otherwise the system will lock.
  // 
  // 
  // External interrupts activation.
  // 
  sti ();
  // 
  // Calculate how many times can be executed the
  // following loop. This data will be used by
  // `k_sleep()', if the clock pulse does not work
  // for some reason.
  // 
  time_elapsed = 0;
  count = 0;
  time_start = s_clock ((pid_t) 0);
  for (; time_elapsed < 10; count++)
    {
      time_now = s_clock ((pid_t) 0);
      time_elapsed = time_now - time_start;
    }
  proc_loops_per_clock = count / 10;
  // 
  // Set up data-memory devices:
  // it works only after enabling interrupts.
  // 
  dm_init ();
  // 
  // Mount root file system.
  // 
  inode = NULL;
  sb = sb_mount (DEV_DM02, &inode, MOUNT_DEFAULT);
  if (sb == NULL || inode == NULL)
    {
      k_perror
        ("Kernel panic: cannot mount " "root file system:");
      k_exit ();
    }
  // 
  // Add the inode to the process table item for the
  // kernel.
  // 
  proc_table[0].inode_cwd = inode;      // Root fs
  // inode.
}
