#include <kernel/proc.h>
#include <stdint.h>
#include <kernel/lib_k.h>
#include <kernel/ibm_i386.h>
#include <stdint.h>
//----------------------------------------------------------
void
proc_timer_init (clock_t freq)
{
  int input_freq = 1193180;
  // 
  // La frequenza di riferimento è 1,19318 MHz, la
  // quale va
  // divisa per la frequenza che si intende avere
  // effettivamente.
  // 
  int divisor = input_freq / freq;
  // 
  // Il risultato deve essere un valore intero
  // maggiore di zero
  // e inferiore di UINT16_MAX, altrimenti è stata
  // chiesta una
  // frequenza troppo elevata o troppo bassa.
  // 
  if (divisor == 0 || divisor > UINT16_MAX)
    {
      k_printf
        ("[%s] ERROR: IRQ 0 frequency wrong: %i Hz!\n"
         "[%s]        The min allowed frequency \n"
         "[%s]        is 18.22 Hz.\n",
         "[%s]        The max allowed frequency \n"
         "[%s]        is 1.19 MHz.\n",
         __func__, freq, __func__, __func__, __func__,
         __func__);
      return;
    }
  // 
  // Il valore che si ottiene, ovvero il «divisore», 
  // va
  // comunicato al PIT (programmable interval timer),
  // spezzandolo in due parti.
  // 
  out_8 ((uint32_t) 0x43, (uint32_t) 0x36);
  // 
  // Lower byte.
  // 
  out_8 ((uint32_t) 0x40, (uint32_t) (divisor & 0x0F));
  // 
  // Higher byte.
  // 
  out_8 ((uint32_t) 0x40, (uint32_t) (divisor / 0x10));
}
