#ifndef _NETINET_TCP_H
#define _NETINET_TCP_H  1
//----------------------------------------------------------
// GNU C compatible UDP header.
//----------------------------------------------------------
#include <sys/types.h>
//----------------------------------------------------------
struct tcphdr
{
  uint16_t source;
  uint16_t dest;
  uint32_t seq;
  uint32_t ack_seq;
  uint16_t res1:4,
    doff:4,
    fin:1, syn:1, rst:1, psh:1, ack:1, urg:1, res2:2;
  uint16_t window;
  uint16_t check;
  uint16_t urg_ptr;
};
//----------------------------------------------------------
// ATTENZIONE: per dare un significato allo stato di
// una connessione, occorre distinguere in che modo si
// trova inizialmente il socket:
// attivo o passivo (passivo quando rimane in ascolto
// per una connessione).
//
enum
{
  TCP_LISTEN = 1,       // waiting a connection
  // request
  TCP_SYN_SENT, // SYN was sent, waiting from the
  // response SYN
  TCP_SYN_RECV, // SYN received, waiting for ACK
  TCP_ESTABLISHED,      // SYN sent, SYN received and
  // ACK sent
  TCP_FIN_WAIT1,        // local close, FIN sent,
  // waiting ACK or FIN
  TCP_FIN_WAIT2,        // FIN sent, ACK received,
  // waiting FIN
  TCP_CLOSE_WAIT,       // FIN received, ACK sent,
  // waiting local close
  TCP_CLOSING,  // FIN sent, FIN received, ACK sent,
  // waiting ACK
  TCP_LAST_ACK, // FIN received, ACK and FIN sent,
  // waiting ACK
  TCP_TIME_WAIT,        // after TCP_LAST_ACK, wait a
  // little and remove
  TCP_CLOSE,    // connection removed
  TCP_RESET     // connection reset (not standard)
};

#define TCPOPT_EOL		0
#define TCPOPT_NOP		1
#define TCPOPT_MAXSEG		2
#define TCPOLEN_MAXSEG		4
#define TCPOPT_WINDOW		3
#define TCPOLEN_WINDOW		3
#define TCPOPT_SACK_PERMITTED	4
#define TCPOLEN_SACK_PERMITTED	2
#define TCPOPT_SACK		5
#define TCPOPT_TIMESTAMP	8
#define TCPOLEN_TIMESTAMP	10
//----------------------------------------------------------
//
// TCP max segment size: IP_MSS - IP header size.
// Suppose to have a max IP header of 56 bytes,
// TCP_MSS == 520.
//
#define TCP_MSS         520
//----------------------------------------------------------
// LA STRUTTURA SEGUENTE È DA VALUTARE, forse conviene
// fare una tabella a parte per le connessioni TCP.
//
struct tcp_info
{
  uint8_t tcpi_state;
  uint8_t tcpi_ca_state;
  uint8_t tcpi_retransmits;
  uint8_t tcpi_probes;
  uint8_t tcpi_backoff;
  uint8_t tcpi_options;
  uint8_t tcpi_snd_wscale:4, tcpi_rcv_wscale:4;

  uint32_t tcpi_rto;
  uint32_t tcpi_ato;
  uint32_t tcpi_snd_mss;
  uint32_t tcpi_rcv_mss;

  uint32_t tcpi_unacked;
  uint32_t tcpi_sacked;
  uint32_t tcpi_lost;
  uint32_t tcpi_retrans;
  uint32_t tcpi_fackets;

  /* Times. */
  uint32_t tcpi_last_data_sent;

  /* Not remembered, sorry.  */
  uint32_t tcpi_last_ack_sent;

  uint32_t tcpi_last_data_recv;
  uint32_t tcpi_last_ack_recv;

  /* Metrics. */
  uint32_t tcpi_pmtu;
  uint32_t tcpi_rcv_ssthresh;
  uint32_t tcpi_rtt;
  uint32_t tcpi_rttvar;
  uint32_t tcpi_snd_ssthresh;
  uint32_t tcpi_snd_cwnd;
  uint32_t tcpi_advmss;
  uint32_t tcpi_reordering;

  uint32_t tcpi_rcv_rtt;
  uint32_t tcpi_rcv_space;

  uint32_t tcpi_total_retrans;
};


//----------------------------------------------------------
#endif
