#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <errno.h>
#include <stddef.h>
//----------------------------------------------------------
//
// This function is not standard and is able to do the
// work of both `vfscanf()' and `vsscanf()'. 
//
//----------------------------------------------------------
#define WIDTH_MAX       64
//----------------------------------------------------------
static intmax_t strtointmax (const char *restrict
                             string,
                             const char **restrict
                             endptr, int base,
                             size_t max_width);
static int ass_or_eof (int consumed, int assigned);
//----------------------------------------------------------
int
vfsscanf (FILE * restrict fp, const char *string,
          const char *restrict format, va_list ap)
{
  int f = 0;    // Format index.
  char buffer[BUFSIZ];
  const char *input = string;   // Default.
  const char *start = input;    // Default.
  const char *restrict next = NULL;
  int scanned = 0;
  // 
  bool stream = 0;
  bool flag_star = 0;
  bool specifier = 0;
  bool specifier_flags = 0;
  bool specifier_width = 0;
  bool specifier_type = 0;
  bool inverted = 0;
  // 
  char *ptr_char;
  signed char *ptr_schar;
  unsigned char *ptr_uchar;
  short int *ptr_sshort;
  unsigned short int *ptr_ushort;
  int *ptr_sint;
  unsigned int *ptr_uint;
  long int *ptr_slong;
  unsigned long int *ptr_ulong;
  intmax_t *ptr_simax;
  uintmax_t *ptr_uimax;
  size_t *ptr_size;
  ptrdiff_t *ptr_ptrdiff;
  void **ptr_void;
  // 
  size_t width;
  char width_string[WIDTH_MAX + 1];
  int w;        // Index inside width string.
  int assigned = 0;     // Assignment counter.
  int consumed = 0;     // Consumed counter.
  // 
  intmax_t value_i;
  uintmax_t value_u;
  // 
  const char *end_format;
  const char *end_input;
  int count;    // Generic counter.
  int index;    // Generic index.
  bool ascii[128];
  // 
  void *pstatus;
  // 
  // Initialize some data.
  // 
  width_string[0] = '\0';
  end_format = format + (strlen (format));
  // 
  // Check arguments and find where input comes.
  // 
  if (fp == NULL && (string == NULL || string[0] == 0))
    {
      errset (EINVAL);  // Invalid argument.
      return (EOF);
    }
  // 
  if (fp != NULL && string != NULL && string[0] != 0)
    {
      errset (EINVAL);  // Invalid argument.
      return (EOF);
    }
  // 
  if (fp != NULL)
    {
      stream = 1;
    }
  // 
  // 
  // 
  for (;;)
    {
      if (stream)
        {
          pstatus = fgets (buffer, BUFSIZ, fp);
          // 
          if (pstatus == NULL)
            {
              return (ass_or_eof (consumed, assigned));
            }
          // 
          input = buffer;
          start = input;
          next = NULL;
        }
      // 
      // Calculate end input.
      // 
      end_input = input + (strlen (input));
      // 
      // Scan format and input strings. Index `f' is
      // not reset.
      // 
      while (&format[f] < end_format && input < end_input)
        {
          if (!specifier)
            {
              // --------------------------------------
              // The context is not
              // inside a specifier.
              // --------------------------------------
              if (isspace (format[f]))
                {
                  // --------------------------- Space.
                  while (isspace (*input))
                    {
                      input++;
                    }
                  // 
                  // Verify that the input string is
                  // not finished.
                  // 
                  if (input[0] == 0)
                    {
                      // 
                      // As the input string is
                      // finished, the format
                      // string index is not advanced, 
                      // because there
                      // might be more spaces on the
                      // next line (if
                      // there is a next line, of
                      // course).
                      // 
                      continue;
                    }
                  else
                    {
                      f++;
                      continue;
                    }
                }
              if (format[f] != '%')
                {
                  // -------------- Ordinary character.
                  if (format[f] == *input)
                    {
                      input++;
                      f++;
                      continue;
                    }
                  else
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                }
              if (format[f] == '%' && format[f + 1] == '%')
                {
                  // ---------- Matching a literal '%'.
                  f++;
                  if (format[f] == *input)
                    {
                      input++;
                      f++;
                      continue;
                    }
                  else
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                }
              if (format[f] == '%')
                {
                  // ---------- Percent of a specifier.
                  f++;
                  specifier = 1;
                  specifier_flags = 1;
                  continue;
                }
            }
          // 
          if (specifier && specifier_flags)
            {
              // --------------------------------------
              // The context is inside 
              // specifier flags.
              // --------------------------------------
              if (format[f] == '*')
                {
                  // ----- Assignment suppression star.
                  flag_star = 1;
                  f++;
                }
              else
                {
                  // ----------------------------------
                  // End of flags and begin of
                  // specifier length.
                  // ----------------------------------
                  specifier_flags = 0;
                  specifier_width = 1;
                }
            }
          // 
          if (specifier && specifier_width)
            {
              // --------------------------------------
              // The context is inside a 
              // specifier width.
              // --------------------------------------
              for (w = 0;
                   format[f] >= '0'
                   && format[f] <= '9'
                   && w < WIDTH_MAX; w++)
                {
                  width_string[w] = format[f];
                  f++;
                }
              width_string[w] = '\0';
              width = atoi (width_string);
              if (width > WIDTH_MAX)
                {
                  width = WIDTH_MAX;
                }
              // 
              // --------------------------------------
              // A zero width means an unspecified
              // limit for the field
              // length.
              // --------------------------------------
              // End of spec. width and
              // begin of spec. type.
              // --------------------------------------
              specifier_width = 0;
              specifier_type = 1;
            }
          // 
          if (specifier && specifier_type)
            {
              // 
              // Specifiers with length modifier.
              // 
              if (format[f] == 'h' && format[f + 1] == 'h')
                {
                  // ---------------------------- char.
                  if (format[f + 2] == 'd')
                    {
                      // -------- signed char, base 10.
                      value_i =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_schar =
                            va_arg (ap, signed char *);
                          *ptr_schar = value_i;
                          assigned++;
                        }
                      f += 3;
                      input = next;
                    }
                  else if (format[f + 2] == 'i')
                    {
                      // ------------------------------
                      // signed char, base unknown.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 0,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_schar =
                            va_arg (ap, signed char *);
                          *ptr_schar = value_i;
                          assigned++;
                        }
                      f += 3;
                      input = next;
                    }
                  else if (format[f + 2] == 'o')
                    {
                      // ------------------------------
                      // signed char, base 8.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 8,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_schar =
                            va_arg (ap, signed char *);
                          *ptr_schar = value_i;
                          assigned++;
                        }
                      f += 3;
                      input = next;
                    }
                  else if (format[f + 2] == 'u')
                    {
                      // ------------------------------
                      // unsigned char, base 10.
                      // ------------------------------
                      value_u =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_uchar =
                            va_arg (ap, unsigned char *);
                          *ptr_uchar = value_u;
                          assigned++;
                        }
                      f += 3;
                      input = next;
                    }
                  else if (format[f + 2] == 'x'
                           || format[f + 2] == 'X')
                    {
                      // ------------------------------
                      // signed char, base 16.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 16,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_schar =
                            va_arg (ap, signed char *);
                          *ptr_schar = value_i;
                          assigned++;
                        }
                      f += 3;
                      input = next;
                    }
                  else if (format[f + 2] == 'n')
                    {
                      // ------------------------------
                      // signed char,
                      // string index counter.
                      // ------------------------------
                      ptr_schar =
                        va_arg (ap, signed char *);
                      *ptr_schar =
                        (signed char) (input - start +
                                       scanned);
                      f += 3;
                    }
                  else
                    {
                      // ------------------------------
                      // unsupported or
                      // unknown specifier.
                      // ------------------------------
                      f += 2;
                    }
                }
              else if (format[f] == 'h')
                {
                  // --------------------------- short.
                  if (format[f + 1] == 'd')
                    {
                      // ------------------------------
                      // signed short, base 10.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_sshort =
                            va_arg (ap, signed short *);
                          *ptr_sshort = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'i')
                    {
                      // ------------------------------
                      // signed
                      // short, base unknown.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 0,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_sshort =
                            va_arg (ap, signed short *);
                          *ptr_sshort = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'o')
                    {
                      // ------------------------------
                      // signed short, base 8.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 8,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_sshort =
                            va_arg (ap, signed short *);
                          *ptr_sshort = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'u')
                    {
                      // ------------------------------
                      // unsigned short, base 10.
                      // ------------------------------
                      value_u =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_ushort =
                            va_arg (ap, unsigned short *);
                          *ptr_ushort = value_u;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'x'
                           || format[f + 2] == 'X')
                    {
                      // ------------------------------
                      // signed short, base 16.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 16,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_sshort =
                            va_arg (ap, signed short *);
                          *ptr_sshort = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'n')
                    {
                      // ------------------------------
                      // signed char,
                      // string index counter.
                      // ------------------------------
                      ptr_sshort =
                        va_arg (ap, signed short *);
                      *ptr_sshort =
                        (signed short) (input - start +
                                        scanned);
                      f += 2;
                    }
                  else
                    {
                      // ------------------------------
                      // unsupported or
                      // unknown specifier.
                      // ------------------------------
                      f += 1;
                    }
                }
              // --------- There is no `long long int'.
              else if (format[f] == 'l')
                {
                  // ------------------------ long int.
                  if (format[f + 1] == 'd')
                    {
                      // ------------------------------
                      // signed long, base 10.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_slong =
                            va_arg (ap, signed long *);
                          *ptr_slong = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'i')
                    {
                      // ------------------------------
                      // signed
                      // long, base unknown.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 0,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_slong =
                            va_arg (ap, signed long *);
                          *ptr_slong = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'o')
                    {
                      // ------------------------------
                      // signed long, base 8.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 8,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_slong =
                            va_arg (ap, signed long *);
                          *ptr_slong = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'u')
                    {
                      // ------------------------------
                      // unsigned long, base 10.
                      // ------------------------------
                      value_u =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_ulong =
                            va_arg (ap, unsigned long *);
                          *ptr_ulong = value_u;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'x'
                           || format[f + 2] == 'X')
                    {
                      // ------------------------------
                      // signed long, base 16.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 16,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_slong =
                            va_arg (ap, signed long *);
                          *ptr_slong = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'n')
                    {
                      // ------------------------------
                      // signed char,
                      // string index counter.
                      // ------------------------------
                      ptr_slong =
                        va_arg (ap, signed long *);
                      *ptr_slong =
                        (signed long) (input - start +
                                       scanned);
                      f += 2;
                    }
                  else
                    {
                      // ------------------------------
                      // unsupported or
                      // unknown specifier.
                      // ------------------------------
                      f += 1;
                    }
                }
              else if (format[f] == 'j')
                {
                  // -----------------.------ intmax_t.
                  if (format[f + 1] == 'd')
                    {
                      // ----------- intmax_t, base 10.
                      value_i =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_simax =
                            va_arg (ap, intmax_t *);
                          *ptr_simax = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'i')
                    {
                      // ------------------------------
                      // intmax_t, base unknown.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 0,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_simax =
                            va_arg (ap, intmax_t *);
                          *ptr_simax = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'o')
                    {
                      // ------------------------------
                      // intmax_t, base 8.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 8,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_simax =
                            va_arg (ap, intmax_t *);
                          *ptr_simax = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'u')
                    {
                      // ------------------------------
                      // uintmax_t, base 10.
                      // ------------------------------
                      value_u =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_uimax =
                            va_arg (ap, uintmax_t *);
                          *ptr_uimax = value_u;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'x'
                           || format[f + 2] == 'X')
                    {
                      // ------------------------------
                      // intmax_t, base 16.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 16,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_simax =
                            va_arg (ap, intmax_t *);
                          *ptr_simax = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'n')
                    {
                      // ------------------------------
                      // signed char,
                      // string index counter.
                      // ------------------------------
                      ptr_simax = va_arg (ap, intmax_t *);
                      *ptr_simax =
                        (intmax_t) (input - start +
                                    scanned);
                      f += 2;
                    }
                  else
                    {
                      // ------------------------------
                      // unsupported or
                      // unknown specifier.
                      // ------------------------------
                      f += 1;
                    }
                }
              else if (format[f] == 'z')
                {
                  // -------------------------- size_t.
                  if (format[f + 1] == 'd')
                    {
                      // ------------------------------
                      // size_t, base 10.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_size = va_arg (ap, size_t *);
                          *ptr_size = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'i')
                    {
                      // ------------------------------
                      // size_t, base unknown.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 0,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_size = va_arg (ap, size_t *);
                          *ptr_size = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'o')
                    {
                      // ------------------------------
                      // size_t, base 8.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 8,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_size = va_arg (ap, size_t *);
                          *ptr_size = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'u')
                    {
                      // ------------------------------
                      // size_t, base 10.
                      // ------------------------------
                      value_u =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_size = va_arg (ap, size_t *);
                          *ptr_size = value_u;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'x'
                           || format[f + 2] == 'X')
                    {
                      // ------------------------------
                      // size_t, base 16.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 16,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_size = va_arg (ap, size_t *);
                          *ptr_size = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'n')
                    {
                      // ------------------------------
                      // signed char,
                      // string index counter.
                      // ------------------------------
                      ptr_size = va_arg (ap, size_t *);
                      *ptr_size =
                        (size_t) (input - start + scanned);
                      f += 2;
                    }
                  else
                    {
                      // ------------------------------
                      // unsupported or
                      // unknown specifier.
                      // ------------------------------
                      f += 1;
                    }
                }
              else if (format[f] == 't')
                {
                  // ----------------------- ptrdiff_t.
                  if (format[f + 1] == 'd')
                    {
                      // ------------------------------
                      // ptrdiff_t, base 10.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_ptrdiff =
                            va_arg (ap, ptrdiff_t *);
                          *ptr_ptrdiff = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'i')
                    {
                      // ------------------------------
                      // ptrdiff_t, base unknown.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 0,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_ptrdiff =
                            va_arg (ap, ptrdiff_t *);
                          *ptr_ptrdiff = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'o')
                    {
                      // ------------------------------
                      // ptrdiff_t, base 8.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 8,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_ptrdiff =
                            va_arg (ap, ptrdiff_t *);
                          *ptr_ptrdiff = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'u')
                    {
                      // ------------------------------
                      // ptrdiff_t, base 10.
                      // ------------------------------
                      value_u =
                        strtointmax (input, &next, 10,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_ptrdiff =
                            va_arg (ap, ptrdiff_t *);
                          *ptr_ptrdiff = value_u;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'x'
                           || format[f + 2] == 'X')
                    {
                      // ------------------------------
                      // ptrdiff_t, base 16.
                      // ------------------------------
                      value_i =
                        strtointmax (input, &next, 16,
                                     width);
                      if (input == next)
                        {
                          return (ass_or_eof
                                  (consumed, assigned));
                        }
                      consumed++;
                      if (!flag_star)
                        {
                          ptr_ptrdiff =
                            va_arg (ap, ptrdiff_t *);
                          *ptr_ptrdiff = value_i;
                          assigned++;
                        }
                      f += 2;
                      input = next;
                    }
                  else if (format[f + 1] == 'n')
                    {
                      // ------------------------------
                      // signed char,
                      // string index counter.
                      // ------------------------------
                      ptr_ptrdiff =
                        va_arg (ap, ptrdiff_t *);
                      *ptr_ptrdiff =
                        (ptrdiff_t) (input - start +
                                     scanned);
                      f += 2;
                    }
                  else
                    {
                      // ------------------------------
                      // unsupported or
                      // unknown specifier.
                      // ------------------------------
                      f += 1;
                    }
                }
              // 
              // Specifiers with no length modifier.
              // 
              if (format[f] == 'd')
                {
                  // ---------- signed short, base 10.
                  value_i =
                    strtointmax (input, &next, 10, width);
                  if (input == next)
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                  consumed++;
                  if (!flag_star)
                    {
                      ptr_sshort =
                        va_arg (ap, signed short *);
                      *ptr_sshort = value_i;
                      assigned++;
                    }
                  f += 1;
                  input = next;
                }
              else if (format[f] == 'i')
                {
                  // ----------------------------------
                  // signed
                  // int, base unknown.
                  // ----------------------------------
                  value_i =
                    strtointmax (input, &next, 0, width);
                  if (input == next)
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                  consumed++;
                  if (!flag_star)
                    {
                      ptr_sint = va_arg (ap, signed int *);
                      *ptr_sint = value_i;
                      assigned++;
                    }
                  f += 1;
                  input = next;
                }
              else if (format[f] == 'o')
                {
                  // ----------------------------------
                  // signed int, base 8.
                  // ----------------------------------
                  value_i =
                    strtointmax (input, &next, 8, width);
                  if (input == next)
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                  consumed++;
                  if (!flag_star)
                    {
                      ptr_sint = va_arg (ap, signed int *);
                      *ptr_sint = value_i;
                      assigned++;
                    }
                  f += 1;
                  input = next;
                }
              else if (format[f] == 'u')
                {
                  // ----------------------------------
                  // unsigned short, base 10.
                  // ----------------------------------
                  value_u =
                    strtointmax (input, &next, 10, width);
                  if (input == next)
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                  consumed++;
                  if (!flag_star)
                    {
                      ptr_uint =
                        va_arg (ap, unsigned int *);
                      *ptr_uint = value_u;
                      assigned++;
                    }
                  f += 1;
                  input = next;
                }
              else if (format[f] == 'x' || format[f] == 'X')
                {
                  // ----------------------------------
                  // signed short, base 16.
                  // ----------------------------------
                  value_i =
                    strtointmax (input, &next, 16, width);
                  if (input == next)
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                  consumed++;
                  if (!flag_star)
                    {
                      ptr_sint = va_arg (ap, signed int *);
                      *ptr_sint = value_i;
                      assigned++;
                    }
                  f += 1;
                  input = next;
                }
              else if (format[f] == 'c')
                {
                  // -------------------------- char[].
                  if (width == 0)
                    width = 1;
                  // 
                  if (!flag_star)
                    ptr_char = va_arg (ap, char *);
                  // 
                  for (count = 0;
                       width > 0 && *input != 0;
                       width--, ptr_char++, input++)
                    {
                      if (!flag_star)
                        *ptr_char = *input;
                      // 
                      count++;
                    }
                  // 
                  if (count)
                    consumed++;
                  if (count && !flag_star)
                    assigned++;
                  // 
                  f += 1;
                }
              else if (format[f] == 's')
                {
                  // -------------------------- string.
                  if (!flag_star)
                    ptr_char = va_arg (ap, char *);
                  // 
                  for (count = 0;
                       !isspace (*input)
                       && *input != 0; ptr_char++, input++)
                    {
                      if (!flag_star)
                        *ptr_char = *input;
                      // 
                      count++;
                    }
                  if (!flag_star)
                    *ptr_char = 0;
                  // 
                  if (count)
                    consumed++;
                  if (count && !flag_star)
                    assigned++;
                  // 
                  f += 1;
                }
              else if (format[f] == '[')
                {
                  // 
                  f++;
                  // 
                  if (format[f] == '^')
                    {
                      inverted = 1;
                      f++;
                    }
                  else
                    {
                      inverted = 0;
                    }
                  // 
                  // Reset ascii array.
                  // 
                  for (index = 0; index < 128; index++)
                    {
                      ascii[index] = inverted;
                    }
                  // 
                  // 
                  // 
                  for (count = 0;
                       &format[f] < end_format; count++)
                    {
                      if (format[f] == ']' && count > 0)
                        {
                          break;
                        }
                      // 
                      // Check for an interval.
                      // 
                      if (format[f + 1] == '-'
                          && format[f + 2] != ']'
                          && format[f + 2] != 0)
                        {
                          // 
                          // Interval.
                          // 
                          for (index = format[f];
                               index <= format[f + 2];
                               index++)
                            {
                              ascii[index] = !inverted;
                            }
                          f += 3;
                          continue;
                        }
                      // 
                      // Single character.
                      // 
                      index = format[f];
                      ascii[index] = !inverted;
                      f++;
                    }
                  // 
                  // Is the scan correctly finished?.
                  // 
                  if (format[f] != ']')
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                  // 
                  // The ascii table is populated.
                  // 
                  if (width == 0)
                    width = SIZE_MAX;
                  // 
                  // Scan the input string.
                  // 
                  if (!flag_star)
                    ptr_char = va_arg (ap, char *);
                  // 
                  for (count = 0;
                       width > 0 && *input != 0;
                       width--, ptr_char++, input++)
                    {
                      index = *input;
                      if (ascii[index])
                        {
                          if (!flag_star)
                            *ptr_char = *input;
                          count++;
                        }
                      else
                        {
                          break;
                        }
                    }
                  // 
                  if (count)
                    consumed++;
                  if (count && !flag_star)
                    assigned++;
                  // 
                  f += 1;
                }
              else if (format[f] == 'p')
                {
                  // -------------------------- void *.
                  value_i =
                    strtointmax (input, &next, 16, width);
                  if (input == next)
                    {
                      return (ass_or_eof
                              (consumed, assigned));
                    }
                  consumed++;
                  if (!flag_star)
                    {
                      ptr_void = va_arg (ap, void **);
                      *ptr_void = (void *) ((int) value_i);
                      assigned++;
                    }
                  f += 1;
                  input = next;
                }
              else if (format[f] == 'n')
                {
                  // ----------------------------------
                  // signed char,
                  // string index counter.
                  // ----------------------------------
                  ptr_sint = va_arg (ap, signed int *);
                  *ptr_sint =
                    (signed char) (input - start + scanned);
                  f += 1;
                }
              else
                {
                  // ----------------------------------
                  // unsupported or
                  // unknown specifier.
                  // ----------------------------------
                  ;
                }

              // --------------------------------------
              // End of specifier.
              // --------------------------------------

              width_string[0] = '\0';
              specifier = 0;
              specifier_flags = 0;
              specifier_width = 0;
              specifier_type = 0;
              flag_star = 0;

            }
        }
      // 
      // The format or the input string is terminated.
      // 
      if (&format[f] < end_format && stream)
        {
          // 
          // Only the input string is finished, and
          // the input comes
          // from a stream, so another read will be
          // done.
          // 
          scanned += (int) (input - start);
          continue;
        }
      // 
      // The format string is terminated.
      // 
      return (ass_or_eof (consumed, assigned));
    }
}

//----------------------------------------------------------
static intmax_t
strtointmax (const char *restrict string,
             const char **restrict endptr, int base,
             size_t max_width)
{
  int i;
  int d;        // Digits counter.
  int sign = +1;
  intmax_t number;
  intmax_t previous;
  int digit;
  // 
  bool flag_prefix_oct = 0;
  bool flag_prefix_exa = 0;
  bool flag_prefix_dec = 0;
  // 
  // If the `max_width' value is zero, fix it to the
  // maximum
  // that it can represent.
  // 
  if (max_width == 0)
    {
      max_width = SIZE_MAX;
    }
  // 
  // Eat initial spaces, but if there are spaces,
  // there is an
  // error inside the calling function!
  // 
  for (i = 0; isspace (string[i]); i++)
    {
      fprintf (stderr,
               "libc error: file \"%s\", line %i\n",
               __FILE__, __LINE__);
      ;
    }
  // 
  // Check sign. The `max_width' counts also the sign, 
  // if there is
  // one.
  // 
  if (string[i] == '+')
    {
      sign = +1;
      i++;
      max_width--;
    }
  else if (string[i] == '-')
    {
      sign = -1;
      i++;
      max_width--;
    }
  // 
  // Check for prefix.
  // 
  if (string[i] == '0')
    {
      if (string[i + 1] == 'x' || string[i + 1] == 'X')
        {
          flag_prefix_exa = 1;
        }
      if (isdigit (string[i + 1]))
        {
          flag_prefix_oct = 1;
        }
    }
  // 
  if (string[i] > '0' && string[i] <= '9')
    {
      flag_prefix_dec = 1;
    }
  // 
  // Check compatibility with requested base.
  // 
  if (flag_prefix_exa)
    {
      if (base == 0)
        {
          base = 16;
        }
      else if (base == 16)
        {
          ;     // Ok.
        }
      else
        {
          // 
          // Incompatible sequence: only the initial
          // zero is reported.
          // 
          *endptr = &string[i + 1];
          return ((intmax_t) 0);
        }
      // 
      // Move on, after the `0x' prefix.
      // 
      i += 2;
    }
  // 
  if (flag_prefix_oct)
    {
      if (base == 0)
        {
          base = 8;
        }
      // 
      // Move on, after the `0' prefix.
      // 
      i += 1;
    }
  // 
  if (flag_prefix_dec)
    {
      if (base == 0)
        {
          base = 10;
        }
    }
  // 
  // Scan the string.
  // 
  for (d = 0, number = 0;
       d < max_width && string[i] != 0; i++, d++)
    {
      if (string[i] >= '0' && string[i] <= '9')
        {
          digit = string[i] - '0';
        }
      else if (string[i] >= 'A' && string[i] <= 'F')
        {
          digit = string[i] - 'A' + 10;
        }
      else if (string[i] >= 'a' && string[i] <= 'f')
        {
          digit = string[i] - 'a' + 10;
        }
      else
        {
          digit = 999;
        }
      // 
      // Give a sign to the digit.
      // 
      digit *= sign;
      // 
      // Compare with the base.
      // 
      if (base > (digit * sign))
        {
          // 
          // Check if the current digit can be safely
          // computed.
          // 
          previous = number;
          number *= base;
          number += digit;
          if (number / base != previous)
            {
              // 
              // Out of range.
              // 
              *endptr = &string[i + 1];
              errset (ERANGE);  // Result too large.
              if (sign > 0)
                {
                  return (INTMAX_MAX);
                }
              else
                {
                  return (INTMAX_MIN);
                }
            }
        }
      else
        {
          *endptr = &string[i];
          return (number);
        }
    }
  // 
  // The string is finished or the max digits length
  // is reached.
  // 
  *endptr = &string[i];
  // 
  return (number);
}

//----------------------------------------------------------
static int
ass_or_eof (int consumed, int assigned)
{
  if (consumed == 0)
    {
      return (EOF);
    }
  else
    {
      return (assigned);
    }
}

//----------------------------------------------------------
