#ifndef _STDLIB_H
#define _STDLIB_H       1
//----------------------------------------------------------
#include <size_t.h>
#include <wchar_t.h>
#include <NULL.h>
#include <limits.h>
#include <restrict.h>
#include <stdint.h>
//----------------------------------------------------------
typedef struct
{
  int quot;
  int rem;
} div_t;
//----------------------------------------------------------
typedef struct
{
  long int quot;
  long int rem;
} ldiv_t;
//----------------------------------------------------------
typedef struct
{
  long long int quot;
  long long int rem;
} lldiv_t;
//----------------------------------------------------------
typedef void (*atexit_t) (void);        // Non standard.
                                  // [1]
//
// [1] The type `atexit_t' is a pointer to a function
//     for the "at exit" procedure, with no parameters
//     and returning void. With the declaration of type
//     `atexit_t', the function prototype of `atexit()'
//     is easier to declare and to understand. Original
//     declaration is:
//
//     int atexit (void (*function) (void));
//
//----------------------------------------------------------
typedef struct
{
  uintptr_t allocated:1, filler:1, next:30;
} _alloc_head_t;        // Non standard [2]
//
// [2] This is used for the `malloc()' management, as
//     the pointer to the following element of memory,
//     that might be free or allocated.
//
// La dimensione di «uintptr_t» condiziona la struttura
// «mm_head_t» e la dimensione delle unità minime di
// memoria allocata. «uintptr_t» è da 32 bit, così
// l'immagine del kernel è allineata a blocchi da
// 32 bit e così deve essere anche per gli altri
// blocchi di memoria.
// Essendo i blocchi di memoria multipli di 32 bit, gli
// indirizzi sono sempre multipli di 4 (4 byte);
// pertanto, servono solo 30 bit per rappresentare
// l'indirizzo, che poi viene ottenuto moltiplicandolo
// per quattro. Di conseguenza, il bit meno
// significativo viene usato per annotare se il blocco
// di memoria è libero e il bit successivo non viene
// usato. Questo meccanismo potrebbe essere usato anche
// con un indirizzamento a 16 bit, dove servirebbero 15
// bit per indirizzi multipli di due byte.
//
//----------------------------------------------------------
#define EXIT_FAILURE    1
#define EXIT_SUCCESS    0
#define RAND_MAX        INT_MAX
#define MB_CUR_MAX      ((size_t) MB_LEN_MAX)
//----------------------------------------------------------
void _Exit (int status);
void abort (void);
int abs (int j);
int atexit (atexit_t function);
int atoi (const char *string);
long int atol (const char *string);
#define  calloc(b, s) (malloc ((b) * (s)))
div_t div (int numer, int denom);
void exit (int status);
void free (void *ptr);
char *getenv (const char *name);
long int labs (long int j);
long long int llabs (long long int j);
ldiv_t ldiv (long int numer, long int denom);
lldiv_t lldiv (long long int numer, long long int denom);
void *malloc (size_t size);
int putenv (const char *string);
void qsort (void *base, size_t nmemb, size_t size,
            int (*compare) (const void *, const void *));
int rand (void);
void *realloc (void *ptr, size_t size);
int setenv (const char *name, const char *value,
            int overwrite);
void srand (unsigned int seed);
long int strtol (const char *restrict string,
                 char **restrict endptr, int base);
unsigned long int strtoul (const char *restrict string,
                           char **restrict endptr,
                           int base);
//int             system   (const char *string);
int unsetenv (const char *name);
//----------------------------------------------------------
#endif
