#include <stdlib.h>
#include <string.h>
//----------------------------------------------------------
// This file contains a non standard definition,
// related to the environment handling.
//
// The file `crt0.s', before calling the main function,
// calls the function `_environment_setup(), that is
// responsible for initializing the array
// `_environment_table[][]' and for copying the content
// of the environment, as it comes from the `exec()'
// system call.
//
// The pointers to the environment strings organised
// inside the array `_environment_table[][]', are also
// copied inside the array of pointers
// `_environment[]'.
//
// After all that is done, inside `crt0.s', the pointer
// to `_environment[]' is copied to the traditional
// variable `environ' and also to the previous value of
// the pointer variable `envp'.
//
// This way, applications will get the environment, but
// organised inside the table `_environment_table[][]'.
// So, functions like `getenv()' and `setenv()' do know
// where to look for.
//
// It is useful to notice that there is no prototype
// and no extern declaration inside the file
// <stdlib.h>, about this function and these arrays,
// because applications do not have to know about it.
//
// Please notice that `environ' could be just the same
// as `_environment' here, but the common use puts
// `environ' inside <unistd.h>, although for this
// implementation it should be better placed inside
// <stdlib.h>.
//
//----------------------------------------------------------
char _environment_table[ARG_MAX / 32][ARG_MAX / 16];
char *_environment[ARG_MAX / 32 + 1];
//----------------------------------------------------------
void
_environment_setup (char *envp[])
{
  int e;
  int s;
  // 
  // Reset the `_environment_table[][]' array.
  // 
  for (e = 0; e < ARG_MAX / 32; e++)
    {
      for (s = 0; s < ARG_MAX / 16; s++)
        {
          _environment_table[e][s] = 0;
        }
    }
  // 
  // Set the `_environment[]' pointers. The final
  // extra element must
  // be a NULL pointer.
  // 
  for (e = 0; e < ARG_MAX / 32; e++)
    {
      _environment[e] = _environment_table[e];
    }
  _environment[ARG_MAX / 32] = NULL;
  // 
  // Copy the environment inside the array, but only
  // if `envp' is
  // not NULL.
  // 
  if (envp != NULL)
    {
      for (e = 0; envp[e] != NULL && e < ARG_MAX / 32; e++)
        {
          strncpy (_environment_table[e], envp[e],
                   (ARG_MAX / 16) - 1);
        }
    }
}
