#include <stdlib.h>
#include <string.h>
#include <errno.h>
//----------------------------------------------------------
extern char *_environment[];
//----------------------------------------------------------
int
putenv (const char *string)
{
  int e;        // First index: environment table
  // items.
  int f;        // Second index: environment string
  // scan.
  // 
  // Check if the input is empty. No error is
  // reported.
  // 
  if (string == NULL || strlen (string) == 0)
    {
      return (0);
    }
  // 
  // Check if the input is valid: there must be a `='
  // sign.
  // Error here is reported.
  // 
  if (strchr (string, '=') == NULL)
    {
      errset (EINVAL);  // Invalid argument.
      return (-1);
    }
  // 
  // Scan the environment table items, with index `e'. 
  // The intent is
  // to find a previous environment variable with the
  // same name.
  // 
  for (e = 0; e < ARG_MAX / 32; e++)
    {
      // 
      // Scan the string of the environment item, with 
      // index `f'.
      // The scan continue until `string[f]' and
      // `_environment[e][f]'
      // are equal.
      // 
      for (f = 0;
           f < ARG_MAX / 16 - 1
           && string[f] == _environment[e][f]; f++)
        {
          ;     // Just scan.
        }
      // 
      // At this point, `string[f-1]' and
      // `_environment[e][f-1]'
      // should contain `='. If it is so, the
      // environment is replaced.
      // 
      if (string[f - 1] == '='
          && _environment[e][f - 1] == '=')
        {
          // 
          // The environment item was found: now
          // replace the pointer.
          // 
          _environment[e] = (char *) string;
          // 
          // Return.
          // 
          return (0);
        }
    }
  // 
  // The item was not found. Scan again for a free
  // slot.
  // 
  for (e = 0; e < ARG_MAX / 32; e++)
    {
      if (_environment[e] == NULL
          || _environment[e][0] == 0)
        {
          // 
          // An empty item was found and the pointer
          // will be
          // replaced.
          // 
          _environment[e] = (char *) string;
          // 
          // Return.
          // 
          return (0);
        }
    }
  // 
  // Sorry: the empty slot was not found!
  // 
  errset (ENOMEM);      // Not enough space.
  return (-1);
}
