#include <stdlib.h>
#include <string.h>
#include <errno.h>
//----------------------------------------------------------
extern char *_environment[];
extern char *_environment_table[];
//----------------------------------------------------------
int
setenv (const char *name, const char *value, int overwrite)
{
  int e;        // First index: environment table
  // items.
  int f;        // Second index: environment string
  // scan.
  // 
  // Check if the input is empty. No error is
  // reported.
  // 
  if (name == NULL || strlen (name) == 0)
    {
      return (0);
    }
  // 
  // Check if the input is valid: error here is
  // reported.
  // 
  if (strchr (name, '=') != NULL)
    {
      errset (EINVAL);  // Invalid argument.
      return (-1);
    }
  // 
  // Check if the input is too big.
  // 
  if ((strlen (name) + strlen (value) + 2) > ARG_MAX / 16)
    {
      // 
      // The environment to be saved is bigger than
      // the
      // available string size, inside
      // `_environment_table[]'.
      // 
      errset (ENOMEM);  // Not enough space.
      return (-1);
    }
  // 
  // Scan the environment table items, with index `e'. 
  // The intent is
  // to find a previous environment variable with the
  // same name.
  // 
  for (e = 0; e < ARG_MAX / 32; e++)
    {
      // 
      // Scan the string of the environment item, with 
      // index `f'.
      // The scan continue until `name[f]' and
      // `_environment[e][f]'
      // are equal.
      // 
      for (f = 0;
           f < ARG_MAX / 16 - 1
           && name[f] == _environment[e][f]; f++)
        {
          ;     // Just scan.
        }
      // 
      // At this point, `name[f]' and
      // `_environment[e][f]' are
      // different: if `name[f]' is zero the name
      // string is
      // terminated; if `_environment[e][f]' is also
      // equal to `=',
      // the environment item is corresponding to the
      // requested name.
      // 
      if (name[f] == 0 && _environment[e][f] == '=')
        {
          // 
          // The environment item was found; if it can 
          // be overwritten,
          // the write is done.
          // 
          if (overwrite)
            {
              // 
              // To be able to handle both `setenv()'
              // and `putenv()',
              // before removing the item, it is fixed 
              // the pointer to
              // the global environment table.
              // 
              _environment[e] = _environment_table[e];
              // 
              // Now copy the new environment. The
              // string size was
              // already checked.
              // 
              strcpy (_environment[e], name);
              strcat (_environment[e], "=");
              strcat (_environment[e], value);
              // 
              // Return.
              // 
              return (0);
            }
          // 
          // Cannot overwrite!
          // 
          errset (EUNKNOWN);
          return (-1);
        }
    }
  // 
  // The item was not found. Scan again for a free
  // slot.
  // 
  for (e = 0; e < ARG_MAX / 32; e++)
    {
      if (_environment[e] == NULL
          || _environment[e][0] == 0)
        {
          // 
          // An empty item was found. To be able to
          // handle both
          // `setenv()' and `putenv()', it is fixed
          // the pointer to
          // the global environment table.
          // 
          _environment[e] = _environment_table[e];
          // 
          // Now copy the new environment. The string
          // size was
          // already checked.
          // 
          strcpy (_environment[e], name);
          strcat (_environment[e], "=");
          strcat (_environment[e], value);
          // 
          // Return.
          // 
          return (0);
        }
    }
  // 
  // Sorry: the empty slot was not found!
  // 
  errset (ENOMEM);      // Not enough space.
  return (-1);
}
