#ifndef _SYS_OS32_H
#define _SYS_OS32_H      1
//----------------------------------------------------------
// This file contains all the declarations that don't
// have a better place inside standard headers files.
// Even declarations related to device numbers and
// system calls is contained here.
//----------------------------------------------------------
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <stdint.h>
#include <signal.h>
#include <limits.h>
#include <stdio.h>
#include <stddef.h>
#include <restrict.h>
#include <stdarg.h>
#include <termios.h>
//----------------------------------------------------------
typedef uint16_t h_port_t;      // Port number in host 
                                // byte order.
typedef uint32_t h_addr_t;      // IPv4 address in
                                // host byte order.
//----------------------------------------------------------
// Please remember that system calls should never be
// used (called) inside the kernel code, because system
// calls cannot be nested for the os32 simple
// architecture!
// If a particular function is necessary inside the
// kernel, that usually is made by a system call, an
// appropriate k_...() function must be
// made, to avoid the problem.
//----------------------------------------------------------
// Device numbers.
//----------------------------------------------------------
#define DEV_UNDEFINED_MAJOR     ((dev_t) 0x00)
#define DEV_UNDEFINED           ((dev_t) 0x0000)
#define DEV_MEM_MAJOR           ((dev_t) 0x01)
#define DEV_MEM                 ((dev_t) 0x0101)
#define DEV_NULL                ((dev_t) 0x0102)
#define DEV_PORT                ((dev_t) 0x0103)
#define DEV_ZERO                ((dev_t) 0x0104)
#define DEV_TTY_MAJOR           ((dev_t) 0x02)
#define DEV_TTY                 ((dev_t) 0x0200)
//
#define DEV_KMEM_MAJOR          ((dev_t) 0x04)
#define DEV_KMEM_PS             ((dev_t) 0x0401)
#define DEV_KMEM_MMP            ((dev_t) 0x0402)
#define DEV_KMEM_SB             ((dev_t) 0x0403)
#define DEV_KMEM_INODE          ((dev_t) 0x0404)
#define DEV_KMEM_FILE           ((dev_t) 0x0405)
#define DEV_KMEM_ARP            ((dev_t) 0x0406)
#define DEV_KMEM_NET            ((dev_t) 0x0407)
#define DEV_KMEM_ROUTE          ((dev_t) 0x0408)
//
#define DEV_CONSOLE_MAJOR       ((dev_t) 0x05)
#define DEV_CONSOLE             ((dev_t) 0x05FF)
#define DEV_CONSOLE0            ((dev_t) 0x0500)
#define DEV_CONSOLE1            ((dev_t) 0x0501)
#define DEV_CONSOLE2            ((dev_t) 0x0502)
#define DEV_CONSOLE3            ((dev_t) 0x0503)
#define DEV_CONSOLE4            ((dev_t) 0x0504)
//
#define DEV_DM_MAJOR            ((dev_t) 0x08)
#define DEV_DM00                ((dev_t) 0x0800)
#define DEV_DM01                ((dev_t) 0x0801)
#define DEV_DM02                ((dev_t) 0x0802)
#define DEV_DM03                ((dev_t) 0x0803)
#define DEV_DM04                ((dev_t) 0x0804)
#define DEV_DM10                ((dev_t) 0x0810)
#define DEV_DM11                ((dev_t) 0x0811)
#define DEV_DM12                ((dev_t) 0x0812)
#define DEV_DM13                ((dev_t) 0x0813)
#define DEV_DM14                ((dev_t) 0x0814)
#define DEV_DM20                ((dev_t) 0x0820)
#define DEV_DM21                ((dev_t) 0x0821)
#define DEV_DM22                ((dev_t) 0x0822)
#define DEV_DM23                ((dev_t) 0x0823)
#define DEV_DM24                ((dev_t) 0x0824)
#define DEV_DM30                ((dev_t) 0x0830)
#define DEV_DM31                ((dev_t) 0x0831)
#define DEV_DM32                ((dev_t) 0x0832)
#define DEV_DM33                ((dev_t) 0x0833)
#define DEV_DM34                ((dev_t) 0x0834)
//
//----------------------------------------------------------
#define min(a, b) (a < b ? a : b)
#define max(a, b) (a > b ? a : b)
#define sizeof_array(x) (sizeof(x) / sizeof((x)[0]))
#define sizeof_field(t, f) (sizeof(((t*)0)->f))
//----------------------------------------------------------
#define INPUT_LINE_HIDDEN 0
#define INPUT_LINE_ECHO   1
//----------------------------------------------------------
#define MOUNT_DEFAULT     0     // Default mount
                                // options.
#define MOUNT_RO          1     // Read only mount
                                // option.
//----------------------------------------------------------
#define SYS_0                    0      // Nothing to
                                        // do.
#define SYS_CHDIR                1
#define SYS_CHMOD                2
#define SYS_CLOCK                3
#define SYS_CLOSE                4
#define SYS_EXEC                 5
#define SYS_EXIT                 6      // [1] see
                                        // below.
#define SYS_FCHMOD               7
#define SYS_FORK                 8
#define SYS_FSTAT                9
#define SYS_KILL                10
#define SYS_LSEEK               11
#define SYS_MKDIR               12
#define SYS_MKNOD               13
#define SYS_MOUNT               14
#define SYS_OPEN                15
#define SYS_PGRP                16
#define SYS_READ                17
#define SYS_SETEUID             18
#define SYS_SETUID              19
#define SYS_SIGNAL              20
#define SYS_SLEEP               21
#define SYS_STAT                22
#define SYS_TIME                23
#define SYS_UAREA               24
#define SYS_UMASK               25
#define SYS_UMOUNT              26
#define SYS_WAIT                27
#define SYS_WRITE               28
#define SYS_ZPCHAR              29      // [2]
#define SYS_ZPSTRING            30      // [2]
#define SYS_CHOWN               31
#define SYS_DUP                 33
#define SYS_DUP2                34
#define SYS_LINK                35
#define SYS_UNLINK              36
#define SYS_FCNTL               37
#define SYS_STIME               38
#define SYS_FCHOWN              39
#define SYS_BRK                 40
#define SYS_SBRK                41
#define SYS_PIPE                42
#define SYS_TCGETATTR           43
#define SYS_TCSETATTR           44
#define SYS_SETEGID             45
#define SYS_SETGID              46
#define SYS_SETJMP              47
#define SYS_LONGJMP             48
#define SYS_RECVFROM            49
#define SYS_SOCKET              50
#define SYS_CONNECT             51
#define SYS_SEND                52
#define SYS_IPCONFIG            53
#define SYS_ROUTEADD            54
#define SYS_ROUTEDEL            55
#define SYS_BIND                56
#define SYS_LISTEN              57
#define SYS_ACCEPT              58
//
// [1] The files `crt0...' need to know the value used
//     for the exit system call. If this value is
//     modified, all the file `crt0...' have also to be
//     modified the same way.
//
// [2] These system calls were developed at the
//     beginning, when no standard I/O was available.
//     They are to be considered as a last resort for
//     debugging purposes.
//
//----------------------------------------------------------
// The following values must be: 1, 2, 4, 8, 16, 32,...
// so that can be `OR' combined.
//
#define WAKEUP_EVENT_SIGNAL       0x0001
#define WAKEUP_EVENT_TIMER        0x0002
#define WAKEUP_EVENT_DEV_READ     0x0004
#define WAKEUP_EVENT_DEV_WRITE    0x0008
#define WAKEUP_EVENT_PIPE_READ    0x0010
#define WAKEUP_EVENT_PIPE_WRITE   0x0020
#define WAKEUP_EVENT_SOCK_READ    0x0040
#define WAKEUP_EVENT_SOCK_WRITE   0x0080
//----------------------------------------------------------
typedef struct
{
  int sfdn;
  struct sockaddr addr;
  socklen_t addrlen;
  int fl_flags;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_accept_t;
//----------------------------------------------------------
typedef struct
{
  int sfdn;
  struct sockaddr addr;
  socklen_t addrlen;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_bind_t;
//----------------------------------------------------------
typedef struct
{
  void *address;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_brk_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_chdir_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  mode_t mode;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_chmod_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  uid_t uid;
  uid_t gid;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_chown_t;
//----------------------------------------------------------
typedef struct
{
  clock_t ret;
} sysmsg_clock_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_close_t;
//----------------------------------------------------------
typedef struct
{
  int sfdn;
  struct sockaddr addr;
  socklen_t addrlen;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_connect_t;
//----------------------------------------------------------
typedef struct
{
  int fdn_old;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_dup_t;
//----------------------------------------------------------
typedef struct
{
  int fdn_old;
  int fdn_new;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_dup2_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  int argc;
  int envc;
  char arg_data[ARG_MAX / 2];
  char env_data[ARG_MAX / 2];
  uid_t uid;
  uid_t euid;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_exec_t;
//----------------------------------------------------------
typedef struct
{
  int status;
} sysmsg_exit_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  mode_t mode;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_fchmod_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  uid_t uid;
  uid_t gid;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_fchown_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  int cmd;
  int arg;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_fcntl_t;
//----------------------------------------------------------
typedef struct
{
  pid_t ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_fork_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  struct stat stat;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_fstat_t;
//----------------------------------------------------------
typedef struct
{
  int n;
  in_addr_t address;
  int m;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_ipconfig_t;
//----------------------------------------------------------
typedef struct
{
  void *env;
  int ret;
  // 
  // This structure is intentionally reduced.
  // 
} sysmsg_jmp_t;
//----------------------------------------------------------
typedef struct
{
  pid_t pid;
  int signal;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_kill_t;
//----------------------------------------------------------
typedef struct
{
  const char *path_old;
  const char *path_new;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_link_t;
//----------------------------------------------------------
typedef struct
{
  int sfdn;
  int backlog;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_listen_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  off_t offset;
  int whence;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_lseek_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  mode_t mode;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_mkdir_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  mode_t mode;
  dev_t device;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_mknod_t;
//----------------------------------------------------------
typedef struct
{
  const char *path_dev;
  const char *path_mnt;
  int options;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_mount_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  int flags;
  mode_t mode;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_open_t;
//----------------------------------------------------------
typedef struct
{
  int pipefd[2];
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_pipe_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  void *buffer;
  size_t count;
  int fl_flags;
  ssize_t ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_read_t;
//----------------------------------------------------------
typedef struct
{
  int sfdn;
  void *buffer;
  size_t count;
  int flags;
  void *addrfrom;
  void *addrsize;
  int fl_flags;
  ssize_t ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_recvfrom_t;
//----------------------------------------------------------
typedef struct
{
  in_addr_t destination;
  int m;
  in_addr_t router;
  int device;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_route_t;
//----------------------------------------------------------
typedef struct
{
  intptr_t increment;
  void *ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_sbrk_t;
//----------------------------------------------------------
typedef struct
{
  int sfdn;
  const void *buffer;
  size_t count;
  int flags;
  ssize_t ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_send_t;
//----------------------------------------------------------
typedef struct
{
  gid_t egid;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_setegid_t;
//----------------------------------------------------------
typedef struct
{
  uid_t euid;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_seteuid_t;
//----------------------------------------------------------
typedef struct
{
  gid_t gid;
  gid_t egid;
  gid_t sgid;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_setgid_t;
//----------------------------------------------------------
typedef struct
{
  uid_t uid;
  uid_t euid;
  uid_t suid;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_setuid_t;
//----------------------------------------------------------
typedef struct
{
  uintptr_t wrapper;
  sighandler_t handler;
  int signal;
  sighandler_t ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_signal_t;
//----------------------------------------------------------
typedef struct
{
  int family;
  int type;
  int protocol;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_socket_t;
//----------------------------------------------------------
typedef struct
{
  int events;
  int signal;
  unsigned int seconds;
  time_t ret;
} sysmsg_sleep_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  struct stat stat;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_stat_t;
//----------------------------------------------------------
typedef struct
{
  time_t ret;
} sysmsg_time_t;
//----------------------------------------------------------
typedef struct
{
  time_t timer;
  int ret;
} sysmsg_stime_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  int action;
  struct termios *attr;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_tcattr_t;
//----------------------------------------------------------
typedef struct
{
  uid_t uid;    // Read user ID.
  uid_t euid;   // Effective user ID.
  uid_t suid;   // Saved user ID.
  gid_t gid;    // Read group ID.
  gid_t egid;   // Effective group ID.
  gid_t sgid;   // Saved group ID.
  pid_t pid;    // Process ID.
  pid_t ppid;   // Parent PID.
  pid_t pgrp;   // Process group.
  mode_t umask; // Access permission mask.
  char *path_cwd;
  size_t path_cwd_size; // Max path size.
} sysmsg_uarea_t;
//----------------------------------------------------------
typedef struct
{
  mode_t umask;
  mode_t ret;
} sysmsg_umask_t;
//----------------------------------------------------------
typedef struct
{
  const char *path_mnt;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_umount_t;
//----------------------------------------------------------
typedef struct
{
  const char *path;
  int ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_unlink_t;
//----------------------------------------------------------
typedef struct
{
  int status;
  pid_t ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_wait_t;
//----------------------------------------------------------
typedef struct
{
  int fdn;
  const void *buffer;
  size_t count;
  ssize_t ret;
  int errno;
  int errln;
  char errfn[PATH_MAX];
} sysmsg_write_t;
//----------------------------------------------------------
typedef struct
{
  char c;
} sysmsg_zpchar_t;
//----------------------------------------------------------
typedef struct
{
  char string[BUFSIZ];
} sysmsg_zpstring_t;
//----------------------------------------------------------
void input_line (char *line, char *prompt, size_t size,
                 int type);
int mount (const char *path_dev, const char *path_mnt,
           int options);
int namep (const char *name, char *path, size_t size);
void sys (int syscallnr, void *message, size_t size);
int umount (const char *path_mnt);
void z_perror (const char *string);
int z_printf (const char *restrict format, ...);
int z_vprintf (const char *restrict format, va_list arg);
int ipconfig (int n, h_addr_t address, int m);
int routedel (h_addr_t destination, int m);
int routeadd (h_addr_t destination, int m,
              h_addr_t router, int device);
//----------------------------------------------------------
#endif
