#include <sys/os32.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <unistd.h>
//----------------------------------------------------------
static int terminal_echo (struct termios *orig);
static int terminal_noecho (struct termios *orig);
static int terminal_restore (struct termios *orig);
//----------------------------------------------------------
void
input_line (char *line, char *prompt, size_t size, int type)
{
  void *pstatus;
  int i;
  struct termios attr;
  // 
  // Set terminal configuration.
  // 
  if (type == INPUT_LINE_HIDDEN)
    {
      terminal_noecho (&attr);
    }
  else
    {
      terminal_echo (&attr);
    }
  // 
  if (prompt != NULL || strlen (prompt) > 0)
    {
      printf ("%s", prompt);
    }
  // 
  errno = 0;
  pstatus = fgets (line, (int) size, stdin);
  if (pstatus == NULL)
    {
      if (errno)
        {
          perror (NULL);
        }
      line[0] = 0;
      // 
      // Reset terminal mode.
      // 
      terminal_restore (&attr);
      return;
    }
  // 
  // Find the last position and, if there is a new
  // line code,
  // replace it with zero. If the string is empty, a
  // ^D was
  // received.
  // 
  i = strlen (line);
  if (i > 0 && line[i - 1] == '\n')
    {
      line[i - 1] = '\0';
    }
  // 
  // Restore terminal mode.
  // 
  terminal_restore (&attr);
}

//----------------------------------------------------------
static int
terminal_echo (struct termios *orig)
{
  int status;
  struct termios attr;
  // 
  // Save previous.
  // 
  status = tcgetattr (STDIN_FILENO, orig);
  if (status < 0)
    {
      return (-1);
    }
  // 
  // Get again.
  // 
  status = tcgetattr (STDIN_FILENO, &attr);
  if (status < 0)
    {
      return (-1);
    }
  // 
  attr.c_iflag |= (BRKINT | ICRNL);
  attr.c_iflag &= ~(IGNBRK | INLCR);
  // 
  attr.c_lflag |=
    (ECHO | ECHOE | ECHOK | ECHONL | ICANON | ISIG);
  attr.c_lflag &= ~(IEXTEN);
  // 
  status = tcsetattr (STDIN_FILENO, TCSANOW, &attr);
  // 
  return (status);
}

//----------------------------------------------------------
static int
terminal_noecho (struct termios *orig)
{
  int status;
  struct termios attr;
  // 
  // Save previous.
  // 
  status = tcgetattr (STDIN_FILENO, orig);
  if (status < 0)
    {
      return (-1);
    }
  // 
  // Get again.
  // 
  status = tcgetattr (STDIN_FILENO, &attr);
  if (status < 0)
    {
      return (-1);
    }
  // 
  attr.c_iflag |= (BRKINT | ICRNL);
  attr.c_iflag &= ~(IGNBRK | INLCR);
  // 
  attr.c_lflag |= (ICANON | ISIG);
  attr.c_lflag &= ~(ECHO | IEXTEN);
  // 
  status = tcsetattr (STDIN_FILENO, TCSANOW, &attr);
  // 
  return (status);
}

//----------------------------------------------------------
static int
terminal_restore (struct termios *orig)
{
  int status;
  // 
  // For an unknown reason, when running with Bochs,
  // before
  // restoring the termios configuration, the previous 
  // one
  // is to be read. Here, `attr' is just a placeholder 
  // and
  // the updated content is not used for anything
  // else.
  // 
  struct termios attr;
  status = tcgetattr (STDIN_FILENO, &attr);
  if (status < 0)
    {
      return (-1);
    }
  // 
  // 
  // 
  status = tcsetattr (STDIN_FILENO, TCSANOW, orig);
  // 
  return (status);
}
