#include <sys/os32.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>
#include <fcntl.h>
//----------------------------------------------------------
ssize_t
recvfrom (int sfdn, void *buffer, size_t count,
          int flags, struct sockaddr *addrfrom,
          socklen_t * addrlen)
{
  sysmsg_recvfrom_t msg;
  // 
  // Reduce size of read if necessary.
  // 
  if (count > BUFSIZ)
    {
      count = BUFSIZ;
    }
  // 
  // Fill the message.
  // 
  msg.sfdn = sfdn;
  msg.buffer = buffer;
  msg.count = count;
  msg.flags = flags;
  msg.addrfrom = addrfrom;
  msg.addrsize = addrlen;
  msg.fl_flags = 0;     // Not necessary.
  msg.ret = 0;
  // 
  // Repeat syscall, until something is received or
  // end of file is
  // reached.
  // 
  while (1)
    {
      sys (SYS_RECVFROM, &msg, (sizeof msg));
      if (msg.ret == 0)
        {
          // 
          // Stream closed from the other side.
          // 
          break;
        }
      if (msg.ret < 0
          && (msg.errno == EAGAIN
              || msg.errno == EWOULDBLOCK))
        {
          // 
          // No data at the moment.
          // 
          if (msg.fl_flags & O_NONBLOCK)
            {
              // 
              // Don't block.
              // 
              break;
            }
          else
            {
              // 
              // Keep trying.
              // 
              continue;
            }
        }
      // 
      // Otherwise, we have received something.
      // 
      break;
    }
  // 
  // 
  // 
  if (msg.ret < 0)
    {
      // 
      // No valid read.
      // 
      errno = msg.errno;
      errln = msg.errln;
      strncpy (errfn, msg.errfn, PATH_MAX);
      return (msg.ret);
    }
  // 
  if (msg.ret > count)
    {
      // 
      // A strange value was returned. Considering it
      // a read error.
      // 
      errset (EIO);     // I/O error.
      return (-1);
    }
  // 
  // A valid read: return.
  // 
  return (msg.ret);
}
