#ifndef _TERMIOS_H
#define _TERMIOS_H       1
//----------------------------------------------------------
#include <stdint.h>
//----------------------------------------------------------
typedef uint16_t tcflag_t;
typedef unsigned char cc_t;
//----------------------------------------------------------
#define NCCS    11      // `c_cc[]' size.
//
struct termios
{
  tcflag_t c_iflag;
  tcflag_t c_oflag;
  tcflag_t c_cflag;
  tcflag_t c_lflag;
  cc_t c_cc[NCCS];
};
//
// Subscript names for `c_cc[]' array, inside the
// `termios' structure:
//
#define VEOF      0     // EOF character
#define VEOL      1     // EOL character
#define VERASE    2     // ERASE character
#define VINTR     3     // INTR character
#define VKILL     4     // KILL character
#define VMIN      5     // MIN value
#define VQUIT     6     // QUIT character
#define VSTART    7     // START character
#define VSTOP     8     // STOP character
#define VSUSP     9     // SUSP character
#define VTIME    10     // TIME value
//
// Input modes, for `c_iflag' inside the `termios'
// structure:
//
#define BRKINT    1     // signal interrupt on break
#define ICRNL     2     // map CR to NL on input
#define IGNBRK    4     // ignore break condition
#define IGNCR     8     // ignore CR
#define IGNPAR   16     // ignore characters with
                        // parity errors
#define INLCR    32     // map NL to CR on input
#define INPCK    64     // enable input parity check
#define ISTRIP  128     // strip off eighth bit
#define IXOFF   256     // enable start/stop input
                        // control
#define IXON    512     // enable start/stop output
                        // control
#define PARMRK 1024     // mark parity errors
//
// Output modes, for `c_oflag' inside the `termios'
// structure:
//
#define OPOST     1     // post-process output
//
// Control modes, for `c_cflag' inside the `termios'
// structure:
//   not implemented.
//
//
// Local modes, for `c_lflag' inside the `termios'
// structure:
//
#define ECHO      1     // enable echo
#define ECHOE     2     // echo erase character as
                        // backspace
#define ECHOK     4     // echo KILL
#define ECHONL    8     // echo NL
#define ICANON   16     // canonical input mode
#define IEXTEN   32     // extended input mode
#define ISIG     64     // enable signals
#define NOFLSH  128     // disable flush after
                        // interrupt or quit
#define TOSTOP  256     // send SIGTTOU for background
                        // output
//----------------------------------------------------------
// Optional action for use with `tcsetattr()':
//
#define TCSANOW   1     // change attributes
                        // immediately
#define TCSADRAIN 2     // change attributes when
                        // output has drained
#define TCSAFLUSH 3     // change attributes when
                        // output has drained,
                        // and also flush pending
                        // input
//----------------------------------------------------------
int tcgetattr (int fdn, struct termios *termios_p);
int tcsetattr (int fdn, int action,
               struct termios *termios_p);
//----------------------------------------------------------
#endif
